"""
Batch command implementation for generating multiple conversations.
"""

import json
import os
import random
import sys
from datetime import datetime
from pathlib import Path
from typing import Dict, Any, List, Optional
from concurrent.futures import ThreadPoolExecutor, as_completed

from rich.console import Console
from rich.progress import Progress, SpinnerColumn, TextColumn, BarColumn, TaskProgressColumn
from rich.table import Table
from rich.panel import Panel

from ...core.conversation_engine import ConversationEngine
from ...models.conversation import Conversation
from ...utils.logging_utils import get_logger

console = Console()
logger = get_logger(__name__)


def batch_command(ctx: Dict[str, Any], **kwargs) -> None:
    """
    Generate multiple conversations in batch.
    Now supports multi-persona and all persona options, matching the generate command.
    """
    try:
        # Initialize conversation engine
        engine_config = {}
        if kwargs.get("seed"):
            engine_config["seed"] = kwargs["seed"]
        if kwargs.get("llm_backend"):
            engine_config["llm_backend"] = kwargs["llm_backend"]
        if kwargs.get("llm_model"):
            engine_config["llm_model"] = kwargs["llm_model"]
        if kwargs.get("llm_api_key"):
            engine_config["llm_api_key"] = kwargs["llm_api_key"]
        if kwargs.get("llm_api_url"):
            engine_config["llm_api_url"] = kwargs["llm_api_url"]
        engine = ConversationEngine(config=engine_config)

        # Multi-persona support
        multi_persona = kwargs.get("multi_persona", False)
        persona_names = None
        persona_roles = None
        if multi_persona:
            if kwargs.get("persona_names"):
                persona_names = [name.strip() for name in kwargs["persona_names"].split(",")]
                console.print(f"[green]Multi-persona batch with: {', '.join(persona_names)}[/green]")
            if kwargs.get("persona_roles"):
                roles_list = [role.strip() for role in kwargs["persona_roles"].split(",")]
                persona_roles = dict(zip(persona_names or [], roles_list))
                console.print(f"[green]Persona roles: {persona_roles}[/green]")

        # Handle dynamic persona creation (single persona mode)
        persona_name = kwargs.get("personas")
        if kwargs.get("create_persona") and not multi_persona:
            persona = engine.persona_manager.create_dynamic_persona(
                name=kwargs.get("persona_name", "Dynamic Persona"),
                role=kwargs.get("persona_role", "Professional"),
                domain=kwargs.get("persona_domain", "technology"),
                expertise_level=kwargs.get("persona_expertise", "intermediate"),
                background=kwargs.get("persona_background", ""),
                communication_style=kwargs.get("persona_communication_style", "formal"),
                experience_level=kwargs.get("persona_experience") or 5,
                formality_level=kwargs.get("persona_formality") or 3
            )
            persona_name = persona.name
            console.print(f"[green]Created dynamic persona for batch: {persona.name} ({persona.role})[/green]")
        elif kwargs.get("persona_template") and not multi_persona:
            persona = engine.persona_manager.create_persona_from_template(
                template_name=kwargs.get("persona_template"),
                name=kwargs.get("persona_name", "Template Persona"),
                role=kwargs.get("persona_role", "Professional"),
                domain=kwargs.get("persona_domain"),
                expertise_level=kwargs.get("persona_expertise"),
                background=kwargs.get("persona_background"),
                communication_style=kwargs.get("persona_communication_style")
            )
            persona_name = persona.name
            console.print(f"[green]Created persona from template for batch: {persona.name} ({persona.role})[/green]")

        # Get generation parameters
        num_conversations = kwargs.get("conversations", 1)
        turns_per_conversation = kwargs.get("turns", 5)
        language = kwargs.get("language", "en")
        domain_focus = kwargs.get("domain")
        complexity = kwargs.get("complexity")
        output_format = kwargs.get("output", "json")
        output_dir = kwargs.get("output_dir", "./output")
        output_filename = kwargs.get("output_filename")
        parallel = kwargs.get("parallel", False)
        workers = kwargs.get("workers", 4)
        show_progress = kwargs.get("progress", False)
        context = kwargs.get("context")

        # Create output directory
        output_path = Path(output_dir)
        output_path.mkdir(parents=True, exist_ok=True)

        # Generate conversations
        conversations = []
        if parallel and num_conversations > 1:
            conversations = generate_conversations_parallel_full(
                engine, persona_name, persona_names, multi_persona, persona_roles, num_conversations, turns_per_conversation,
                language, domain_focus, complexity, context, workers, show_progress, kwargs
            )
        else:
            conversations = generate_conversations_sequential_full(
                engine, persona_name, persona_names, multi_persona, persona_roles, num_conversations, turns_per_conversation,
                language, domain_focus, complexity, context, show_progress, kwargs
            )

        # Save results
        save_batch_results(conversations, output_path, output_format, show_progress, output_filename)
        display_batch_summary(conversations, output_path, output_format)
        engine.cleanup()
    except Exception as e:
        logger.error(f"Error in batch generation: {e}")
        console.print(f"[red]Error: {e}[/red]")
        sys.exit(1)


def generate_conversations_sequential(
    engine: ConversationEngine,
    persona_name: Optional[str],
    num_conversations: int,
    turns_per_conversation: int,
    language: str,
    domain_focus: Optional[str],
    complexity: Optional[str],
    show_progress: bool
) -> List[Conversation]:
    """Generate conversations sequentially."""
    conversations = []
    
    if show_progress:
        with Progress(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            BarColumn(),
            TaskProgressColumn(),
            console=console
        ) as progress:
            task = progress.add_task("Generating conversations...", total=num_conversations)
            
            for i in range(num_conversations):
                try:
                    conversation = engine.generate_conversation(
                        persona_name=persona_name,
                        turns=turns_per_conversation,
                        language=language,
                        domain_focus=domain_focus,
                        complexity=complexity
                    )
                    conversations.append(conversation)
                    progress.update(task, advance=1)
                    
                except Exception as e:
                    logger.error(f"Error generating conversation {i+1}: {e}")
                    console.print(f"[red]Error in conversation {i+1}: {e}[/red]")
    else:
        for i in range(num_conversations):
            try:
                conversation = engine.generate_conversation(
                    persona_name=persona_name,
                    turns=turns_per_conversation,
                    language=language,
                    domain_focus=domain_focus,
                    complexity=complexity
                )
                conversations.append(conversation)
                
            except Exception as e:
                logger.error(f"Error generating conversation {i+1}: {e}")
                console.print(f"[red]Error in conversation {i+1}: {e}[/red]")
    
    return conversations


def generate_conversations_parallel(
    engine: ConversationEngine,
    persona_name: Optional[str],
    num_conversations: int,
    turns_per_conversation: int,
    language: str,
    domain_focus: Optional[str],
    complexity: Optional[str],
    workers: int,
    show_progress: bool
) -> List[Conversation]:
    """Generate conversations in parallel."""
    conversations = []
    
    def generate_single_conversation(i: int) -> Optional[Conversation]:
        try:
            # Create a new engine instance for each thread
            engine_config = {}
            if hasattr(engine, 'llm_backend'):
                engine_config["llm_backend"] = engine.llm_backend
            if hasattr(engine, 'llm_model'):
                engine_config["llm_model"] = engine.llm_model
            if hasattr(engine, 'llm_api_key'):
                engine_config["llm_api_key"] = engine.llm_api_key
            if hasattr(engine, 'llm_api_url'):
                engine_config["llm_api_url"] = engine.llm_api_url
            
            thread_engine = ConversationEngine(config=engine_config)
            
            # If we have a dynamic persona, recreate it in this thread
            thread_persona_name = persona_name
            if persona_name and persona_name not in [p.name for p in thread_engine.persona_manager.list_personas()]:
                # Try to find the persona by name in the original engine
                original_persona = engine.persona_manager.get_persona(persona_name)
                if original_persona:
                    # Recreate the persona in this thread
                    thread_engine.persona_manager.add_persona(original_persona)
            
            conversation = thread_engine.generate_conversation(
                persona_name=thread_persona_name,
                turns=turns_per_conversation,
                language=language,
                domain_focus=domain_focus,
                complexity=complexity
            )
            
            thread_engine.cleanup()
            return conversation
            
        except Exception as e:
            logger.error(f"Error generating conversation {i+1}: {e}")
            return None
    
    if show_progress:
        with Progress(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            BarColumn(),
            TaskProgressColumn(),
            console=console
        ) as progress:
            task = progress.add_task("Generating conversations in parallel...", total=num_conversations)
            
            with ThreadPoolExecutor(max_workers=workers) as executor:
                future_to_index = {
                    executor.submit(generate_single_conversation, i): i 
                    for i in range(num_conversations)
                }
                
                for future in as_completed(future_to_index):
                    conversation = future.result()
                    if conversation:
                        conversations.append(conversation)
                    progress.update(task, advance=1)
    else:
        with ThreadPoolExecutor(max_workers=workers) as executor:
            future_to_index = {
                executor.submit(generate_single_conversation, i): i 
                for i in range(num_conversations)
            }
            
            for future in as_completed(future_to_index):
                conversation = future.result()
                if conversation:
                    conversations.append(conversation)
    
    return conversations


def save_batch_results(
    conversations: List[Conversation],
    output_path: Path,
    output_format: str,
    show_progress: bool,
    custom_filename: Optional[str] = None
) -> None:
    """Save batch results in the specified format."""
    
    timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
    
    # Use custom filename if provided, otherwise use default
    base_filename = custom_filename if custom_filename else f"batch_conversations_{timestamp}"
    
    # Map output format to correct file extension
    extension_map = {
        "json": "json",
        "text": "txt",
        "markdown": "md",
        "csv": "csv"
    }
    extension = extension_map.get(output_format, output_format)
    
    if output_format == "json":
        # Save as single JSON file with all conversations
        json_file = output_path / f"{base_filename}.{extension}"
        
        batch_data = {
            "metadata": {
                "generated_at": datetime.now().isoformat(),
                "total_conversations": len(conversations),
                "format": "batch_json"
            },
            "conversations": [conv.to_dict() for conv in conversations]
        }
        
        with open(json_file, 'w', encoding='utf-8') as f:
            json.dump(batch_data, f, indent=2, ensure_ascii=False, default=str)
        
        if show_progress:
            console.print(f"[green]Saved batch results to: {json_file}[/green]")
    
    elif output_format == "csv":
        # Save as CSV file
        csv_file = output_path / f"{base_filename}.{extension}"
        
        # Create CSV header
        csv_lines = [
            "conversation_id,persona_name,language,turns,quality_score,domain_focus,created_at"
        ]
        
        for conv in conversations:
            csv_lines.append(
                f"{conv.id},{conv.primary_persona or 'N/A'},{conv.language},"
                f"{len(conv.turns)},{conv.overall_quality_score or 'N/A'},"
                f"{conv.domain_focus or 'N/A'},{conv.created_at.isoformat()}"
            )
        
        with open(csv_file, 'w', encoding='utf-8') as f:
            f.write('\n'.join(csv_lines))
        
        if show_progress:
            console.print(f"[green]Saved batch results to: {csv_file}[/green]")
    
    elif output_format == "markdown":
        # Save as Markdown file
        md_file = output_path / f"{base_filename}.{extension}"
        
        md_lines = [
            f"# {base_filename.replace('_', ' ').title()} - {timestamp}",
            "",
            f"**Total Conversations:** {len(conversations)}",
            f"**Generated:** {datetime.now().isoformat()}",
            "",
            "---",
            ""
        ]
        
        for i, conv in enumerate(conversations, 1):
            md_lines.extend([
                f"## Conversation {i}",
                "",
                f"**ID:** {conv.id}",
                f"**Persona:** {conv.primary_persona or 'N/A'}",
                f"**Language:** {conv.language}",
                f"**Turns:** {len(conv.turns)}",
                f"**Quality Score:** {conv.overall_quality_score:.2f}" if conv.overall_quality_score else "**Quality Score:** N/A",
                "",
                "### Turns:",
                ""
            ])
            
            for j, turn in enumerate(conv.turns, 1):
                md_lines.extend([
                    f"#### Turn {j}",
                    "",
                    f"**User:** {turn.user_message.content}",
                    ""
                ])
                
                if turn.assistant_message:
                    md_lines.extend([
                        f"**Assistant:** {turn.assistant_message.content}",
                        ""
                    ])
                
                md_lines.append("---")
                md_lines.append("")
        
        with open(md_file, 'w', encoding='utf-8') as f:
            f.write('\n'.join(md_lines))
        
        if show_progress:
            console.print(f"[green]Saved batch results to: {md_file}[/green]")


def display_batch_summary(
    conversations: List[Conversation],
    output_path: Path,
    output_format: str
) -> None:
    """Display a summary of the batch generation results."""
    
    if not conversations:
        console.print("[red]No conversations were generated successfully.[/red]")
        return
    
    # Create summary table
    table = Table(title="Batch Generation Summary")
    table.add_column("Metric", style="cyan", no_wrap=True)
    table.add_column("Value", style="magenta")
    
    # Calculate statistics
    total_turns = sum(len(conv.turns) for conv in conversations)
    avg_quality = sum(conv.overall_quality_score or 0 for conv in conversations) / len(conversations)
    languages = list(set(conv.language for conv in conversations))
    domains = list(set(conv.domain_focus for conv in conversations if conv.domain_focus))
    
    table.add_row("Total Conversations", str(len(conversations)))
    table.add_row("Total Turns", str(total_turns))
    table.add_row("Average Quality Score", f"{avg_quality:.2f}")
    table.add_row("Languages", ", ".join(languages))
    table.add_row("Domains", ", ".join(domains) if domains else "N/A")
    table.add_row("Output Format", output_format)
    table.add_row("Output Directory", str(output_path))
    
    console.print(table)
    
    # Show quality distribution
    quality_scores = [conv.overall_quality_score for conv in conversations if conv.overall_quality_score]
    if quality_scores:
        console.print(f"\n[bold]Quality Score Distribution:[/bold]")
        console.print(f"  Min: {min(quality_scores):.2f}")
        console.print(f"  Max: {max(quality_scores):.2f}")
        console.print(f"  Average: {sum(quality_scores) / len(quality_scores):.2f}")
    
    # Show conversation preview
    if len(conversations) <= 3:
        console.print(f"\n[bold]All Conversations Generated:[/bold]")
        for i, conv in enumerate(conversations, 1):
            console.print(f"  {i}. {conv.id} ({len(conv.turns)} turns, quality: {conv.overall_quality_score:.2f})")
    else:
        console.print(f"\n[bold]Sample Conversations Generated:[/bold]")
        for i, conv in enumerate(conversations[:3], 1):
            console.print(f"  {i}. {conv.id} ({len(conv.turns)} turns, quality: {conv.overall_quality_score:.2f})")
        console.print(f"  ... and {len(conversations) - 3} more") 


# New: Full-featured sequential batch generation

def generate_conversations_sequential_full(
    engine: ConversationEngine,
    persona_name: Optional[str],
    persona_names: Optional[list],
    multi_persona: bool,
    persona_roles: Optional[dict],
    num_conversations: int,
    turns_per_conversation: int,
    language: str,
    domain_focus: Optional[str],
    complexity: Optional[str],
    context: Optional[str],
    show_progress: bool,
    kwargs: dict
) -> List[Conversation]:
    conversations = []
    # Remove keys that are passed explicitly
    for k in ["turns", "language", "context", "persona_name", "persona_names", "multi_persona", "persona_roles"]:
        kwargs.pop(k, None)
    if show_progress:
        with Progress(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            BarColumn(),
            TaskProgressColumn(),
            console=console
        ) as progress:
            task = progress.add_task("Generating conversations...", total=num_conversations)
            for i in range(num_conversations):
                try:
                    conversation = engine.generate_conversation(
                        persona_name=persona_name if not multi_persona else None,
                        persona_names=persona_names if multi_persona else None,
                        turns=turns_per_conversation,
                        language=language,
                        context=context,
                        complexity=complexity,
                        domain_focus=domain_focus,
                        multi_persona=multi_persona,
                        persona_roles=persona_roles,
                        **kwargs
                    )
                    conversations.append(conversation)
                    progress.update(task, advance=1)
                except Exception as e:
                    logger.error(f"Error generating conversation {i+1}: {e}")
                    console.print(f"[red]Error in conversation {i+1}: {e}[/red]")
    else:
        for i in range(num_conversations):
            try:
                conversation = engine.generate_conversation(
                    persona_name=persona_name if not multi_persona else None,
                    persona_names=persona_names if multi_persona else None,
                    turns=turns_per_conversation,
                    language=language,
                    context=context,
                    complexity=complexity,
                    domain_focus=domain_focus,
                    multi_persona=multi_persona,
                    persona_roles=persona_roles,
                    **kwargs
                )
                conversations.append(conversation)
            except Exception as e:
                logger.error(f"Error generating conversation {i+1}: {e}")
                console.print(f"[red]Error in conversation {i+1}: {e}[/red]")
    return conversations

# New: Full-featured parallel batch generation

def generate_conversations_parallel_full(
    engine: ConversationEngine,
    persona_name: Optional[str],
    persona_names: Optional[list],
    multi_persona: bool,
    persona_roles: Optional[dict],
    num_conversations: int,
    turns_per_conversation: int,
    language: str,
    domain_focus: Optional[str],
    complexity: Optional[str],
    context: Optional[str],
    workers: int,
    show_progress: bool,
    kwargs: dict
) -> List[Conversation]:
    conversations = []
    # Remove keys that are passed explicitly
    for k in ["turns", "language", "context", "persona_name", "persona_names", "multi_persona", "persona_roles"]:
        kwargs.pop(k, None)
    def generate_single_conversation(i: int) -> Optional[Conversation]:
        try:
            engine_config = {}
            if hasattr(engine, 'llm_backend'):
                engine_config["llm_backend"] = engine.llm_backend
            if hasattr(engine, 'llm_model'):
                engine_config["llm_model"] = engine.llm_model
            if hasattr(engine, 'llm_api_key'):
                engine_config["llm_api_key"] = engine.llm_api_key
            if hasattr(engine, 'llm_api_url'):
                engine_config["llm_api_url"] = engine.llm_api_url
            thread_engine = ConversationEngine(config=engine_config)
            thread_persona_name = persona_name
            if persona_name and persona_name not in [p.name for p in thread_engine.persona_manager.list_personas()]:
                original_persona = engine.persona_manager.get_persona(persona_name)
                if original_persona:
                    thread_engine.persona_manager.add_persona(original_persona)
            conversation = thread_engine.generate_conversation(
                persona_name=thread_persona_name if not multi_persona else None,
                persona_names=persona_names if multi_persona else None,
                turns=turns_per_conversation,
                language=language,
                context=context,
                complexity=complexity,
                domain_focus=domain_focus,
                multi_persona=multi_persona,
                persona_roles=persona_roles,
                **kwargs
            )
            thread_engine.cleanup()
            return conversation
        except Exception as e:
            logger.error(f"Error generating conversation {i+1}: {e}")
            return None
    if show_progress:
        with Progress(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            BarColumn(),
            TaskProgressColumn(),
            console=console
        ) as progress:
            task = progress.add_task("Generating conversations in parallel...", total=num_conversations)
            with ThreadPoolExecutor(max_workers=workers) as executor:
                future_to_index = {
                    executor.submit(generate_single_conversation, i): i 
                    for i in range(num_conversations)
                }
                for future in as_completed(future_to_index):
                    conversation = future.result()
                    if conversation:
                        conversations.append(conversation)
                    progress.update(task, advance=1)
    else:
        with ThreadPoolExecutor(max_workers=workers) as executor:
            future_to_index = {
                executor.submit(generate_single_conversation, i): i 
                for i in range(num_conversations)
            }
            for future in as_completed(future_to_index):
                conversation = future.result()
                if conversation:
                    conversations.append(conversation)
    return conversations 