"""
Generate command implementation for single conversation generation.
"""

import json
import sys
from typing import Dict, Any, Optional
from rich.console import Console
from rich.panel import Panel
from rich.table import Table
from rich.progress import Progress, SpinnerColumn, TextColumn

from ...core.conversation_engine import ConversationEngine
from ...models.persona import Persona
from ...utils.logging_utils import get_logger

console = Console()
logger = get_logger(__name__)


def generate_command(ctx: Dict[str, Any], **kwargs) -> None:
    """
    Generate a single conversation with specified parameters.
    
    Args:
        ctx: Click context object
        **kwargs: Command line arguments
    """
    
    try:
        # Initialize conversation engine
        engine_config = {}
        if kwargs.get("seed"):
            engine_config["seed"] = kwargs["seed"]
        
        # Add LLM backend configuration
        if kwargs.get("llm_backend"):
            engine_config["llm_backend"] = kwargs["llm_backend"]
        if kwargs.get("llm_model"):
            engine_config["llm_model"] = kwargs["llm_model"]
        if kwargs.get("llm_api_key"):
            engine_config["llm_api_key"] = kwargs["llm_api_key"]
        if kwargs.get("llm_api_url"):
            engine_config["llm_api_url"] = kwargs["llm_api_url"]
        
        engine = ConversationEngine(config=engine_config)
        
        # Handle multi-persona or single persona setup
        multi_persona = kwargs.get("multi_persona", False)
        persona_names = None
        persona_roles = None
        
        if multi_persona:
            # Multi-persona conversation
            if kwargs.get("persona_names"):
                persona_names = [name.strip() for name in kwargs["persona_names"].split(",")]
                console.print(f"[green]Multi-persona conversation with: {', '.join(persona_names)}[/green]")
            
            if kwargs.get("persona_roles"):
                roles_list = [role.strip() for role in kwargs["persona_roles"].split(",")]
                persona_roles = dict(zip(persona_names or [], roles_list))
                console.print(f"[green]Persona roles: {persona_roles}[/green]")
        else:
            # Single persona conversation
            persona_name = kwargs.get("persona")
            if kwargs.get("create_persona"):
                # Create dynamic persona
                persona = engine.persona_manager.create_dynamic_persona(
                    name=kwargs.get("persona_name", "Dynamic Persona"),
                    role=kwargs.get("persona_role", "Professional"),
                    domain=kwargs.get("persona_domain", "technology"),
                    expertise_level=kwargs.get("persona_expertise", "intermediate"),
                    background=kwargs.get("persona_background", ""),
                    communication_style=kwargs.get("persona_communication_style", "formal"),
                    experience_level=kwargs.get("persona_experience") or 5,
                    formality_level=kwargs.get("persona_formality") or 3
                )
                persona_name = persona.name
                console.print(f"[green]Created dynamic persona: {persona.name} ({persona.role})[/green]")
            
            elif kwargs.get("persona_template"):
                # Create persona from template
                persona = engine.persona_manager.create_persona_from_template(
                    template_name=kwargs.get("persona_template"),
                    name=kwargs.get("persona_name", "Template Persona"),
                    role=kwargs.get("persona_role", "Professional"),
                    domain=kwargs.get("persona_domain"),
                    expertise_level=kwargs.get("persona_expertise"),
                    background=kwargs.get("persona_background"),
                    communication_style=kwargs.get("persona_communication_style")
                )
                persona_name = persona.name
                console.print(f"[green]Created persona from template: {persona.name} ({persona.role})[/green]")
        
        # Show generation progress
        with Progress(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            console=console
        ) as progress:
            
            task = progress.add_task("Generating conversation...", total=None)
            
            # Generate conversation
            conversation = engine.generate_conversation(
                persona_name=persona_name if not multi_persona else None,
                persona_names=persona_names if multi_persona else None,
                turns=kwargs.get("turns", 3),
                language=kwargs.get("language", "en"),
                context=kwargs.get("context"),
                complexity=kwargs.get("complexity"),
                domain_focus=kwargs.get("domain"),
                multi_persona=multi_persona,
                persona_roles=persona_roles
            )
            
            progress.update(task, description="Conversation generated successfully!")
        
        # Display conversation summary
        display_conversation_summary(conversation, ctx)
        
        # Output conversation
        output_file = kwargs.get("output_file")
        output_filename = kwargs.get("output_filename")
        output_format = kwargs.get("output", "json")
        
        # If custom filename is provided, construct the full path
        if output_filename and not output_file:
            # Map output format to correct file extension
            extension_map = {
                "json": "json",
                "text": "txt",
                "markdown": "md",
                "csv": "csv"
            }
            extension = extension_map.get(output_format, output_format)
            output_file = f"{output_filename}.{extension}"
        # If no filename is provided but we want to save to file, generate timestamped filename
        elif not output_file and not output_filename:
            from datetime import datetime
            timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
            # Map output format to correct file extension
            extension_map = {
                "json": "json",
                "text": "txt",
                "markdown": "md",
                "csv": "csv"
            }
            extension = extension_map.get(output_format, output_format)
            output_file = f"conversation_{timestamp}.{extension}"
        
        output_conversation(conversation, output_format, output_file)
        
        # Cleanup
        engine.cleanup()
        
    except Exception as e:
        logger.error(f"Error generating conversation: {e}")
        console.print(f"[red]Error: {e}[/red]")
        sys.exit(1)


def display_conversation_summary(conversation, ctx) -> None:
    """Display a summary of the generated conversation."""
    
    if hasattr(ctx, 'obj') and ctx.obj.get("quiet"):
        return
    
    # Create summary table
    table = Table(title="Conversation Summary")
    table.add_column("Property", style="cyan", no_wrap=True)
    table.add_column("Value", style="magenta")
    
    table.add_row("Conversation ID", conversation.id)
    table.add_row("Primary Persona", conversation.primary_persona or "N/A")
    
    if conversation.is_multi_persona():
        table.add_row("Type", "Multi-Persona")
        table.add_row("Active Personas", ", ".join(conversation.get_active_personas()))
        table.add_row("Total Personas", str(len(conversation.personas)))
    else:
        table.add_row("Type", "Single Persona")
    
    table.add_row("Language", conversation.language)
    table.add_row("Turns Generated", str(len(conversation.turns)))
    table.add_row("State", str(conversation.state))
    table.add_row("Quality Score", f"{conversation.overall_quality_score:.2f}" if conversation.overall_quality_score else "N/A")
    table.add_row("Domain Focus", conversation.domain_focus or "N/A")
    table.add_row("Target Complexity", conversation.target_complexity)
    
    console.print(table)
    
    # Show conversation preview
    if conversation.turns:
        console.print("\n[bold]Conversation Preview:[/bold]")
        
        for i, turn in enumerate(conversation.turns[:3], 1):  # Show first 3 turns
            console.print(f"\n[bold cyan]Turn {i}:[/bold cyan]")
            
            # User message
            user_content = turn.user_message.content[:100] + "..." if len(turn.user_message.content) > 100 else turn.user_message.content
            if conversation.is_multi_persona() and turn.user_message.persona_name:
                console.print(f"[green]{turn.user_message.persona_name}:[/green] {user_content}")
            else:
                console.print(f"[green]User:[/green] {user_content}")
            
            # Assistant message
            if turn.assistant_message:
                assistant_content = turn.assistant_message.content[:100] + "..." if len(turn.assistant_message.content) > 100 else turn.assistant_message.content
                console.print(f"[blue]Assistant:[/blue] {assistant_content}")
            
            if i >= 3:
                console.print("[dim]... (showing first 3 turns)[/dim]")
                break


def output_conversation(conversation, output_format: str, output_file: Optional[str]) -> None:
    """Output conversation in the specified format."""
    
    if output_format == "json":
        output_json(conversation, output_file)
    elif output_format == "text":
        output_text(conversation, output_file)
    elif output_format == "markdown":
        output_markdown(conversation, output_file)
    elif output_format == "csv":
        output_csv(conversation, output_file)
    else:
        raise ValueError(f"Unsupported output format: {output_format}")


def output_json(conversation, output_file: Optional[str]) -> None:
    """Output conversation as JSON."""
    
    output_data = conversation.to_dict()
    
    if output_file:
        with open(output_file, 'w', encoding='utf-8') as f:
            json.dump(output_data, f, indent=2, ensure_ascii=False, default=str)
        console.print(f"[green]Conversation saved to: {output_file}[/green]")
    else:
        console.print(json.dumps(output_data, indent=2, ensure_ascii=False, default=str))


def output_text(conversation, output_file: Optional[str]) -> None:
    """Output conversation as plain text."""
    
    lines = []
    lines.append(f"Conversation ID: {conversation.id}")
    lines.append(f"Language: {conversation.language}")
    lines.append(f"Turns: {len(conversation.turns)}")
    lines.append("=" * 50)
    lines.append("")
    
    for i, turn in enumerate(conversation.turns, 1):
        lines.append(f"Turn {i}:")
        
        # Show persona name for multi-persona conversations
        if conversation.is_multi_persona() and turn.user_message.persona_name:
            lines.append(f"{turn.user_message.persona_name}: {turn.user_message.content}")
        else:
            lines.append(f"User: {turn.user_message.content}")
        lines.append("")
        
        if turn.assistant_message:
            lines.append(f"Assistant: {turn.assistant_message.content}")
            lines.append("")
        
        lines.append("-" * 30)
        lines.append("")
    
    output_content = "\n".join(lines)
    
    if output_file:
        with open(output_file, 'w', encoding='utf-8') as f:
            f.write(output_content)
        console.print(f"[green]Conversation saved to: {output_file}[/green]")
    else:
        console.print(output_content)


def output_markdown(conversation, output_file: Optional[str]) -> None:
    """Output conversation as Markdown."""
    
    lines = []
    lines.append(f"# Conversation: {conversation.id}")
    lines.append("")
    lines.append(f"**Language:** {conversation.language}")
    lines.append(f"**Turns:** {len(conversation.turns)}")
    lines.append(f"**Quality Score:** {conversation.overall_quality_score:.2f}" if conversation.overall_quality_score else "**Quality Score:** N/A")
    lines.append("")
    lines.append("---")
    lines.append("")
    
    for i, turn in enumerate(conversation.turns, 1):
        lines.append(f"## Turn {i}")
        lines.append("")
        
        # Show persona name for multi-persona conversations
        if conversation.is_multi_persona() and turn.user_message.persona_name:
            lines.append(f"**{turn.user_message.persona_name}:** {turn.user_message.content}")
        else:
            lines.append(f"**User:** {turn.user_message.content}")
        lines.append("")
        
        if turn.assistant_message:
            lines.append(f"**Assistant:** {turn.assistant_message.content}")
            lines.append("")
        
        lines.append("---")
        lines.append("")
    
    output_content = "\n".join(lines)
    
    if output_file:
        with open(output_file, 'w', encoding='utf-8') as f:
            f.write(output_content)
        console.print(f"[green]Conversation saved to: {output_file}[/green]")
    else:
        console.print(output_content)


def output_csv(conversation, output_file: Optional[str]) -> None:
    """Output conversation as CSV."""
    
    import csv
    
    lines = []
    lines.append("turn_number,role,content,message_type,timestamp")
    
    for turn in conversation.turns:
        # User message
        user_content = turn.user_message.content.replace('"', '""')
        lines.append(f"{turn.turn_number},user,\"{user_content}\",{str(turn.user_message.message_type)},{turn.user_message.timestamp}")
        
        # Assistant message
        if turn.assistant_message:
            assistant_content = turn.assistant_message.content.replace('"', '""')
            lines.append(f"{turn.turn_number},assistant,\"{assistant_content}\",{str(turn.assistant_message.message_type)},{turn.assistant_message.timestamp}")
    
    output_content = "\n".join(lines)
    
    if output_file:
        with open(output_file, 'w', encoding='utf-8', newline='') as f:
            f.write(output_content)
        console.print(f"[green]Conversation saved to: {output_file}[/green]")
    else:
        console.print(output_content) 