"""
List command implementation for listing available resources.
"""

from typing import Dict, Any
from rich.console import Console
from rich.table import Table

console = Console()


def list_command(ctx: Dict[str, Any], **kwargs) -> None:
    """
    List available personas, languages, and other resources.
    
    Args:
        ctx: Click context object
        **kwargs: Command line arguments
    """
    if kwargs.get("personas"):
        list_personas(ctx, **kwargs)
    elif kwargs.get("languages"):
        list_languages(ctx, **kwargs)
    elif kwargs.get("domains"):
        list_domains(ctx, **kwargs)
    elif kwargs.get("expertise_levels"):
        list_expertise_levels(ctx, **kwargs)
    else:
        # Default: list personas
        list_personas(ctx, **kwargs)


def list_personas(ctx: Dict[str, Any], **kwargs) -> None:
    """List available personas."""
    console.print("[yellow]Persona listing not yet fully implemented[/yellow]")
    console.print("Personas can be created dynamically using --create-persona option.")
    
    # Show sample personas
    table = Table(title="Sample Dynamic Personas")
    table.add_column("Name", style="cyan")
    table.add_column("Role", style="magenta")
    table.add_column("Domain", style="green")
    table.add_column("Expertise", style="yellow")
    
    table.add_row("مطور برمجيات", "مطور برمجيات كبير", "Technology", "Expert")
    table.add_row("خباز", "خباز محترف", "Creative", "Expert")
    table.add_row("مدير مشاريع", "مدير مشاريع تقنية", "Business", "Advanced")
    
    console.print(table)


def list_languages(ctx: Dict[str, Any], **kwargs) -> None:
    """List supported languages."""
    table = Table(title="Supported Languages")
    table.add_column("Code", style="cyan")
    table.add_column("Language", style="magenta")
    table.add_column("Status", style="green")
    
    languages = [
        ("en", "English", "Full Support"),
        ("es", "Spanish", "Full Support"),
        ("fr", "French", "Full Support"),
        ("de", "German", "Full Support"),
        ("it", "Italian", "Full Support"),
        ("pt", "Portuguese", "Full Support"),
        ("ja", "Japanese", "Core Support"),
        ("zh", "Chinese", "Core Support"),
        ("ko", "Korean", "Core Support"),
        ("ar", "Arabic", "Core Support"),
    ]
    
    for code, name, status in languages:
        table.add_row(code, name, status)
    
    console.print(table)


def list_domains(ctx: Dict[str, Any], **kwargs) -> None:
    """List available domains."""
    table = Table(title="Available Domains")
    table.add_column("Domain", style="cyan")
    table.add_column("Description", style="magenta")
    table.add_column("Personas", style="green")
    
    domains = [
        ("Technology", "Software development, IT, and technical topics", "12 personas"),
        ("Healthcare", "Medical, nursing, and healthcare administration", "8 personas"),
        ("Education", "Teaching, learning, and educational administration", "8 personas"),
        ("Business", "Management, marketing, finance, and operations", "10 personas"),
        ("Creative", "Arts, design, writing, and creative professions", "8 personas"),
        ("Specialized", "Legal, research, government, and specialized fields", "6 personas"),
    ]
    
    for domain, description, count in domains:
        table.add_row(domain, description, count)
    
    console.print(table)


def list_expertise_levels(ctx: Dict[str, Any], **kwargs) -> None:
    """List expertise levels."""
    table = Table(title="Expertise Levels")
    table.add_column("Level", style="cyan")
    table.add_column("Description", style="magenta")
    table.add_column("Characteristics", style="green")
    
    levels = [
        ("Novice", "Beginner level", "Basic questions, learning-focused"),
        ("Intermediate", "Some experience", "Clarification requests, practical questions"),
        ("Advanced", "Significant experience", "Deep technical questions, optimization focus"),
        ("Expert", "High level expertise", "Challenging questions, best practices focus"),
    ]
    
    for level, description, characteristics in levels:
        table.add_row(level, description, characteristics)
    
    console.print(table) 