#!/usr/bin/env python3
"""
Main CLI entry point for persona-chat application.
"""

import sys
import click
from rich.console import Console
from rich.panel import Panel
from rich.text import Text
from rich import print as rprint

from .commands.generate import generate_command
from .commands.batch import batch_command
from .commands.continue_cmd import continue_command
from .commands.analyze import analyze_command
from .commands.create import create_command
from .commands.import_cmd import import_command
from .commands.export import export_command
from .commands.list_cmd import list_command
from .commands.translate import translate_command

console = Console()


@click.group()
@click.version_option(version="0.1.1", prog_name="persona-chat")
@click.option(
    "--verbose", "-v", 
    is_flag=True, 
    help="Enable verbose output"
)
@click.option(
    "--quiet", "-q", 
    is_flag=True, 
    help="Suppress output except errors"
)
@click.option(
    "--config", "-c", 
    type=click.Path(exists=True), 
    help="Path to configuration file"
)
@click.pass_context
def main(ctx, verbose, quiet, config):
    """
    Persona-Based Conversation Generator CLI
    
    Generate authentic multi-turn conversations using 50+ distinct user personas
    across multiple languages with cultural adaptation and quality analysis.
    """
    # Ensure context object exists
    ctx.ensure_object(dict)
    
    # Store options in context
    ctx.obj["verbose"] = verbose
    ctx.obj["quiet"] = quiet
    ctx.obj["config"] = config
    
    # Set up logging based on verbosity
    if verbose:
        console.print("[bold blue]Verbose mode enabled[/bold blue]")
    if quiet:
        console.print("[bold yellow]Quiet mode enabled[/bold yellow]")


@main.command()
@click.option(
    "--persona", "-p",
    help="Specific persona name or ID to use"
)
@click.option(
    "--custom-persona", 
    type=click.Path(exists=True),
    help="Path to custom persona definition file"
)
@click.option(
    "--persona-inline",
    help="Inline JSON persona definition"
)
@click.option(
    "--create-persona",
    is_flag=True,
    help="Create a dynamic persona on-the-fly"
)
@click.option(
    "--persona-name",
    help="Name for the dynamic persona (used with --create-persona)"
)
@click.option(
    "--persona-role",
    help="Role for the dynamic persona (used with --create-persona)"
)
@click.option(
    "--persona-domain",
    type=click.Choice(["technology", "healthcare", "education", "business", "creative", "specialized"]),
    help="Domain for the dynamic persona (used with --create-persona)"
)
@click.option(
    "--persona-expertise",
    type=click.Choice(["novice", "intermediate", "advanced", "expert"]),
    help="Expertise level for the dynamic persona (used with --create-persona)"
)
@click.option(
    "--persona-background",
    help="Background description for the dynamic persona (used with --create-persona)"
)
@click.option(
    "--persona-communication-style",
    type=click.Choice(["formal", "casual", "technical", "friendly", "business", "academic"]),
    help="Communication style for the dynamic persona (used with --create-persona)"
)
@click.option(
    "--persona-experience",
    type=int,
    help="Years of experience for the dynamic persona (used with --create-persona)"
)
@click.option(
    "--persona-formality",
    type=int,
    help="Formality level (1-5) for the dynamic persona (used with --create-persona)"
)
@click.option(
    "--persona-template",
    type=click.Choice(["senior_developer", "junior_developer", "data_scientist", "product_manager", "designer"]),
    help="Use a predefined persona template"
)
@click.option(
    "--multi-persona",
    is_flag=True,
    help="Enable multi-persona conversation with multiple participants"
)
@click.option(
    "--persona-names",
    help="Comma-separated list of persona names for multi-persona conversation"
)
@click.option(
    "--persona-roles",
    help="Comma-separated list of roles for personas (e.g., 'primary,participant,observer')"
)
@click.option(
    "--domain", "-d",
    type=click.Choice(["technology", "healthcare", "education", "business", "creative", "specialized"]),
    help="Filter personas by domain"
)
@click.option(
    "--expertise", "-e",
    type=click.Choice(["novice", "intermediate", "advanced", "expert"]),
    help="Filter by expertise level"
)
@click.option(
    "--turns", "-t",
    type=int,
    default=3,
    help="Number of conversation turns (default: 3, max: 20)"
)
@click.option(
    "--language", "-l",
    type=click.Choice([
        # LTR Languages
        "en", "es", "fr", "de", "it", "pt", "ru", "nl", "sv", "no", "da", "fi", "pl", "cs", "hu", "ro", "bg", "el", "tr", "ca", "eu", "gl",
        # RTL Languages
        "ar", "he", "fa", "ur", "ku", "sd", "ps", "prs", "uz-cyrl",
        # Asian Languages
        "ja", "zh-cn", "zh-tw", "ko", "th", "vi", "id", "ms", "tl", "hi", "bn", "ta", "te", "mr", "gu", "kn", "ml", "pa", "ne", "si", "my", "km", "lo", "mn", "bo"
    ]),
    default="en",
    help="Target language for generation (default: en)"
)
@click.option(
    "--context", "-ctx",
    help="Initial context or assistant response to continue from"
)
@click.option(
    "--complexity",
    type=click.Choice(["basic", "intermediate", "advanced"]),
    help="Target complexity level"
)
@click.option(
    "--output", "-o",
    type=click.Choice(["json", "text", "markdown", "csv"]),
    default="json",
    help="Output format (default: json)"
)
@click.option(
    "--output-file",
    type=click.Path(),
    help="Output file path (default: stdout)"
)
@click.option(
    "--output-filename",
    help="Custom filename for output (without extension, e.g., 'my_conversation')"
)
@click.option(
    "--seed",
    type=int,
    help="Random seed for reproducible generation"
)
@click.option(
    "--llm-backend",
    type=click.Choice(["openai", "transformers", "none"]),
    help="LLM backend for message generation (openai, transformers, or none)"
)
@click.option(
    "--llm-model",
    help="LLM model name (e.g., gpt-3.5-turbo for OpenAI, gpt2 for Transformers)"
)
@click.option(
    "--llm-api-key",
    help="API key for OpenAI backend"
)
@click.option(
    "--llm-api-url",
    help="Custom OpenAI-compatible API URL (for self-hosted or alternative APIs)"
)
@click.option(
    "--interactive", "-i",
    is_flag=True,
    help="Interactive mode for step-by-step generation"
)
@click.pass_context
def generate(ctx, **kwargs):
    """
    Generate a single conversation with specified parameters.
    
    Examples:
        persona-chat generate
        persona-chat generate --persona "Senior Software Engineer" --turns 8
        persona-chat generate --language es --domain technology --expertise advanced
        persona-chat generate --custom-persona ./my-persona.json --turns 7
        
        # Dynamic persona creation:
        persona-chat generate --create-persona --persona-name "Dr. Smith" --persona-role "Medical Researcher" --persona-domain healthcare --persona-expertise expert --turns 5
        persona-chat generate --persona-template senior_developer --persona-name "Alice" --persona-role "Lead Developer" --turns 6
        
        # Multi-persona conversations:
        persona-chat generate --multi-persona --persona-names "Senior Developer,Junior Developer,Product Manager" --turns 6
        persona-chat generate --multi-persona --persona-names "Doctor,Nurse,Patient" --persona-roles "primary,participant,observer" --turns 8
    """
    generate_command(ctx, **kwargs)


@main.command()
@click.option(
    "--conversations", "-n",
    type=int,
    default=1,
    help="Number of conversations to generate (default: 1, max: 100)"
)
@click.option(
    "--personas", "-p",
    help="Comma-separated list of persona names or 'random' for random selection"
)
@click.option(
    "--create-persona",
    is_flag=True,
    help="Create a dynamic persona on-the-fly for batch generation"
)
@click.option(
    "--persona-name",
    help="Name for the dynamic persona (used with --create-persona)"
)
@click.option(
    "--persona-role",
    help="Role for the dynamic persona (used with --create-persona)"
)
@click.option(
    "--persona-domain",
    type=click.Choice(["technology", "healthcare", "education", "business", "creative", "specialized"]),
    help="Domain for the dynamic persona (used with --create-persona)"
)
@click.option(
    "--persona-expertise",
    type=click.Choice(["novice", "intermediate", "advanced", "expert"]),
    help="Expertise level for the dynamic persona (used with --create-persona)"
)
@click.option(
    "--persona-background",
    help="Background description for the dynamic persona (used with --create-persona)"
)
@click.option(
    "--persona-communication-style",
    type=click.Choice(["formal", "casual", "technical", "friendly", "business", "academic"]),
    help="Communication style for the dynamic persona (used with --create-persona)"
)
@click.option(
    "--persona-experience",
    type=int,
    help="Years of experience for the dynamic persona (used with --create-persona)"
)
@click.option(
    "--persona-formality",
    type=int,
    help="Formality level (1-5) for the dynamic persona (used with --create-persona)"
)
@click.option(
    "--persona-template",
    type=click.Choice(["senior_developer", "junior_developer", "data_scientist", "product_manager", "designer"]),
    help="Use a predefined persona template for batch generation"
)
@click.option(
    "--multi-persona",
    is_flag=True,
    help="Enable multi-persona conversation with multiple participants"
)
@click.option(
    "--persona-names",
    help="Comma-separated list of persona names for multi-persona conversation"
)
@click.option(
    "--persona-roles",
    help="Comma-separated list of roles for personas (e.g., 'primary,participant,observer')"
)
@click.option(
    "--domain", "-d",
    type=click.Choice(["technology", "healthcare", "education", "business", "creative", "specialized"]),
    help="Filter personas by domain"
)
@click.option(
    "--expertise", "-e",
    type=click.Choice(["novice", "intermediate", "advanced", "expert", "mixed"]),
    help="Filter by expertise level or 'mixed' for variety"
)
@click.option(
    "--turns", "-t",
    type=int,
    default=5,
    help="Number of conversation turns per conversation (default: 5, max: 20)"
)
@click.option(
    "--language", "-l",
    type=click.Choice([
        # LTR Languages
        "en", "es", "fr", "de", "it", "pt", "ru", "nl", "sv", "no", "da", "fi", "pl", "cs", "hu", "ro", "bg", "el", "tr", "ca", "eu", "gl",
        # RTL Languages
        "ar", "he", "fa", "ur", "ku", "sd", "ps", "prs", "uz-cyrl",
        # Asian Languages
        "ja", "zh-cn", "zh-tw", "ko", "th", "vi", "id", "ms", "tl", "hi", "bn", "ta", "te", "mr", "gu", "kn", "ml", "pa", "ne", "si", "my", "km", "lo", "mn", "bo",
        # Special
        "mixed"
    ]),
    default="en",
    help="Target language or 'mixed' for variety (default: en)"
)
@click.option(
    "--parallel", "-par",
    is_flag=True,
    help="Generate conversations in parallel for faster processing"
)
@click.option(
    "--workers",
    type=int,
    default=4,
    help="Number of parallel workers (default: 4)"
)
@click.option(
    "--output", "-o",
    type=click.Choice(["json", "csv", "markdown"]),
    default="json",
    help="Output format (default: json)"
)
@click.option(
    "--output-dir",
    type=click.Path(),
    default="./output",
    help="Output directory for batch results (default: ./output)"
)
@click.option(
    "--output-filename",
    help="Custom filename for batch results (without extension, e.g., 'my_conversations')"
)
@click.option(
    "--progress", "-prog",
    is_flag=True,
    help="Show progress bar during generation"
)
@click.option(
    "--llm-backend",
    type=click.Choice(["openai", "transformers", "none"]),
    help="LLM backend for message generation (openai, transformers, or none)"
)
@click.option(
    "--llm-model",
    help="LLM model name (e.g., gpt-3.5-turbo for OpenAI, gpt2 for Transformers)"
)
@click.option(
    "--llm-api-key",
    help="API key for OpenAI backend"
)
@click.option(
    "--llm-api-url",
    help="Custom OpenAI-compatible API URL (for self-hosted or alternative APIs)"
)
@click.pass_context
def batch(ctx, **kwargs):
    """
    Generate multiple conversations in batch.
    
    Examples:
        persona-chat batch --conversations 10 --turns 6
        persona-chat batch --conversations 25 --domain technology --parallel
        persona-chat batch --conversations 15 --expertise mixed --language mixed
        
        # Dynamic persona batch generation:
        persona-chat batch --conversations 5 --create-persona --persona-name "Dr. Smith" --persona-role "Medical Researcher" --persona-domain healthcare --turns 4
        persona-chat batch --conversations 10 --persona-template senior_developer --persona-name "Alice" --persona-role "Lead Developer" --parallel
        # Multi-persona batch generation:
        persona-chat batch --conversations 3 --turns 3 --language ar --multi-persona --persona-names "Teacher,Student,Parent" --persona-roles "primary,participant,observer"
    """
    batch_command(ctx, **kwargs)


@main.command()
@click.option(
    "--conversation-id", "-id",
    required=True,
    help="ID of conversation to continue"
)
@click.option(
    "--context", "-ctx",
    help="New context or assistant response to continue from"
)
@click.option(
    "--turns", "-t",
    type=int,
    default=3,
    help="Additional turns to generate (default: 3, max: 10)"
)
@click.option(
    "--language", "-l",
    type=click.Choice(["en", "es", "fr", "de", "it", "pt", "ja", "zh", "ko", "ar"]),
    help="Switch language for continuation"
)
@click.option(
    "--persona-switch",
    help="Switch to different persona for continuation"
)
@click.option(
    "--output", "-o",
    type=click.Choice(["json", "text", "markdown"]),
    default="json",
    help="Output format (default: json)"
)
@click.pass_context
def continue_cmd(ctx, **kwargs):
    """
    Continue an existing conversation.
    
    Examples:
        persona-chat continue --conversation-id conv_123 --turns 5
        persona-chat continue --conversation-id conv_456 --language fr --context "New context"
    """
    continue_command(ctx, **kwargs)


@main.command()
@click.option(
    "--file", "-f",
    type=click.Path(exists=True),
    help="Path to conversation file to analyze"
)
@click.option(
    "--batch-file",
    type=click.Path(exists=True),
    help="Path to batch results file to analyze"
)
@click.option(
    "--conversations-dir",
    type=click.Path(exists=True),
    help="Directory containing conversations to analyze"
)
@click.option(
    "--metrics",
    type=click.Choice(["all", "quality", "coherence", "persona", "language"]),
    default="all",
    help="Metrics to analyze (default: all)"
)
@click.option(
    "--language-analysis",
    is_flag=True,
    help="Include detailed language quality analysis"
)
@click.option(
    "--persona-stats",
    is_flag=True,
    help="Generate persona usage statistics"
)
@click.option(
    "--output", "-o",
    type=click.Choice(["json", "csv", "markdown", "html"]),
    default="json",
    help="Output format (default: json)"
)
@click.option(
    "--output-file",
    type=click.Path(),
    help="Output file path (default: stdout)"
)
@click.pass_context
def analyze(ctx, **kwargs):
    """
    Analyze conversation quality and patterns.
    
    Examples:
        persona-chat analyze --file conversation.json --metrics quality
        persona-chat analyze --batch-file results.json --persona-stats
        persona-chat analyze --conversations-dir ./data --language-analysis
    """
    analyze_command(ctx, **kwargs)


@main.command()
@click.option(
    "--name", "-n",
    required=True,
    help="Persona name"
)
@click.option(
    "--role", "-r",
    required=True,
    help="Professional role or title"
)
@click.option(
    "--domain", "-d",
    type=click.Choice(["technology", "healthcare", "education", "business", "creative", "specialized"]),
    required=True,
    help="Primary domain of expertise"
)
@click.option(
    "--expertise", "-e",
    type=click.Choice(["novice", "intermediate", "advanced", "expert"]),
    required=True,
    help="Expertise level"
)
@click.option(
    "--interactive", "-i",
    is_flag=True,
    help="Interactive mode for guided persona creation"
)
@click.option(
    "--template",
    type=click.Choice(["basic", "technical", "business", "creative", "academic"]),
    help="Use template as starting point"
)
@click.option(
    "--output", "-o",
    type=click.Path(),
    help="Output file path for created persona"
)
@click.pass_context
def create(ctx, **kwargs):
    """
    Create a custom persona definition.
    
    Examples:
        persona-chat create --name "Maria Rodriguez" --role "Marketing Director" --domain business --expertise expert
        persona-chat create --template technical --interactive
    """
    create_command(ctx, **kwargs)


@main.command()
@click.option(
    "--file", "-f",
    type=click.Path(exists=True),
    help="Path to persona file to import"
)
@click.option(
    "--directory", "-d",
    type=click.Path(exists=True),
    help="Directory containing persona files to import"
)
@click.option(
    "--format",
    type=click.Choice(["json", "yaml", "csv"]),
    default="json",
    help="Input format (default: json)"
)
@click.option(
    "--validate", "-v",
    is_flag=True,
    help="Validate personas after import"
)
@click.option(
    "--overwrite",
    is_flag=True,
    help="Overwrite existing personas with same name"
)
@click.pass_context
def import_cmd(ctx, **kwargs):
    """
    Import personas from files.
    
    Examples:
        persona-chat import --file my-persona.json
        persona-chat import --directory ./personas --format yaml --validate
    """
    import_command(ctx, **kwargs)


@main.command()
@click.option(
    "--conversation-id", "-id",
    help="Export specific conversation by ID"
)
@click.option(
    "--batch-file",
    type=click.Path(exists=True),
    help="Export batch results file"
)
@click.option(
    "--conversations-dir",
    type=click.Path(exists=True),
    help="Export all conversations from directory"
)
@click.option(
    "--format", "-f",
    type=click.Choice(["json", "csv", "markdown", "xml", "excel"]),
    default="json",
    help="Export format (default: json)"
)
@click.option(
    "--group-by",
    type=click.Choice(["language", "domain", "persona", "date"]),
    help="Group exported data by field"
)
@click.option(
    "--include-metadata",
    is_flag=True,
    help="Include metadata in export"
)
@click.option(
    "--output", "-o",
    type=click.Path(),
    help="Output file or directory path"
)
@click.pass_context
def export(ctx, **kwargs):
    """
    Export conversation data in various formats.
    
    Examples:
        persona-chat export --conversation-id conv_123 --format markdown
        persona-chat export --batch-file results.json --format csv --group-by language
        persona-chat export --conversations-dir ./data --format excel --include-metadata
    """
    export_command(ctx, **kwargs)


@main.command()
@click.option(
    "--personas", "-p",
    is_flag=True,
    help="List available personas"
)
@click.option(
    "--languages", "-l",
    is_flag=True,
    help="List supported languages"
)
@click.option(
    "--domains", "-d",
    is_flag=True,
    help="List available domains"
)
@click.option(
    "--expertise-levels", "-e",
    is_flag=True,
    help="List expertise levels"
)
@click.option(
    "--format", "-f",
    type=click.Choice(["table", "json", "csv"]),
    default="table",
    help="Output format (default: table)"
)
@click.option(
    "--filter", "-fil",
    help="Filter results (e.g., 'domain:technology' or 'expertise:expert')"
)
@click.pass_context
def list_cmd(ctx, **kwargs):
    """
    List available personas, languages, and other resources.
    
    Examples:
        persona-chat list --personas
        persona-chat list --languages --format json
        persona-chat list --personas --filter "domain:technology"
    """
    list_command(ctx, **kwargs)


@main.command()
@click.option(
    "--file", "-f",
    type=click.Path(exists=True),
    required=True,
    help="Path to conversation file to translate"
)
@click.option(
    "--target-language", "-t",
    type=click.Choice(["en", "es", "fr", "de", "it", "pt", "ja", "zh", "ko", "ar"]),
    required=True,
    help="Target language for translation"
)
@click.option(
    "--source-language", "-s",
    type=click.Choice(["en", "es", "fr", "de", "it", "pt", "ja", "zh", "ko", "ar"]),
    help="Source language (auto-detected if not specified)"
)
@click.option(
    "--preserve-persona",
    is_flag=True,
    help="Preserve persona characteristics in translation"
)
@click.option(
    "--cultural-adaptation",
    is_flag=True,
    help="Apply cultural adaptation during translation"
)
@click.option(
    "--output", "-o",
    type=click.Path(),
    help="Output file path (default: stdout)"
)
@click.pass_context
def translate(ctx, **kwargs):
    """
    Translate existing conversations to different languages.
    
    Examples:
        persona-chat translate --file conversation.json --target-language es
        persona-chat translate --file batch_results.json --target-language ja --cultural-adaptation
    """
    translate_command(ctx, **kwargs)


def show_banner():
    """Display application banner."""
    banner_text = Text()
    banner_text.append("Persona-Based Conversation Generator", style="bold blue")
    banner_text.append("\n", style="bold blue")
    banner_text.append("Generate authentic multi-turn conversations", style="italic")
    banner_text.append("\n", style="italic")
    banner_text.append("using 50+ distinct user personas", style="italic")
    
    panel = Panel(
        banner_text,
        title="[bold green]persona-chat[/bold green]",
        border_style="green",
        padding=(1, 2)
    )
    console.print(panel)


if __name__ == "__main__":
    try:
        show_banner()
        main()
    except KeyboardInterrupt:
        console.print("\n[yellow]Operation cancelled by user[/yellow]")
        sys.exit(1)
    except Exception as e:
        console.print(f"\n[red]Error: {e}[/red]")
        sys.exit(1) 