"""
Batch processor for generating multiple conversations.
"""

from typing import List, Dict, Any, Optional
from ..models.conversation import Conversation
from ..core.conversation_engine import ConversationEngine
from ..utils.logging_utils import get_logger

logger = get_logger(__name__)


class BatchProcessor:
    """Processes batch conversation generation."""
    
    def __init__(self):
        """Initialize the batch processor."""
        self.logger = get_logger(__name__)
    
    def generate_batch(
        self,
        count: int,
        engine: ConversationEngine,
        **kwargs
    ) -> List[Conversation]:
        """
        Generate a batch of conversations.
        
        Args:
            count: Number of conversations to generate
            engine: Conversation engine to use
            **kwargs: Generation parameters
            
        Returns:
            List of generated conversations
        """
        self.logger.info(f"Starting batch generation of {count} conversations")
        
        conversations = []
        for i in range(count):
            try:
                conversation = engine.generate_conversation(**kwargs)
                conversations.append(conversation)
                self.logger.debug(f"Generated conversation {i+1}/{count}: {conversation.id}")
            except Exception as e:
                self.logger.error(f"Failed to generate conversation {i+1}: {e}")
        
        self.logger.info(f"Completed batch generation: {len(conversations)} conversations")
        return conversations 