"""
Context analyzer for understanding conversation context and patterns.
"""

import re
from typing import Dict, Any, Optional, List
from ..models.conversation import Conversation, Turn
from ..utils.logging_utils import get_logger

logger = get_logger(__name__)


class ContextAnalyzer:
    """Analyzes conversation context and extracts patterns."""
    
    def __init__(self):
        """Initialize the context analyzer."""
        self.logger = get_logger(__name__)
    
    def analyze_context(self, context: str) -> Dict[str, Any]:
        """
        Analyze a context string to extract patterns and information.
        
        Args:
            context: The context string to analyze
            
        Returns:
            Analysis results
        """
        analysis = {
            "type": self._determine_context_type(context),
            "topics": self._extract_topics(context),
            "entities": self._extract_entities(context),
            "sentiment": self._analyze_sentiment(context),
            "complexity": self._assess_complexity(context),
            "key_terms": self._extract_key_terms(context)
        }
        
        return analysis
    
    def analyze_turn_context(self, turn: Turn, conversation: Conversation) -> Dict[str, Any]:
        """
        Analyze the context of a specific turn.
        
        Args:
            turn: The turn to analyze
            conversation: The conversation containing the turn
            
        Returns:
            Turn context analysis
        """
        analysis = {
            "turn_number": turn.turn_number,
            "user_message_analysis": self.analyze_context(turn.user_message.content),
            "assistant_message_analysis": self.analyze_context(turn.assistant_message.content) if turn.assistant_message else None,
            "conversation_progress": turn.turn_number / conversation.max_turns,
            "topic_evolution": self._analyze_topic_evolution(turn, conversation)
        }
        
        return analysis
    
    def extract_key_terms(self, text: str) -> List[str]:
        """Extract key terms from text."""
        # Simple implementation - in a real system, this would use NLP techniques
        words = re.findall(r'\b\w+\b', text.lower())
        # Filter out common stop words
        stop_words = {'the', 'a', 'an', 'and', 'or', 'but', 'in', 'on', 'at', 'to', 'for', 'of', 'with', 'by', 'is', 'are', 'was', 'were', 'be', 'been', 'have', 'has', 'had', 'do', 'does', 'did', 'will', 'would', 'could', 'should', 'may', 'might', 'can', 'this', 'that', 'these', 'those', 'i', 'you', 'he', 'she', 'it', 'we', 'they', 'me', 'him', 'her', 'us', 'them'}
        key_terms = [word for word in words if word not in stop_words and len(word) > 3]
        return list(set(key_terms))[:10]  # Return top 10 unique terms
    
    def extract_topic_evolution(self, turn: Turn, conversation: Conversation) -> List[str]:
        """Extract topic evolution from a turn."""
        topics = []
        
        # Extract topics from user message
        user_topics = self._extract_topics(turn.user_message.content)
        topics.extend(user_topics)
        
        # Extract topics from assistant message
        if turn.assistant_message:
            assistant_topics = self._extract_topics(turn.assistant_message.content)
            topics.extend(assistant_topics)
        
        return list(set(topics))
    
    def analyze_topic_progression(self, conversation: Conversation) -> List[str]:
        """Analyze how topics progress throughout the conversation."""
        all_topics = []
        
        for turn in conversation.turns:
            turn_topics = self.extract_topic_evolution(turn, conversation)
            all_topics.extend(turn_topics)
        
        return list(set(all_topics))
    
    def analyze_sentiment_trend(self, conversation: Conversation) -> List[float]:
        """Analyze sentiment trend throughout the conversation."""
        sentiments = []
        
        for turn in conversation.turns:
            user_sentiment = self._analyze_sentiment(turn.user_message.content)
            sentiments.append(user_sentiment)
            
            if turn.assistant_message:
                assistant_sentiment = self._analyze_sentiment(turn.assistant_message.content)
                sentiments.append(assistant_sentiment)
        
        return sentiments
    
    def analyze_complexity_progression(self, conversation: Conversation) -> List[float]:
        """Analyze complexity progression throughout the conversation."""
        complexities = []
        
        for turn in conversation.turns:
            user_complexity = self._assess_complexity(turn.user_message.content)
            complexities.append(user_complexity)
            
            if turn.assistant_message:
                assistant_complexity = self._assess_complexity(turn.assistant_message.content)
                complexities.append(assistant_complexity)
        
        return complexities
    
    def _determine_context_type(self, context: str) -> str:
        """Determine the type of context."""
        context_lower = context.lower()
        
        if any(word in context_lower for word in ["list", "steps", "options", "ways", "methods"]):
            return "list"
        elif any(word in context_lower for word in ["explain", "describe", "define", "what is"]):
            return "explanation"
        elif any(word in context_lower for word in ["how to", "guide", "tutorial", "process"]):
            return "how_to"
        elif any(word in context_lower for word in ["problem", "issue", "error", "trouble"]):
            return "problem"
        else:
            return "general"
    
    def _extract_topics(self, text: str) -> List[str]:
        """Extract topics from text."""
        # Simple implementation - in a real system, this would use topic modeling
        topics = []
        
        # Look for domain-specific keywords
        tech_keywords = ["programming", "software", "development", "code", "algorithm", "database", "api"]
        business_keywords = ["strategy", "management", "marketing", "finance", "operations", "leadership"]
        health_keywords = ["patient", "treatment", "diagnosis", "care", "medical", "health"]
        
        text_lower = text.lower()
        
        if any(keyword in text_lower for keyword in tech_keywords):
            topics.append("technology")
        if any(keyword in text_lower for keyword in business_keywords):
            topics.append("business")
        if any(keyword in text_lower for keyword in health_keywords):
            topics.append("healthcare")
        
        return topics
    
    def _extract_entities(self, text: str) -> List[str]:
        """Extract named entities from text."""
        # Simple implementation - in a real system, this would use NER
        entities = []
        
        # Look for capitalized words that might be entities
        words = text.split()
        for word in words:
            if word[0].isupper() and len(word) > 2:
                entities.append(word)
        
        return list(set(entities))
    
    def _analyze_sentiment(self, text: str) -> float:
        """Analyze sentiment of text (returns value between -1 and 1)."""
        # Simple implementation - in a real system, this would use sentiment analysis
        positive_words = ["good", "great", "excellent", "amazing", "wonderful", "helpful", "useful"]
        negative_words = ["bad", "terrible", "awful", "difficult", "confusing", "frustrating", "problem"]
        
        text_lower = text.lower()
        positive_count = sum(1 for word in positive_words if word in text_lower)
        negative_count = sum(1 for word in negative_words if word in text_lower)
        
        if positive_count == 0 and negative_count == 0:
            return 0.0
        
        return (positive_count - negative_count) / (positive_count + negative_count)
    
    def _assess_complexity(self, text: str) -> float:
        """Assess the complexity of text (returns value between 0 and 1)."""
        # Simple implementation based on word length and sentence structure
        words = text.split()
        if not words:
            return 0.0
        
        avg_word_length = sum(len(word) for word in words) / len(words)
        sentence_count = len(text.split('.'))
        
        # Normalize complexity score
        complexity = (avg_word_length - 3) / 7 + (sentence_count - 1) / 10
        return max(0.0, min(1.0, complexity))
    
    def _extract_key_terms(self, text: str) -> List[str]:
        """Extract key terms from text."""
        # Simple implementation - in a real system, this would use NLP techniques
        words = re.findall(r'\b\w+\b', text.lower())
        # Filter out common stop words
        stop_words = {'the', 'a', 'an', 'and', 'or', 'but', 'in', 'on', 'at', 'to', 'for', 'of', 'with', 'by', 'is', 'are', 'was', 'were', 'be', 'been', 'have', 'has', 'had', 'do', 'does', 'did', 'will', 'would', 'could', 'should', 'may', 'might', 'can', 'this', 'that', 'these', 'those', 'i', 'you', 'he', 'she', 'it', 'we', 'they', 'me', 'him', 'her', 'us', 'them'}
        key_terms = [word for word in words if word not in stop_words and len(word) > 3]
        return list(set(key_terms))[:10]  # Return top 10 unique terms
    
    def _analyze_topic_evolution(self, turn: Turn, conversation: Conversation) -> List[str]:
        """Analyze how topics evolve in a specific turn."""
        # Compare with previous turns to see topic evolution
        current_topics = self.extract_topic_evolution(turn, conversation)
        
        # Get topics from previous turns
        previous_topics = []
        for prev_turn in conversation.turns[:turn.turn_number - 1]:
            prev_topics = self.extract_topic_evolution(prev_turn, conversation)
            previous_topics.extend(prev_topics)
        
        # Find new topics introduced in this turn
        new_topics = [topic for topic in current_topics if topic not in previous_topics]
        
        return new_topics 