"""
Main conversation generation engine.
"""

import time
import random
from typing import List, Optional, Dict, Any, Tuple
from datetime import datetime

from ..models.persona import Persona
from ..models.conversation import Conversation, Turn, Message, MessageRole, MessageType
from ..models.context import ContextType, ContextPattern
from .turn_manager import TurnManager
from .session_manager import SessionManager
from .context_analyzer import ContextAnalyzer
from .quality_controller import QualityController
from ..personas.manager import PersonaManager
from ..language.processor import LanguageProcessor
from ..utils.logging_utils import get_logger

# LLM backend imports (optional, only import if available)
try:
    import openai
except ImportError:
    openai = None
try:
    from transformers import pipeline, AutoModelForCausalLM, AutoTokenizer
except ImportError:
    pipeline = None
    AutoModelForCausalLM = None
    AutoTokenizer = None

logger = get_logger(__name__)


class ConversationEngine:
    """
    Main engine for generating authentic multi-turn conversations.
    
    Orchestrates the entire conversation generation process, including:
    - Persona selection and management
    - Turn generation and progression
    - Context analysis and response generation
    - Quality control and validation
    - Multi-language support
    """
    
    def __init__(self, config: Optional[Dict[str, Any]] = None):
        """Initialize the conversation engine."""
        self.config = config or {}
        
        # Initialize core components
        self.persona_manager = PersonaManager()
        self.language_processor = LanguageProcessor()
        self.turn_manager = TurnManager(conversation_engine=self)  # Pass self
        self.session_manager = SessionManager()
        self.context_analyzer = ContextAnalyzer()
        self.quality_controller = QualityController()
        
        # LLM backend selection
        self.llm_backend = self.config.get("llm_backend", "none")  # 'openai', 'transformers', or 'none'
        self.llm_model = self.config.get("llm_model", None)
        self.llm_api_key = self.config.get("llm_api_key", None)
        self.llm_api_url = self.config.get("llm_api_url", None)  # Custom OpenAI-compatible API URL
        self._init_llm_backend()
        
        # Set random seed if provided
        if "seed" in self.config:
            random.seed(self.config["seed"])
        
        logger.info("ConversationEngine initialized successfully")
    
    def _init_llm_backend(self):
        """Initialize LLM backend if configured."""
        if self.llm_backend == "openai" and openai:
            if self.llm_api_key:
                openai.api_key = self.llm_api_key
            if self.llm_api_url:
                openai.api_base = self.llm_api_url
                logger.info(f"OpenAI backend enabled with custom API URL: {self.llm_api_url}")
            else:
                logger.info("OpenAI backend enabled for conversation generation.")
        elif self.llm_backend == "transformers" and pipeline:
            model_name = self.llm_model or "gpt2"
            self.transformers_pipeline = pipeline("text-generation", model=model_name)
            logger.info(f"Transformers backend enabled with model: {model_name}")
        elif self.llm_backend == "none":
            logger.info("No LLM backend enabled. Using rule-based generation.")
        else:
            logger.warning("No valid LLM backend found. Falling back to rule-based generation.")

    def _llm_generate(self, prompt: str, max_tokens: int = 2048, temperature: float = 0.7) -> str:
        """Generate text using the selected LLM backend."""
        if self.llm_backend == "openai" and openai:
            try:
                from openai import OpenAI
                # For local Ollama, we can use a dummy API key
                api_key = self.llm_api_key or "ollama-local"
                client = OpenAI(
                    api_key=api_key,
                    base_url=self.llm_api_url
                )
                response = client.chat.completions.create(
                    model=self.llm_model or "gpt-3.5-turbo",
                    messages=[{"role": "user", "content": prompt}],
                    max_tokens=max_tokens,
                    temperature=temperature,
                )
                return response.choices[0].message.content.strip()
            except Exception as e:
                logger.error(f"OpenAI generation error: {e}")
                return "[OpenAI generation error]"
        elif self.llm_backend == "transformers" and hasattr(self, "transformers_pipeline"):
            try:
                outputs = self.transformers_pipeline(prompt, max_length=max_tokens, do_sample=True, temperature=temperature)
                return outputs[0]["generated_text"].strip()
            except Exception as e:
                logger.error(f"Transformers generation error: {e}")
                return "[Transformers generation error]"
        else:
            logger.warning("No LLM backend enabled. Using rule-based fallback.")
            return None
    
    def generate_conversation(
        self,
        persona: Optional[Persona] = None,
        persona_name: Optional[str] = None,
        personas: Optional[List[Persona]] = None,
        persona_names: Optional[List[str]] = None,
        turns: int = 3,
        language: str = "en",
        context: Optional[str] = None,
        complexity: Optional[str] = None,
        domain_focus: Optional[str] = None,
        multi_persona: bool = False,
        persona_roles: Optional[Dict[str, str]] = None,
        **kwargs
    ) -> Conversation:
        """
        Generate a complete conversation with the specified parameters.
        
        Args:
            persona: Persona object to use for generation
            persona_name: Name of persona to load and use
            turns: Number of conversation turns to generate
            language: Target language for conversation
            context: Initial context or assistant response
            complexity: Target complexity level
            domain_focus: Primary domain focus for conversation
            **kwargs: Additional generation parameters
            
        Returns:
            Generated conversation object
        """
        logger.info(f"Starting conversation generation: {turns} turns, language: {language}")
        
        # Load or validate personas
        conversation_personas = []
        
        if multi_persona:
            # Multi-persona conversation
            if personas:
                conversation_personas = personas
            elif persona_names:
                conversation_personas = []
                for name in persona_names:
                    persona = self.persona_manager.get_persona(name)
                    if persona is None:
                        # Create dynamic persona if it doesn't exist
                        persona = self.persona_manager.create_dynamic_persona(
                            name=name,
                            role=name,  # Use name as role for now
                            domain="technology",  # Default domain
                            expertise_level="intermediate"  # Default expertise
                        )
                    conversation_personas.append(persona)
            else:
                # Generate random personas for multi-persona conversation
                conversation_personas = [self.persona_manager.get_random_persona() for _ in range(2, 4)]
                conversation_personas = [p for p in conversation_personas if p is not None]
            
            if len(conversation_personas) < 2:
                raise ValueError("Multi-persona conversation requires at least 2 personas")
        else:
            # Single persona conversation
            if persona is None and persona_name:
                persona = self.persona_manager.get_persona(persona_name)
            elif persona is None:
                persona = self.persona_manager.get_random_persona()
            
            if persona is None:
                raise ValueError("No valid persona available for conversation generation")
            
            conversation_personas = [persona]
        
        # Create conversation session
        conversation = self._create_conversation(
            personas=conversation_personas,
            turns=turns,
            language=language,
            complexity=complexity,
            domain_focus=domain_focus,
            multi_persona=multi_persona,
            persona_roles=persona_roles,
            **kwargs
        )
        
        # Generate conversation turns
        for turn_num in range(1, turns + 1):
            logger.debug(f"Generating turn {turn_num}/{turns}")
            
            # Generate turn
            turn = self._generate_turn(
                conversation=conversation,
                personas=conversation_personas,
                turn_number=turn_num,
                context=context if turn_num == 1 else None,
                **kwargs
            )
            
            # Add turn to conversation
            conversation.add_turn(turn)
            
            # Update context for next turn
            if turn.assistant_message:
                context = turn.assistant_message.content
        
        # Finalize conversation
        self._finalize_conversation(conversation)
        
        logger.info(f"Conversation generation completed: {conversation.id}")
        return conversation
    
    def _create_conversation(
        self,
        personas: List[Persona],
        turns: int,
        language: str,
        complexity: Optional[str] = None,
        domain_focus: Optional[str] = None,
        multi_persona: bool = False,
        persona_roles: Optional[Dict[str, str]] = None,
        **kwargs
    ) -> Conversation:
        """Create a new conversation with initial setup."""
        
        # Determine complexity if not specified
        if complexity is None:
            complexity = personas[0].get_complexity_preference()
        
        # Determine domain focus if not specified
        if domain_focus is None:
            domain_focus = str(personas[0].domain)
        
        # Set up persona information
        persona_ids = [p.id for p in personas]
        primary_persona_id = persona_ids[0]
        active_personas = persona_ids if multi_persona else [primary_persona_id]
        
        # Set up persona roles
        roles = persona_roles or {}
        for i, persona in enumerate(personas):
            if persona.id not in roles:
                roles[persona.id] = "primary" if i == 0 else "participant"
        
        conversation = Conversation(
            personas=persona_ids,
            primary_persona=primary_persona_id,
            active_personas=active_personas,
            persona_roles=roles,
            language=language,
            max_turns=turns,
            target_complexity=complexity,
            domain_focus=domain_focus,
            created_at=datetime.now(),
            updated_at=datetime.now()
        )
        
        # Store conversation in session manager
        self.session_manager.store_conversation(conversation)
        
        logger.debug(f"Created {'multi-persona' if multi_persona else 'single-persona'} conversation: {conversation.id}")
        return conversation
    
    def _generate_turn(
        self,
        conversation: Conversation,
        personas: List[Persona],
        turn_number: int,
        context: Optional[str] = None,
        **kwargs
    ) -> Turn:
        """Generate a single conversation turn."""
        
        start_time = time.time()
        
        # Select active persona for this turn
        active_persona = self._select_active_persona(conversation, personas, turn_number)
        
        # Generate user message
        user_message = self._generate_user_message(
            conversation=conversation,
            persona=active_persona,
            turn_number=turn_number,
            context=context,
            **kwargs
        )
        
        # Generate assistant response
        assistant_message = self._generate_assistant_response(
            conversation=conversation,
            persona=active_persona,
            user_message=user_message,
            turn_number=turn_number,
            **kwargs
        )
        
        # Create turn
        turn = Turn(
            turn_number=turn_number,
            user_message=user_message,
            assistant_message=assistant_message,
            generation_time=time.time() - start_time,
            parameters=kwargs
        )
        
        # Analyze turn context and quality
        self._analyze_turn(turn, conversation, active_persona)
        
        logger.debug(f"Generated turn {turn_number} with persona {active_persona.name} in {turn.generation_time:.2f}s")
        return turn
    
    def _generate_user_message(
        self,
        conversation: Conversation,
        persona: Persona,
        turn_number: int,
        context: Optional[str] = None,
        **kwargs
    ) -> Message:
        """Generate a user message based on persona and context."""
        
        # Analyze context if provided
        context_analysis = None
        if context:
            context_analysis = self.context_analyzer.analyze_context(context)
        
        # Generate message content
        content = self.turn_manager.generate_user_message(
            persona=persona,
            conversation=conversation,
            turn_number=turn_number,
            context_analysis=context_analysis,
            language=conversation.language,
            **kwargs
        )
        
        # Create message
        message = Message(
            role=MessageRole.USER,
            content=content,
            message_type=self._determine_message_type(content, turn_number),
            persona_id=persona.id,
            persona_name=persona.name,
            expertise_level=str(persona.expertise_level),
            language=conversation.language,
            context_hints=self._extract_context_hints(content, context_analysis)
        )
        
        return message
    
    def _generate_assistant_response(
        self,
        conversation: Conversation,
        persona: Persona,
        user_message: Message,
        turn_number: int,
        **kwargs
    ) -> Message:
        """Generate an assistant response to the user message."""
        
        # Generate response content
        content = self.turn_manager.generate_assistant_response(
            user_message=user_message,
            persona=persona,
            conversation=conversation,
            turn_number=turn_number,
            language=conversation.language,
            **kwargs
        )
        
        # Create message
        message = Message(
            role=MessageRole.ASSISTANT,
            content=content,
            message_type=self._determine_message_type(content, turn_number),
            language=conversation.language
        )
        
        return message
    
    def _select_active_persona(
        self,
        conversation: Conversation,
        personas: List[Persona],
        turn_number: int
    ) -> Persona:
        """Select which persona should be active for the current turn."""
        
        if not conversation.is_multi_persona():
            # Single persona conversation - always use the primary persona
            return personas[0]
        
        # Multi-persona conversation - implement persona selection logic
        active_persona_ids = conversation.get_active_personas()
        
        if len(active_persona_ids) == 1:
            # Only one active persona
            persona_id = active_persona_ids[0]
            return next((p for p in personas if p.id == persona_id), personas[0])
        
        # Multiple active personas - implement selection strategy
        # For now, use round-robin selection
        persona_index = (turn_number - 1) % len(active_persona_ids)
        selected_persona_id = active_persona_ids[persona_index]
        
        selected_persona = next((p for p in personas if p.id == selected_persona_id), personas[0])
        
        logger.debug(f"Selected persona {selected_persona.name} for turn {turn_number}")
        return selected_persona
    
    def _determine_message_type(self, content: str, turn_number: int) -> MessageType:
        """Determine the type of message based on content and turn number."""
        
        content_lower = content.lower()
        
        # Check for question patterns
        if any(q in content_lower for q in ["?", "what", "how", "why", "when", "where", "which", "who"]):
            return MessageType.QUESTION
        
        # Check for clarification patterns
        if any(c in content_lower for c in ["clarify", "explain", "mean", "understand", "confused"]):
            return MessageType.CLARIFICATION
        
        # Check for follow-up patterns
        if any(f in content_lower for f in ["also", "additionally", "furthermore", "moreover", "besides"]):
            return MessageType.FOLLOW_UP
        
        # Check for acknowledgment patterns
        if any(a in content_lower for a in ["thanks", "thank you", "appreciate", "got it", "understood"]):
            return MessageType.ACKNOWLEDGMENT
        
        # Default to answer for assistant messages, question for user messages
        return MessageType.ANSWER
    
    def _extract_context_hints(
        self, 
        content: str, 
        context_analysis: Optional[Dict[str, Any]]
    ) -> List[str]:
        """Extract context hints from message content and analysis."""
        
        hints = []
        
        # Extract key terms from content
        key_terms = self.context_analyzer.extract_key_terms(content)
        hints.extend(key_terms)
        
        # Add context analysis hints
        if context_analysis:
            if "topics" in context_analysis:
                hints.extend(context_analysis["topics"])
            if "entities" in context_analysis:
                hints.extend(context_analysis["entities"])
        
        return list(set(hints))  # Remove duplicates
    
    def _analyze_turn(
        self, 
        turn: Turn, 
        conversation: Conversation, 
        persona: Persona
    ) -> None:
        """Analyze turn quality and context."""
        
        # Analyze context
        turn.context_analysis = self.context_analyzer.analyze_turn_context(turn, conversation)
        
        # Calculate quality score
        turn.quality_score = self.quality_controller.calculate_turn_quality(
            turn=turn,
            persona=persona,
            conversation=conversation
        )
        
        # Extract topic evolution
        turn.topic_evolution = self.context_analyzer.extract_topic_evolution(turn, conversation)
    
    def _finalize_conversation(self, conversation: Conversation) -> None:
        """Finalize conversation with quality metrics and analysis."""
        
        # Calculate overall quality metrics
        conversation.overall_quality_score = self.quality_controller.calculate_conversation_quality(conversation)
        conversation.coherence_score = self.quality_controller.calculate_coherence_score(conversation)
        conversation.persona_consistency_score = self.quality_controller.calculate_persona_consistency(conversation)
        
        # Analyze conversation progression
        conversation.topic_progression = self.context_analyzer.analyze_topic_progression(conversation)
        conversation.sentiment_trend = self.context_analyzer.analyze_sentiment_trend(conversation)
        conversation.complexity_progression = self.context_analyzer.analyze_complexity_progression(conversation)
        
        # Mark as completed
        conversation.state = "completed"
        conversation.completed_at = datetime.now()
        conversation.updated_at = datetime.now()
        
        # Update session
        self.session_manager.update_conversation(conversation)
        
        logger.info(f"Conversation finalized: {conversation.id}, Quality: {conversation.overall_quality_score:.2f}")
    
    def continue_conversation(
        self,
        conversation_id: str,
        additional_turns: int = 3,
        context: Optional[str] = None,
        **kwargs
    ) -> Conversation:
        """
        Continue an existing conversation with additional turns.
        
        Args:
            conversation_id: ID of conversation to continue
            additional_turns: Number of additional turns to generate
            context: New context to continue from
            **kwargs: Additional generation parameters
            
        Returns:
            Updated conversation object
        """
        
        # Load existing conversation
        conversation = self.session_manager.get_conversation(conversation_id)
        if not conversation:
            raise ValueError(f"Conversation {conversation_id} not found")
        
        if conversation.is_complete():
            raise ValueError(f"Conversation {conversation_id} is already complete")
        
        # Load persona
        persona = self.persona_manager.get_persona_by_id(conversation.primary_persona)
        if not persona:
            raise ValueError(f"Primary persona not found for conversation {conversation_id}")
        
        logger.info(f"Continuing conversation {conversation_id} with {additional_turns} additional turns")
        
        # Generate additional turns
        for turn_num in range(conversation.get_current_turn_number(), 
                             conversation.get_current_turn_number() + additional_turns):
            
            if conversation.is_complete():
                break
            
            # Generate turn
            turn = self._generate_turn(
                conversation=conversation,
                persona=persona,
                turn_number=turn_num,
                context=context if turn_num == conversation.get_current_turn_number() else None,
                **kwargs
            )
            
            # Add turn to conversation
            conversation.add_turn(turn)
            
            # Update context for next turn
            if turn.assistant_message:
                context = turn.assistant_message.content
        
        # Finalize conversation
        self._finalize_conversation(conversation)
        
        return conversation
    
    def get_conversation_summary(self, conversation_id: str) -> Dict[str, Any]:
        """Get a summary of a conversation."""
        
        conversation = self.session_manager.get_conversation(conversation_id)
        if not conversation:
            raise ValueError(f"Conversation {conversation_id} not found")
        
        return conversation.get_conversation_summary()
    
    def cleanup(self) -> None:
        """Clean up resources and close connections."""
        
        self.session_manager.cleanup()
        logger.info("ConversationEngine cleanup completed") 