"""
Quality controller for assessing conversation quality and consistency.
"""

from typing import Dict, Any, Optional, List
from ..models.conversation import Conversation, Turn
from ..models.persona import Persona
from ..utils.logging_utils import get_logger

logger = get_logger(__name__)


class QualityController:
    """Controls and assesses conversation quality and consistency."""
    
    def __init__(self):
        """Initialize the quality controller."""
        self.logger = get_logger(__name__)
    
    def calculate_turn_quality(
        self,
        turn: Turn,
        persona: Persona,
        conversation: Conversation
    ) -> float:
        """
        Calculate quality score for a single turn.
        
        Args:
            turn: The turn to assess
            persona: The persona involved
            conversation: The conversation containing the turn
            
        Returns:
            Quality score between 0 and 1
        """
        scores = []
        
        # Check message relevance
        relevance_score = self._assess_message_relevance(turn, conversation)
        scores.append(relevance_score)
        
        # Check persona consistency
        consistency_score = self._assess_persona_consistency(turn, persona)
        scores.append(consistency_score)
        
        # Check conversation flow
        flow_score = self._assess_conversation_flow(turn, conversation)
        scores.append(flow_score)
        
        # Check message completeness
        completeness_score = self._assess_message_completeness(turn)
        scores.append(completeness_score)
        
        # Calculate average score
        return sum(scores) / len(scores)
    
    def calculate_conversation_quality(self, conversation: Conversation) -> float:
        """
        Calculate overall quality score for a conversation.
        
        Args:
            conversation: The conversation to assess
            
        Returns:
            Quality score between 0 and 1
        """
        if not conversation.turns:
            return 0.0
        
        # Calculate average turn quality
        turn_scores = []
        for turn in conversation.turns:
            if turn.quality_score is not None:
                turn_scores.append(turn.quality_score)
        
        if not turn_scores:
            return 0.0
        
        avg_turn_quality = sum(turn_scores) / len(turn_scores)
        
        # Additional conversation-level factors
        coherence_score = self.calculate_coherence_score(conversation)
        consistency_score = self.calculate_persona_consistency(conversation)
        
        # Weighted average
        overall_quality = (avg_turn_quality * 0.5 + coherence_score * 0.3 + consistency_score * 0.2)
        
        return max(0.0, min(1.0, overall_quality))
    
    def calculate_coherence_score(self, conversation: Conversation) -> float:
        """
        Calculate conversation coherence score.
        
        Args:
            conversation: The conversation to assess
            
        Returns:
            Coherence score between 0 and 1
        """
        if len(conversation.turns) < 2:
            return 1.0
        
        coherence_scores = []
        
        for i in range(1, len(conversation.turns)):
            prev_turn = conversation.turns[i - 1]
            curr_turn = conversation.turns[i]
            
            # Check if current turn relates to previous turn
            relevance = self._assess_turn_relevance(prev_turn, curr_turn)
            coherence_scores.append(relevance)
        
        return sum(coherence_scores) / len(coherence_scores) if coherence_scores else 1.0
    
    def calculate_persona_consistency(self, conversation: Conversation) -> float:
        """
        Calculate persona consistency throughout the conversation.
        
        Args:
            conversation: The conversation to assess
            
        Returns:
            Consistency score between 0 and 1
        """
        if not conversation.turns:
            return 1.0
        
        consistency_scores = []
        
        for turn in conversation.turns:
            # Check if user message is consistent with persona
            if hasattr(turn.user_message, 'persona_id') and turn.user_message.persona_id:
                # In a real implementation, you would load the persona and check consistency
                consistency_scores.append(0.8)  # Placeholder score
            else:
                consistency_scores.append(1.0)  # No persona to check against
        
        return sum(consistency_scores) / len(consistency_scores) if consistency_scores else 1.0
    
    def validate_conversation(self, conversation: Conversation) -> Dict[str, Any]:
        """
        Validate a conversation for quality issues.
        
        Args:
            conversation: The conversation to validate
            
        Returns:
            Validation results
        """
        validation_results = {
            "is_valid": True,
            "issues": [],
            "warnings": [],
            "quality_score": 0.0
        }
        
        # Check for basic issues
        if not conversation.turns:
            validation_results["is_valid"] = False
            validation_results["issues"].append("No turns in conversation")
        
        # Check for incomplete turns
        incomplete_turns = [turn for turn in conversation.turns if not turn.is_complete()]
        if incomplete_turns:
            validation_results["warnings"].append(f"{len(incomplete_turns)} incomplete turns found")
        
        # Check conversation length
        if len(conversation.turns) < 2:
            validation_results["warnings"].append("Conversation is very short")
        
        # Calculate quality score
        validation_results["quality_score"] = self.calculate_conversation_quality(conversation)
        
        return validation_results
    
    def _assess_message_relevance(self, turn: Turn, conversation: Conversation) -> float:
        """Assess how relevant the turn is to the conversation."""
        # Simple implementation - in a real system, this would use semantic similarity
        if turn.turn_number == 1:
            return 1.0  # First turn is always relevant
        
        # Check if current turn references previous content
        current_content = turn.user_message.content.lower()
        if turn.assistant_message:
            current_content += " " + turn.assistant_message.content.lower()
        
        # Look for references to previous content
        prev_turn = conversation.turns[turn.turn_number - 2]  # Previous turn
        prev_content = prev_turn.user_message.content.lower()
        if prev_turn.assistant_message:
            prev_content += " " + prev_turn.assistant_message.content.lower()
        
        # Simple keyword overlap check
        current_words = set(current_content.split())
        prev_words = set(prev_content.split())
        
        if not prev_words:
            return 1.0
        
        overlap = len(current_words.intersection(prev_words)) / len(prev_words)
        return min(1.0, overlap * 2)  # Scale up the score
    
    def _assess_persona_consistency(self, turn: Turn, persona: Persona) -> float:
        """Assess how consistent the turn is with the persona."""
        # Check if user message uses persona-specific vocabulary
        user_content = turn.user_message.content.lower()
        
        # Count persona vocabulary usage
        vocab_matches = sum(1 for vocab in persona.vocabulary_patterns if vocab.lower() in user_content)
        
        # Check formality level
        formality_score = self._assess_formality_consistency(user_content, persona.formality_level)
        
        # Check question style consistency
        question_style_score = self._assess_question_style_consistency(turn, persona)
        
        # Combine scores
        vocab_score = min(1.0, vocab_matches / max(1, len(persona.vocabulary_patterns)))
        overall_score = (vocab_score * 0.4 + formality_score * 0.3 + question_style_score * 0.3)
        
        return overall_score
    
    def _assess_conversation_flow(self, turn: Turn, conversation: Conversation) -> float:
        """Assess how well the turn flows in the conversation."""
        if turn.turn_number == 1:
            return 1.0  # First turn always flows well
        
        # Check if turn builds on previous content
        prev_turn = conversation.turns[turn.turn_number - 2]
        
        # Simple flow check based on question-answer patterns
        prev_has_question = "?" in prev_turn.user_message.content or any(word in prev_turn.user_message.content.lower() for word in ["what", "how", "why", "when", "where"])
        curr_has_answer = turn.assistant_message and len(turn.assistant_message.content) > 20
        
        if prev_has_question and curr_has_answer:
            return 1.0
        elif not prev_has_question and not curr_has_answer:
            return 0.8
        else:
            return 0.6
    
    def _assess_message_completeness(self, turn: Turn) -> float:
        """Assess how complete the messages are."""
        user_completeness = len(turn.user_message.content.strip()) > 10
        assistant_completeness = turn.assistant_message and len(turn.assistant_message.content.strip()) > 20
        
        if user_completeness and assistant_completeness:
            return 1.0
        elif user_completeness:
            return 0.7
        else:
            return 0.4
    
    def _assess_turn_relevance(self, prev_turn: Turn, curr_turn: Turn) -> float:
        """Assess relevance between two consecutive turns."""
        # Simple keyword overlap check
        prev_content = prev_turn.user_message.content.lower()
        if prev_turn.assistant_message:
            prev_content += " " + prev_turn.assistant_message.content.lower()
        
        curr_content = curr_turn.user_message.content.lower()
        if curr_turn.assistant_message:
            curr_content += " " + curr_turn.assistant_message.content.lower()
        
        prev_words = set(prev_content.split())
        curr_words = set(curr_content.split())
        
        if not prev_words:
            return 1.0
        
        overlap = len(prev_words.intersection(curr_words)) / len(prev_words)
        return min(1.0, overlap * 2)
    
    def _assess_formality_consistency(self, content: str, target_formality: int) -> float:
        """Assess if content matches the target formality level."""
        # Simple formality assessment
        formal_indicators = ["therefore", "furthermore", "moreover", "consequently", "thus"]
        informal_indicators = ["hey", "cool", "awesome", "gonna", "wanna", "yeah"]
        
        formal_count = sum(1 for indicator in formal_indicators if indicator in content)
        informal_count = sum(1 for indicator in informal_indicators if indicator in content)
        
        if target_formality >= 4:  # High formality
            return 1.0 if formal_count > informal_count else 0.5
        elif target_formality <= 2:  # Low formality
            return 1.0 if informal_count > formal_count else 0.5
        else:  # Medium formality
            return 0.8  # Neutral score
    
    def _assess_question_style_consistency(self, turn: Turn, persona: Persona) -> float:
        """Assess if question style matches persona preferences."""
        user_content = turn.user_message.content.lower()
        
        # Check if content contains questions
        has_question = "?" in user_content or any(word in user_content for word in ["what", "how", "why", "when", "where", "which", "who"])
        
        if not has_question:
            return 0.8  # Neutral score for non-questions
        
        # Check question style against persona preferences
        if persona.question_styles:
            # Simple keyword matching
            style_matches = sum(1 for style in persona.question_styles if any(word in user_content for word in style.lower().split()))
            return min(1.0, style_matches / len(persona.question_styles))
        
        return 0.8  # Default score 