"""
Session manager for handling conversation sessions and state.
"""

from typing import Dict, Any, Optional, List
from ..models.conversation import Conversation
from ..utils.logging_utils import get_logger

logger = get_logger(__name__)


class SessionManager:
    """Manages conversation sessions and state persistence."""
    
    def __init__(self):
        """Initialize the session manager."""
        self.conversations: Dict[str, Conversation] = {}
        self.logger = get_logger(__name__)
    
    def store_conversation(self, conversation: Conversation) -> None:
        """Store a conversation in the session."""
        self.conversations[conversation.id] = conversation
        self.logger.debug(f"Stored conversation: {conversation.id}")
    
    def get_conversation(self, conversation_id: str) -> Optional[Conversation]:
        """Get a conversation by ID."""
        return self.conversations.get(conversation_id)
    
    def update_conversation(self, conversation: Conversation) -> None:
        """Update a stored conversation."""
        if conversation.id in self.conversations:
            self.conversations[conversation.id] = conversation
            self.logger.debug(f"Updated conversation: {conversation.id}")
    
    def list_conversations(self) -> List[Conversation]:
        """List all stored conversations."""
        return list(self.conversations.values())
    
    def delete_conversation(self, conversation_id: str) -> bool:
        """Delete a conversation from the session."""
        if conversation_id in self.conversations:
            del self.conversations[conversation_id]
            self.logger.debug(f"Deleted conversation: {conversation_id}")
            return True
        return False
    
    def cleanup(self) -> None:
        """Clean up session resources."""
        self.conversations.clear()
        self.logger.info("Session manager cleaned up") 