"""
Turn manager for generating conversation turns.
"""

import random
from typing import Dict, Any, Optional, List
from ..models.persona import Persona
from ..models.conversation import Conversation, Message, MessageRole, MessageType
from ..utils.logging_utils import get_logger

logger = get_logger(__name__)


class TurnManager:
    """Manages the generation of individual conversation turns."""
    
    def __init__(self, conversation_engine=None):
        """Initialize the turn manager."""
        self.logger = get_logger(__name__)
        self.conversation_engine = conversation_engine  # Reference to ConversationEngine
    
    def generate_user_message(
        self,
        persona: Persona,
        conversation: Conversation,
        turn_number: int,
        context_analysis: Optional[Dict[str, Any]] = None,
        language: str = "en",
        **kwargs
    ) -> str:
        """
        Generate a user message based on persona and context.
        """
        # Use LLM backend if available
        engine = getattr(self, 'conversation_engine', None)
        if engine and hasattr(engine, '_llm_generate') and engine.llm_backend != 'none':
            prompt = self._build_user_prompt(persona, conversation, turn_number, context_analysis, language, **kwargs)
            llm_output = engine._llm_generate(prompt)
            if llm_output:
                return llm_output
        # Fallback to rule-based
        if turn_number == 1:
            return self._generate_initial_question(persona, conversation, language)
        else:
            return self._generate_follow_up_message(persona, conversation, turn_number, context_analysis, language)
    
    def generate_assistant_response(
        self,
        user_message: Message,
        persona: Persona,
        conversation: Conversation,
        turn_number: int,
        language: str = "en",
        **kwargs
    ) -> str:
        """
        Generate an assistant response to a user message.
        """
        # Use LLM backend if available
        engine = getattr(self, 'conversation_engine', None)
        if engine and hasattr(engine, '_llm_generate') and engine.llm_backend != 'none':
            prompt = self._build_assistant_prompt(user_message, persona, conversation, turn_number, language, **kwargs)
            llm_output = engine._llm_generate(prompt)
            if llm_output:
                return llm_output
        # Fallback to rule-based
        return self._generate_assistant_message(user_message, persona, conversation, turn_number, language)

    def _build_user_prompt(self, persona, conversation, turn_number, context_analysis, language, **kwargs):
        """Build a prompt for the LLM to generate a user message."""
        # Language-specific instructions
        language_instructions = {
            "ar": "أجب باللغة العربية فقط. استخدم أسلوباً طبيعياً ومحادثة.",
            "en": "Respond in English only. Use natural conversational style.",
            "es": "Responde solo en español. Usa un estilo conversacional natural.",
            "fr": "Réponds uniquement en français. Utilise un style conversationnel naturel.",
            "de": "Antworte nur auf Deutsch. Verwende einen natürlichen Gesprächsstil.",
            "ja": "日本語でのみ回答してください。自然な会話スタイルを使用してください。",
            "zh-cn": "仅用中文回答。使用自然的对话风格。",
            "ko": "한국어로만 답변하세요. 자연스러운 대화 스타일을 사용하세요."
        }
        
        lang_instruction = language_instructions.get(language, language_instructions["en"])
        
        # Build conversation history context
        conversation_context = ""
        if conversation.turns:
            recent_turns = conversation.turns[-3:]  # Last 3 turns for context
            for turn in recent_turns:
                if turn.user_message:
                    conversation_context += f"User: {turn.user_message.content}\n"
                if turn.assistant_message:
                    conversation_context += f"Assistant: {turn.assistant_message.content}\n"
        
        prompt = f"""You are generating a user message for a conversation in {language.upper()}.

{lang_instruction}

IMPORTANT: You MUST respond ONLY in {language.upper()}. Do not use English or any other language.

Persona Information:
- Name: {persona.name}
- Role: {persona.role}
- Domain: {persona.domain}
- Expertise Level: {persona.expertise_level}
- Background: {persona.background}
- Communication Tone: {persona.tone}

Conversation Context:
- Turn Number: {turn_number}
- Language: {language.upper()}
- Total Turns: {len(conversation.turns) + 1}

Recent Conversation History:
{conversation_context}

Task: Generate a natural user message in {language.upper()} that this persona would say in this conversation.
The message should be:
1. Written ENTIRELY in {language.upper()} language
2. Appropriate for the persona's expertise level and domain
3. Natural and conversational in {language.upper()}
4. Relevant to the conversation context
5. Engaging and likely to continue the conversation

CRITICAL: Respond ONLY in {language.upper()}. Do not use English.

Generate only the user message in {language.upper()}:"""

        return prompt

    def _build_initial_question_prompt(self, persona, conversation, language):
        """Build a prompt for the LLM to generate an initial question."""
        # Language-specific instructions
        language_instructions = {
            "ar": "أجب باللغة العربية فقط. استخدم أسلوباً طبيعياً ومحادثة.",
            "en": "Respond in English only. Use natural conversational style.",
            "es": "Responde solo en español. Usa un estilo conversacional natural.",
            "fr": "Réponds uniquement en français. Utilise un style conversationnel naturel.",
            "de": "Antworte nur auf Deutsch. Verwende einen natürlichen Gesprächsstil.",
            "ja": "日本語でのみ回答してください。自然な会話スタイルを使用してください。",
            "zh-cn": "仅用中文回答。使用自然的对话风格。",
            "ko": "한국어로만 답변하세요. 자연스러운 대화 스타일을 사용하세요."
        }
        
        lang_instruction = language_instructions.get(language, language_instructions["en"])
        
        prompt = f"""You are generating an initial question for a conversation in {language.upper()}.

{lang_instruction}

IMPORTANT: You MUST respond ONLY in {language.upper()}. Do not use English or any other language.

Persona Information:
- Name: {persona.name}
- Role: {persona.role}
- Domain: {persona.domain}
- Expertise Level: {persona.expertise_level}
- Background: {persona.background}
- Communication Tone: {persona.tone}

Task: Generate a natural initial question in {language.upper()} that this persona would ask to start a conversation.
The question should be:
1. Written ENTIRELY in {language.upper()} language
2. Appropriate for the persona's expertise level and domain
3. Natural and conversational in {language.upper()}
4. Relevant to the persona's background and role
5. Engaging and likely to lead to a meaningful conversation

CRITICAL: Respond ONLY in {language.upper()}. Do not use English if it's not the language of the conversation.
CRITICAL: Your response MUST be a single, natural question in {language.upper()} to start the conversation. Do NOT greet or ask how you can help.

Generate only the question in {language.upper()}:"""

        return prompt

    def _build_assistant_prompt(self, user_message, persona, conversation, turn_number, language, **kwargs):
        """Build a prompt for the LLM to generate an assistant response."""
        # Language-specific instructions
        language_instructions = {
            "ar": "أجب باللغة العربية فقط. استخدم أسلوباً مهنياً ومفيداً.",
            "en": "Respond in English only. Use professional and helpful style.",
            "es": "Responde solo en español. Usa un estilo profesional y útil.",
            "fr": "Réponds uniquement en français. Utilise un style professionnel et utile.",
            "de": "Antworte nur auf Deutsch. Verwende einen professionellen und hilfreichen Stil.",
            "ja": "日本語でのみ回答してください。専門的で役立つスタイルを使用してください。",
            "zh-cn": "仅用中文回答。使用专业和有用的风格。",
            "ko": "한국어로만 답변하세요. 전문적이고 도움이 되는 스타일을 사용하세요."
        }
        
        lang_instruction = language_instructions.get(language, language_instructions["en"])
        
        # Build conversation history context
        conversation_context = ""
        if conversation.turns:
            recent_turns = conversation.turns[-3:]  # Last 3 turns for context
            for turn in recent_turns:
                if turn.user_message:
                    conversation_context += f"User: {turn.user_message.content}\n"
                if turn.assistant_message:
                    conversation_context += f"Assistant: {turn.assistant_message.content}\n"
        
        prompt = f"""You are generating an assistant response for a conversation in {language.upper()}.

{lang_instruction}

IMPORTANT: You MUST respond ONLY in {language.upper()}. Do not use English or any other language.

Persona Information:
- Name: {persona.name}
- Role: {persona.role}
- Domain: {persona.domain}
- Expertise Level: {persona.expertise_level}
- Background: {persona.background}
- Communication Tone: {persona.tone}

Conversation Context:
- Turn Number: {turn_number}
- Language: {language.upper()}
- Total Turns: {len(conversation.turns) + 1}

Recent Conversation History:
{conversation_context}

Current User Message: {user_message.content}

Task: Generate a helpful assistant response in {language.upper()} that:
1. Is written ENTIRELY in {language.upper()} language
2. Addresses the user's question or statement appropriately
3. Demonstrates expertise in the persona's domain
4. Uses the persona's communication style and expertise level
5. Is natural and conversational in {language.upper()}
6. Provides valuable information or guidance

CRITICAL: Respond ONLY in {language.upper()}. Do not use English.

Generate only the assistant response in {language.upper()}:"""

        return prompt
    
    def _generate_initial_question(self, persona: Persona, conversation: Conversation, language: str) -> str:
        """Generate an initial question for the conversation."""
        
        # Try LLM generation first
        engine = getattr(self, 'conversation_engine', None)
        if engine and hasattr(engine, '_llm_generate') and engine.llm_backend != 'none':
            prompt = self._build_initial_question_prompt(persona, conversation, language)
            llm_output = engine._llm_generate(prompt, max_tokens=500, temperature=0.8)
            if llm_output and llm_output.strip():
                return llm_output.strip()
        
        # Fallback to rule-based generation
        questions_by_language = {
            "ar": {
                "technology": "كيف يمكنني تحسين مهاراتي في مجال التكنولوجيا؟",
                "healthcare": "كيف يمكنني تحسين الرعاية الصحية في هذا المجال؟",
                "education": "ما هي أفضل استراتيجيات التدريس لهذا الموضوع؟",
                "business": "كيف يمكنني تحسين هذه العملية التجارية؟",
                "creative": "ما هي الطرق الإبداعية التي يمكنني استخدامها لهذا المشروع؟",
                "specialized": "ما هي الاعتبارات الرئيسية لهذا المجال المتخصص؟"
            },
            "en": {
                "technology": "What are the best practices for this technology?",
                "healthcare": "How can I improve patient care in this area?",
                "education": "What teaching strategies work best for this topic?",
                "business": "How can I optimize this business process?",
                "creative": "What creative approaches can I use for this project?",
                "specialized": "What are the key considerations for this specialized area?"
            }
        }
        
        # Get questions for the target language, fallback to English
        language_questions = questions_by_language.get(language, questions_by_language["en"])
        
        # Select a question based on persona's common questions or domain
        if language != "en" and persona.common_questions:
            first_question = persona.common_questions[0] if persona.common_questions else ""
            if any(word in first_question.lower() for word in ["what", "how", "why", "when", "where", "which", "who"]):
                question = language_questions.get(str(persona.domain), language_questions.get("technology", "How can I approach this topic?"))
            else:
                question = random.choice(persona.common_questions)
        elif persona.common_questions:
            question = random.choice(persona.common_questions)
        else:
            question = language_questions.get(str(persona.domain), language_questions.get("technology", "How can I approach this topic?"))
        
        # Add persona-specific vocabulary patterns
        if persona.vocabulary_patterns:
            vocab = random.choice(persona.vocabulary_patterns)
            if language == "ar":
                question = f"بخصوص {vocab}، {question}"
            else:
                question = f"Regarding {vocab}, {question.lower()}"
        
        return question
    
    def _generate_follow_up_message(
        self,
        persona: Persona,
        conversation: Conversation,
        turn_number: int,
        context_analysis: Optional[Dict[str, Any]],
        language: str
    ) -> str:
        """Generate a follow-up message based on the conversation context."""
        
        # Language-specific follow-up messages
        follow_up_messages = {
            "ar": {
                "asks for more details": "هل يمكنك تقديم تفاصيل أكثر تحديداً حول ذلك؟",
                "suggests alternatives": "ما هي بعض الطرق البديلة التي يمكن النظر فيها؟",
                "questions the reasoning": "ما هو المنطق وراء هذا النهج؟",
                "requests examples": "هل يمكنك إعطائي بعض الأمثلة العملية؟",
                "default": "هذا مثير للاهتمام. هل يمكنك توضيح هذه النقطة؟"
            },
            "en": {
                "asks for more details": "Could you provide more specific details about that?",
                "suggests alternatives": "What are some alternative approaches to consider?",
                "questions the reasoning": "What's the reasoning behind that approach?",
                "requests examples": "Can you give me some practical examples?",
                "default": "That's interesting. Could you elaborate on that point?"
            }
        }
        
        # Get messages for the target language, fallback to English
        language_messages = follow_up_messages.get(language, follow_up_messages["en"])
        
        # Select a follow-up pattern based on persona
        if persona.follow_up_patterns:
            pattern = random.choice(persona.follow_up_patterns)
        else:
            pattern = "asks for more details"
        
        # Generate appropriate follow-up based on pattern
        if "asks for more details" in pattern:
            return language_messages["asks for more details"]
        elif "suggests alternatives" in pattern:
            return language_messages["suggests alternatives"]
        elif "questions the reasoning" in pattern:
            return language_messages["questions the reasoning"]
        elif "requests examples" in pattern:
            return language_messages["requests examples"]
        else:
            return language_messages["default"]
    
    def _generate_assistant_message(
        self,
        user_message: Message,
        persona: Persona,
        conversation: Conversation,
        turn_number: int,
        language: str
    ) -> str:
        """Generate an assistant response to the user message."""
        
        # Analyze user message to determine response type
        message_content = user_message.content.lower()
        
        if any(word in message_content for word in ["what", "how", "why", "when", "where", "which", "who"]):
            # Question - provide informative answer
            return self._generate_informative_answer(persona, user_message, conversation)
        elif any(word in message_content for word in ["explain", "clarify", "understand", "confused"]):
            # Clarification request - provide detailed explanation
            return self._generate_detailed_explanation(persona, user_message, conversation)
        elif any(word in message_content for word in ["example", "show", "demonstrate"]):
            # Example request - provide practical example
            return self._generate_practical_example(persona, user_message, conversation)
        else:
            # General response
            return self._generate_general_response(persona, user_message, conversation)
    
    def _generate_informative_answer(self, persona: Persona, user_message: Message, conversation: Conversation) -> str:
        """Generate an informative answer to a question."""
        
        # Language-specific responses
        responses_by_language = {
            "ar": {
                "expert": "بناءً على خبرتي الواسعة في هذا المجال، إليك إجابة شاملة تأخذ في الاعتبار عوامل متعددة وأفضل الممارسات...",
                "advanced": "من فهمي المتقدم لهذا الموضوع، إليك استجابة مفصلة تغطي الجوانب الرئيسية...",
                "intermediate": "بناءً على ما تعلمته حتى الآن، إليك ما أفهمه حول هذا الموضوع...",
                "novice": "ما زلت أتعلم حول هذا، لكن إليك ما أعرفه حتى الآن..."
            },
            "en": {
                "expert": "Based on my extensive experience in this field, here's a comprehensive answer that considers multiple factors and best practices...",
                "advanced": "From my advanced understanding of this topic, here's a detailed response that covers the key aspects...",
                "intermediate": "Based on what I've learned so far, here's what I understand about this topic...",
                "novice": "I'm still learning about this, but here's what I know so far..."
            }
        }
        
        # Get responses for the conversation language, fallback to English
        language_responses = responses_by_language.get(conversation.language, responses_by_language["en"])
        
        # Base response based on persona expertise
        if str(persona.expertise_level) == "expert":
            return language_responses["expert"]
        elif str(persona.expertise_level) == "advanced":
            return language_responses["advanced"]
        elif str(persona.expertise_level) == "intermediate":
            return language_responses["intermediate"]
        else:
            return language_responses["novice"]
    
    def _generate_detailed_explanation(self, persona: Persona, user_message: Message, conversation: Conversation) -> str:
        """Generate a detailed explanation."""
        
        explanations = {
            "ar": "دعني أوضح هذا خطوة بخطوة لجعله أوضح. المفهوم الرئيسي هنا هو...",
            "en": "Let me break this down step by step to make it clearer. The key concept here is..."
        }
        return explanations.get(conversation.language, explanations["en"])
    
    def _generate_practical_example(self, persona: Persona, user_message: Message, conversation: Conversation) -> str:
        """Generate a practical example."""
        
        examples = {
            "ar": "إليك مثال عملي لتوضيح هذا المفهوم. فكر في سيناريو حيث...",
            "en": "Here's a concrete example to illustrate this concept. Consider a scenario where..."
        }
        return examples.get(conversation.language, examples["en"])
    
    def _generate_general_response(self, persona: Persona, user_message: Message, conversation: Conversation) -> str:
        """Generate a general response."""
        
        responses = {
            "ar": "هذه نقطة ممتازة. دعني أشارك بعض الرؤى حول هذا الموضوع...",
            "en": "That's a great point. Let me share some insights on this topic..."
        }
        return responses.get(conversation.language, responses["en"]) 