"""
Cultural norms adaptation functionality.
"""

from typing import Dict, Any, List, Optional
from ..utils.logging_utils import get_logger

logger = get_logger(__name__)


class CulturalNormsAdapter:
    """Adapt content for different cultural norms and expectations."""
    
    def __init__(self):
        """Initialize the cultural norms adapter."""
        self._load_cultural_norms()
    
    def _load_cultural_norms(self):
        """Load cultural norms and expectations for different regions."""
        self.cultural_norms = {
            "en": {
                "us": {
                    "formality_level": "moderate",
                    "directness": "high",
                    "personal_space": "high",
                    "time_orientation": "future",
                    "communication_style": "direct",
                    "taboo_topics": ["politics", "religion", "personal_income"],
                    "greeting_style": "casual",
                    "respect_indicators": ["please", "thank you", "excuse me"]
                },
                "uk": {
                    "formality_level": "high",
                    "directness": "moderate",
                    "personal_space": "high",
                    "time_orientation": "future",
                    "communication_style": "polite",
                    "taboo_topics": ["politics", "religion", "personal_income"],
                    "greeting_style": "formal",
                    "respect_indicators": ["please", "thank you", "pardon"]
                }
            },
            "ja": {
                "jp": {
                    "formality_level": "very_high",
                    "directness": "low",
                    "personal_space": "low",
                    "time_orientation": "future",
                    "communication_style": "indirect",
                    "taboo_topics": ["personal_income", "age", "weight"],
                    "greeting_style": "very_formal",
                    "respect_indicators": ["honorifics", "bowing", "apologies"]
                }
            },
            "ko": {
                "kr": {
                    "formality_level": "very_high",
                    "directness": "low",
                    "personal_space": "low",
                    "time_orientation": "future",
                    "communication_style": "indirect",
                    "taboo_topics": ["personal_income", "age", "weight"],
                    "greeting_style": "very_formal",
                    "respect_indicators": ["honorifics", "bowing", "apologies"]
                }
            },
            "ar": {
                "sa": {
                    "formality_level": "high",
                    "directness": "moderate",
                    "personal_space": "moderate",
                    "time_orientation": "present",
                    "communication_style": "respectful",
                    "taboo_topics": ["religion", "politics", "gender_roles"],
                    "greeting_style": "formal",
                    "respect_indicators": ["blessings", "respectful_titles", "hospitality"]
                }
            }
        }
    
    def adapt_content_for_culture(
        self, 
        content: str, 
        source_language: str, 
        target_language: str,
        source_region: Optional[str] = None,
        target_region: Optional[str] = None
    ) -> str:
        """Adapt content for cultural norms."""
        if source_language == target_language and source_region == target_region:
            return content
        
        # Get cultural norms for source and target
        source_norms = self.get_cultural_norms(source_language, source_region)
        target_norms = self.get_cultural_norms(target_language, target_region)
        
        if not source_norms or not target_norms:
            return content
        
        adapted_content = content
        
        # Adapt formality level
        if source_norms["formality_level"] != target_norms["formality_level"]:
            adapted_content = self._adapt_formality(
                adapted_content, 
                source_norms["formality_level"], 
                target_norms["formality_level"]
            )
        
        # Adapt directness
        if source_norms["directness"] != target_norms["directness"]:
            adapted_content = self._adapt_directness(
                adapted_content, 
                source_norms["directness"], 
                target_norms["directness"]
            )
        
        # Adapt respect indicators
        adapted_content = self._adapt_respect_indicators(
            adapted_content, 
            source_norms["respect_indicators"], 
            target_norms["respect_indicators"]
        )
        
        return adapted_content
    
    def get_cultural_norms(self, language: str, region: Optional[str] = None) -> Optional[Dict[str, Any]]:
        """Get cultural norms for a language and region."""
        if language not in self.cultural_norms:
            return None
        
        if region and region in self.cultural_norms[language]:
            return self.cultural_norms[language][region]
        
        # Return default region if available
        regions = list(self.cultural_norms[language].keys())
        if regions:
            return self.cultural_norms[language][regions[0]]
        
        return None
    
    def _adapt_formality(
        self, 
        content: str, 
        source_formality: str, 
        target_formality: str
    ) -> str:
        """Adapt formality level of content."""
        # Simple formality adaptation
        if source_formality == "very_high" and target_formality == "moderate":
            # Make less formal
            content = content.replace("I would like to", "I want to")
            content = content.replace("Could you please", "Can you")
        elif source_formality == "moderate" and target_formality == "very_high":
            # Make more formal
            content = content.replace("I want to", "I would like to")
            content = content.replace("Can you", "Could you please")
        
        return content
    
    def _adapt_directness(
        self, 
        content: str, 
        source_directness: str, 
        target_directness: str
    ) -> str:
        """Adapt directness level of content."""
        # Simple directness adaptation
        if source_directness == "high" and target_directness == "low":
            # Make less direct
            content = content.replace("This is wrong", "This might not be correct")
            content = content.replace("You should", "You might want to consider")
        elif source_directness == "low" and target_directness == "high":
            # Make more direct
            content = content.replace("This might not be correct", "This is wrong")
            content = content.replace("You might want to consider", "You should")
        
        return content
    
    def _adapt_respect_indicators(
        self, 
        content: str, 
        source_indicators: List[str], 
        target_indicators: List[str]
    ) -> str:
        """Adapt respect indicators in content."""
        # Simple respect indicator adaptation
        if "honorifics" in target_indicators and "honorifics" not in source_indicators:
            # Add honorifics for languages that use them
            content = content.replace("Mr. ", "Mr. ")
            content = content.replace("Ms. ", "Ms. ")
        
        return content
    
    def get_cultural_differences(
        self, 
        language1: str, 
        region1: Optional[str], 
        language2: str, 
        region2: Optional[str]
    ) -> Dict[str, Any]:
        """Get cultural differences between two language-region combinations."""
        norms1 = self.get_cultural_norms(language1, region1)
        norms2 = self.get_cultural_norms(language2, region2)
        
        if not norms1 or not norms2:
            return {}
        
        differences = {}
        for key in norms1.keys():
            if key in norms2 and norms1[key] != norms2[key]:
                differences[key] = {
                    "region1": norms1[key],
                    "region2": norms2[key]
                }
        
        return differences
    
    def is_culturally_appropriate(
        self, 
        content: str, 
        language: str, 
        region: Optional[str] = None
    ) -> Dict[str, Any]:
        """Check if content is culturally appropriate."""
        norms = self.get_cultural_norms(language, region)
        if not norms:
            return {
                "is_appropriate": True,
                "warnings": ["No cultural norms available for validation"],
                "suggestions": []
            }
        
        results = {
            "is_appropriate": True,
            "warnings": [],
            "suggestions": []
        }
        
        # Check for taboo topics
        taboo_topics = norms.get("taboo_topics", [])
        content_lower = content.lower()
        for topic in taboo_topics:
            if topic.replace("_", " ") in content_lower:
                results["warnings"].append(f"Content contains potentially taboo topic: {topic}")
                results["suggestions"].append(f"Consider avoiding or rephrasing content about {topic}")
        
        # Check formality level
        if norms["formality_level"] == "very_high":
            informal_phrases = ["hey", "what's up", "cool", "awesome"]
            for phrase in informal_phrases:
                if phrase in content_lower:
                    results["warnings"].append(f"Content may be too informal for {language} culture")
                    results["suggestions"].append("Consider using more formal language")
                    break
        
        if results["warnings"]:
            results["is_appropriate"] = False
        
        return results 