"""
Vocabulary localization functionality.
"""

from typing import Dict, Any, List, Optional
from ..utils.logging_utils import get_logger

logger = get_logger(__name__)


class VocabularyLocalizer:
    """Localize vocabulary and terminology for different regions and contexts."""
    
    def __init__(self):
        """Initialize the vocabulary localizer."""
        self._load_vocabulary_mappings()
    
    def _load_vocabulary_mappings(self):
        """Load vocabulary mappings for different languages and regions."""
        self.vocabulary_mappings = {
            "en": {
                "us": {
                    "elevator": "elevator",
                    "lift": "elevator",
                    "lorry": "truck",
                    "truck": "truck",
                    "autumn": "fall",
                    "fall": "fall"
                },
                "uk": {
                    "elevator": "lift",
                    "lift": "lift",
                    "lorry": "lorry",
                    "truck": "lorry",
                    "autumn": "autumn",
                    "fall": "autumn"
                }
            },
            "es": {
                "es": {
                    "computer": "ordenador",
                    "car": "coche",
                    "phone": "móvil"
                },
                "mx": {
                    "computer": "computadora",
                    "car": "carro",
                    "phone": "celular"
                }
            },
            "fr": {
                "fr": {
                    "computer": "ordinateur",
                    "car": "voiture",
                    "phone": "téléphone"
                },
            },
            "de": {
                "de": {
                    "computer": "Computer",
                    "car": "Auto",
                    "phone": "Telefon"
                }
            }
        }
    
    def localize_vocabulary(
        self, 
        text: str, 
        language: str, 
        region: Optional[str] = None
    ) -> str:
        """Localize vocabulary in text for specific language and region."""
        if language not in self.vocabulary_mappings:
            return text
        
        # Use default region if not specified
        if region is None:
            regions = list(self.vocabulary_mappings[language].keys())
            region = regions[0] if regions else None
        
        if not region or region not in self.vocabulary_mappings[language]:
            return text
        
        localized_text = text
        mappings = self.vocabulary_mappings[language][region]
        
        # Apply vocabulary mappings
        for original, localized in mappings.items():
            localized_text = localized_text.replace(original, localized)
        
        return localized_text
    
    def get_regional_variants(self, language: str) -> List[str]:
        """Get available regional variants for a language."""
        if language not in self.vocabulary_mappings:
            return []
        
        return list(self.vocabulary_mappings[language].keys())
    
    def add_vocabulary_mapping(
        self, 
        language: str, 
        region: str, 
        original: str, 
        localized: str
    ) -> None:
        """Add a new vocabulary mapping."""
        if language not in self.vocabulary_mappings:
            self.vocabulary_mappings[language] = {}
        
        if region not in self.vocabulary_mappings[language]:
            self.vocabulary_mappings[language][region] = {}
        
        self.vocabulary_mappings[language][region][original] = localized
    
    def get_vocabulary_differences(
        self, 
        language: str, 
        region1: str, 
        region2: str
    ) -> Dict[str, Dict[str, str]]:
        """Get vocabulary differences between two regions."""
        if language not in self.vocabulary_mappings:
            return {}
        
        if region1 not in self.vocabulary_mappings[language] or region2 not in self.vocabulary_mappings[language]:
            return {}
        
        vocab1 = self.vocabulary_mappings[language][region1]
        vocab2 = self.vocabulary_mappings[language][region2]
        
        differences = {
            "region1_only": {},
            "region2_only": {},
            "different": {}
        }
        
        # Find words only in region1
        for word, translation in vocab1.items():
            if word not in vocab2:
                differences["region1_only"][word] = translation
        
        # Find words only in region2
        for word, translation in vocab2.items():
            if word not in vocab1:
                differences["region2_only"][word] = translation
        
        # Find words with different translations
        for word in set(vocab1.keys()) & set(vocab2.keys()):
            if vocab1[word] != vocab2[word]:
                differences["different"][word] = {
                    "region1": vocab1[word],
                    "region2": vocab2[word]
                }
        
        return differences 