"""
Translation functionality for conversations and personas.
"""

from typing import Dict, Any, List, Optional
from ..models.conversation import Conversation, Message, Turn
from ..models.persona import Persona
from ..utils.logging_utils import get_logger

logger = get_logger(__name__)


class PersonaTranslator:
    """Translate conversations and maintain persona characteristics."""
    
    def __init__(self):
        """Initialize the translator."""
        self.supported_languages = ["en", "es", "fr", "de", "it", "pt", "ja", "zh", "ko", "ar"]
    
    def translate_conversation(
        self, 
        conversation: Conversation, 
        target_language: str,
        preserve_persona: bool = True
    ) -> Conversation:
        """Translate a conversation to the target language."""
        if target_language not in self.supported_languages:
            raise ValueError(f"Unsupported language: {target_language}")
        
        if conversation.language == target_language:
            return conversation
        
        # Create translated conversation
        translated_conversation = Conversation(
            id=conversation.id,
            title=conversation.title,
            description=conversation.description,
            personas=conversation.personas,
            primary_persona=conversation.primary_persona,
            language=target_language,
            turns=[],
            state=conversation.state,
            created_at=conversation.created_at,
            updated_at=conversation.updated_at,
            completed_at=conversation.completed_at,
            max_turns=conversation.max_turns,
            target_complexity=conversation.target_complexity,
            domain_focus=conversation.domain_focus,
            overall_quality_score=conversation.overall_quality_score,
            coherence_score=conversation.coherence_score,
            persona_consistency_score=conversation.persona_consistency_score,
            topic_progression=conversation.topic_progression,
            sentiment_trend=conversation.sentiment_trend,
            complexity_progression=conversation.complexity_progression,
            tags=conversation.tags,
            category=conversation.category
        )
        
        # Translate turns
        for turn in conversation.turns:
            translated_turn = self._translate_turn(turn, target_language, preserve_persona)
            translated_conversation.turns.append(translated_turn)
        
        return translated_conversation
    
    def _translate_turn(
        self, 
        turn: Turn, 
        target_language: str, 
        preserve_persona: bool
    ) -> Turn:
        """Translate a conversation turn."""
        # Translate user message
        translated_user_message = self._translate_message(
            turn.user_message, 
            target_language, 
            preserve_persona
        )
        
        # Translate assistant message if it exists
        translated_assistant_message = None
        if turn.assistant_message:
            translated_assistant_message = self._translate_message(
                turn.assistant_message, 
                target_language, 
                preserve_persona
            )
        
        # Create translated turn
        translated_turn = Turn(
            id=turn.id,
            turn_number=turn.turn_number,
            user_message=translated_user_message,
            assistant_message=translated_assistant_message,
            timestamp=turn.timestamp,
            context_analysis=turn.context_analysis,
            quality_score=turn.quality_score,
            topic_evolution=turn.topic_evolution,
            generation_time=turn.generation_time,
            model_used=turn.model_used,
            parameters=turn.parameters
        )
        
        return translated_turn
    
    def _translate_message(
        self, 
        message: Message, 
        target_language: str, 
        preserve_persona: bool
    ) -> Message:
        """Translate a message while preserving persona characteristics."""
        # For now, use a simple placeholder translation
        # In a real implementation, this would use a translation service
        translated_content = self._simple_translate(message.content, target_language)
        
        # Create translated message
        translated_message = Message(
            id=message.id,
            role=message.role,
            content=translated_content,
            message_type=message.message_type,
            timestamp=message.timestamp,
            context_hints=message.context_hints,
            confidence_score=message.confidence_score,
            language=target_language,
            persona_id=message.persona_id,
            persona_name=message.persona_name,
            expertise_level=message.expertise_level,
            sentiment_score=message.sentiment_score,
            complexity_score=message.complexity_score,
            topic_tags=message.topic_tags
        )
        
        return translated_message
    
    def _simple_translate(self, text: str, target_language: str) -> str:
        """Simple placeholder translation."""
        # This is a placeholder implementation
        # In a real system, this would use a proper translation service
        translations = {
            "es": f"[ES] {text}",
            "fr": f"[FR] {text}",
            "de": f"[DE] {text}",
            "it": f"[IT] {text}",
            "pt": f"[PT] {text}",
            "ja": f"[JA] {text}",
            "zh": f"[ZH] {text}",
            "ko": f"[KO] {text}",
            "ar": f"[AR] {text}"
        }
        
        return translations.get(target_language, text)
    
    def translate_persona_description(
        self, 
        persona: Persona, 
        target_language: str
    ) -> Dict[str, str]:
        """Translate persona descriptions and attributes."""
        translations = {}
        
        # Translate role
        translations["role"] = self._simple_translate(persona.role, target_language)
        
        # Translate description if available
        if hasattr(persona, 'description') and persona.description:
            translations["description"] = self._simple_translate(persona.description, target_language)
        
        # Translate background if available
        if hasattr(persona, 'background') and persona.background:
            translations["background"] = self._simple_translate(persona.background, target_language)
        
        return translations
    
    def is_language_supported(self, language: str) -> bool:
        """Check if a language is supported for translation."""
        return language in self.supported_languages 