"""
Language validation functionality.
"""

import re
from typing import Dict, Any, List, Optional, Tuple
from ..utils.logging_utils import get_logger

logger = get_logger(__name__)


class LanguageValidator:
    """Validate language quality and characteristics."""
    
    def __init__(self):
        """Initialize the language validator."""
        self.supported_languages = ["en", "es", "fr", "de", "it", "pt", "ja", "zh", "ko", "ar"]
        self._load_language_rules()
    
    def _load_language_rules(self):
        """Load language-specific validation rules."""
        self.language_rules = {
            "en": {
                "min_word_length": 2,
                "max_sentence_length": 200,
                "common_words": ["the", "a", "an", "and", "or", "but", "in", "on", "at", "to", "for", "of", "with", "by"],
                "punctuation": [".", "!", "?", ",", ";", ":"],
                "capitalization": True
            },
            "es": {
                "min_word_length": 2,
                "max_sentence_length": 200,
                "common_words": ["el", "la", "los", "las", "un", "una", "y", "o", "pero", "en", "de", "con", "por", "para"],
                "punctuation": [".", "!", "?", ",", ";", ":"],
                "capitalization": True
            },
            "fr": {
                "min_word_length": 2,
                "max_sentence_length": 200,
                "common_words": ["le", "la", "les", "un", "une", "et", "ou", "mais", "dans", "de", "avec", "pour", "par"],
                "punctuation": [".", "!", "?", ",", ";", ":"],
                "capitalization": True
            },
            "de": {
                "min_word_length": 2,
                "max_sentence_length": 200,
                "common_words": ["der", "die", "das", "ein", "eine", "und", "oder", "aber", "in", "von", "mit", "für", "durch"],
                "punctuation": [".", "!", "?", ",", ";", ":"],
                "capitalization": True
            }
        }
    
    def validate_text(
        self, 
        text: str, 
        language: str
    ) -> Dict[str, Any]:
        """Validate text quality for a specific language."""
        if language not in self.supported_languages:
            return {
                "is_valid": False,
                "errors": [f"Unsupported language: {language}"],
                "warnings": [],
                "score": 0.0
            }
        
        results = {
            "is_valid": True,
            "errors": [],
            "warnings": [],
            "score": 1.0,
            "metrics": {}
        }
        
        # Basic text validation
        if not text or not text.strip():
            results["errors"].append("Text is empty or contains only whitespace")
            results["is_valid"] = False
            results["score"] = 0.0
            return results
        
        # Language-specific validation
        if language in self.language_rules:
            rules = self.language_rules[language]
            
            # Check sentence length
            sentences = self._split_sentences(text)
            for i, sentence in enumerate(sentences):
                if len(sentence) > rules["max_sentence_length"]:
                    results["warnings"].append(f"Sentence {i+1} is too long ({len(sentence)} chars)")
            
            # Check word length
            words = text.split()
            short_words = [word for word in words if len(word) < rules["min_word_length"]]
            if short_words:
                results["warnings"].append(f"Found {len(short_words)} words shorter than {rules['min_word_length']} characters")
            
            # Check punctuation
            if not any(punct in text for punct in rules["punctuation"]):
                results["warnings"].append("No punctuation found")
            
            # Check capitalization
            if rules["capitalization"]:
                sentences = self._split_sentences(text)
                for i, sentence in enumerate(sentences):
                    if sentence and not sentence[0].isupper():
                        results["warnings"].append(f"Sentence {i+1} doesn't start with capital letter")
        
        # Calculate quality metrics
        results["metrics"] = self._calculate_metrics(text, language)
        
        # Adjust score based on errors and warnings
        if results["errors"]:
            results["score"] = max(0.0, results["score"] - 0.5)
        if results["warnings"]:
            results["score"] = max(0.0, results["score"] - 0.1 * len(results["warnings"]))
        
        return results
    
    def _split_sentences(self, text: str) -> List[str]:
        """Split text into sentences."""
        # Simple sentence splitting
        sentences = re.split(r'[.!?]+', text)
        return [s.strip() for s in sentences if s.strip()]
    
    def _calculate_metrics(self, text: str, language: str) -> Dict[str, Any]:
        """Calculate text quality metrics."""
        words = text.split()
        sentences = self._split_sentences(text)
        
        metrics = {
            "word_count": len(words),
            "sentence_count": len(sentences),
            "avg_sentence_length": len(words) / len(sentences) if sentences else 0,
            "avg_word_length": sum(len(word) for word in words) / len(words) if words else 0,
            "unique_words": len(set(words)),
            "vocabulary_diversity": len(set(words)) / len(words) if words else 0
        }
        
        return metrics
    
    def detect_language(self, text: str) -> Tuple[str, float]:
        """Detect the language of text with confidence score."""
        # Simple language detection based on common words
        # In a real implementation, this would use a proper language detection library
        
        language_scores = {}
        
        for language in self.supported_languages:
            if language in self.language_rules:
                rules = self.language_rules[language]
                common_words = rules["common_words"]
                
                text_lower = text.lower()
                word_count = 0
                for word in common_words:
                    word_count += text_lower.count(word)
                
                language_scores[language] = word_count / len(text.split()) if text.split() else 0
        
        if not language_scores:
            return "en", 0.0
        
        detected_language = max(language_scores.items(), key=lambda x: x[1])
        return detected_language[0], detected_language[1]
    
    def validate_conversation_language(
        self, 
        conversation_text: str, 
        expected_language: str
    ) -> Dict[str, Any]:
        """Validate that conversation text matches expected language."""
        detected_language, confidence = self.detect_language(conversation_text)
        
        results = {
            "is_valid": detected_language == expected_language,
            "detected_language": detected_language,
            "expected_language": expected_language,
            "confidence": confidence,
            "errors": [],
            "warnings": []
        }
        
        if detected_language != expected_language:
            results["errors"].append(
                f"Language mismatch: detected {detected_language}, expected {expected_language}"
            )
        
        if confidence < 0.3:
            results["warnings"].append(f"Low language detection confidence: {confidence:.2f}")
        
        return results 