"""
Application configuration models.
"""

from typing import Dict, Any, Optional, List
from pydantic import BaseModel, Field


class AppConfig(BaseModel):
    """Main application configuration."""
    
    # Logging configuration
    log_level: str = Field(default="INFO", description="Logging level")
    log_file: Optional[str] = Field(None, description="Log file path")
    log_format: str = Field(default="json", description="Log format")
    
    # Data storage
    data_dir: Optional[str] = Field(None, description="Data directory (optional)")
    personas_dir: Optional[str] = Field(None, description="Personas directory (optional)")
    conversations_dir: str = Field(default="./output", description="Conversations output directory")
    
    # LLM configuration
    llm_backend: str = Field(default="none", description="LLM backend (openai, transformers, none)")
    llm_model: Optional[str] = Field(None, description="LLM model name")
    llm_api_key: Optional[str] = Field(None, description="LLM API key")
    llm_api_url: Optional[str] = Field(None, description="LLM API URL")
    
    # Generation settings
    default_turns: int = Field(default=3, ge=1, le=20, description="Default number of turns")
    default_language: str = Field(default="en", description="Default language")
    max_conversation_length: int = Field(default=50, ge=1, le=100, description="Maximum conversation length")
    
    # Quality control
    enable_quality_control: bool = Field(default=True, description="Enable quality control")
    min_quality_score: float = Field(default=0.7, ge=0.0, le=1.0, description="Minimum quality score")
    enable_sentiment_analysis: bool = Field(default=True, description="Enable sentiment analysis")
    
    # Performance settings
    parallel_processing: bool = Field(default=True, description="Enable parallel processing")
    max_workers: int = Field(default=4, ge=1, le=16, description="Maximum parallel workers")
    cache_enabled: bool = Field(default=True, description="Enable caching")
    
    class Config:
        """Pydantic configuration."""
        env_prefix = "PERSONA_CHAT_"
    
    def to_dict(self) -> Dict[str, Any]:
        """Convert config to dictionary."""
        return self.dict()
    
    @classmethod
    def from_env(cls) -> 'AppConfig':
        """Create config from environment variables."""
        return cls()


class GenerationConfig(BaseModel):
    """Configuration for conversation generation."""
    
    # Persona settings
    persona_name: Optional[str] = Field(None, description="Persona name to use")
    persona_domain: Optional[str] = Field(None, description="Persona domain filter")
    persona_expertise: Optional[str] = Field(None, description="Persona expertise level")
    
    # Conversation settings
    turns: int = Field(default=3, ge=1, le=20, description="Number of turns to generate")
    language: str = Field(default="en", description="Target language")
    complexity: str = Field(default="medium", description="Target complexity level")
    domain_focus: Optional[str] = Field(None, description="Domain focus for conversation")
    
    # Context and flow
    initial_context: Optional[str] = Field(None, description="Initial conversation context")
    topic_evolution: bool = Field(default=True, description="Allow topic evolution")
    maintain_coherence: bool = Field(default=True, description="Maintain conversation coherence")
    
    # Quality settings
    quality_threshold: float = Field(default=0.7, ge=0.0, le=1.0, description="Quality threshold")
    enable_validation: bool = Field(default=True, description="Enable generation validation")
    retry_failed_generations: bool = Field(default=True, description="Retry failed generations")
    
    # Advanced settings
    temperature: float = Field(default=0.7, ge=0.0, le=2.0, description="Generation temperature")
    max_tokens: int = Field(default=256, ge=1, le=2048, description="Maximum tokens per message")
    seed: Optional[int] = Field(None, description="Random seed for reproducibility")
    
    class Config:
        """Pydantic configuration."""
        use_enum_values = True
    
    def to_dict(self) -> Dict[str, Any]:
        """Convert config to dictionary."""
        return self.dict()
    
    def merge_with(self, other: 'GenerationConfig') -> 'GenerationConfig':
        """Merge with another config, other takes precedence."""
        current_dict = self.dict()
        other_dict = other.dict()
        
        # Remove None values from other_dict
        other_dict = {k: v for k, v in other_dict.items() if v is not None}
        
        # Merge dictionaries
        merged_dict = {**current_dict, **other_dict}
        return GenerationConfig(**merged_dict) 