"""
Context analysis models and types.
"""

from enum import Enum
from typing import List, Dict, Any, Optional
from pydantic import BaseModel, Field


class ContextType(str, Enum):
    """Types of context patterns."""
    QUESTION = "question"
    EXPLANATION = "explanation"
    CODE_EXAMPLE = "code_example"
    LIST = "list"
    RECOMMENDATION = "recommendation"
    CLARIFICATION = "clarification"
    TRANSITION = "transition"
    ACKNOWLEDGMENT = "acknowledgment"


class ContextPattern(BaseModel):
    """Pattern detected in conversation context."""
    
    pattern_type: ContextType = Field(..., description="Type of context pattern")
    confidence: float = Field(..., ge=0.0, le=1.0, description="Confidence in pattern detection")
    keywords: List[str] = Field(default_factory=list, description="Keywords that triggered this pattern")
    metadata: Dict[str, Any] = Field(default_factory=dict, description="Additional pattern metadata")
    
    class Config:
        """Pydantic configuration."""
        use_enum_values = True
    
    def to_dict(self) -> Dict[str, Any]:
        """Convert pattern to dictionary."""
        return self.dict()


class ContextAnalysis(BaseModel):
    """Complete context analysis for a conversation turn."""
    
    detected_patterns: List[ContextPattern] = Field(default_factory=list, description="Detected context patterns")
    primary_pattern: Optional[ContextType] = Field(None, description="Primary pattern type")
    complexity_level: str = Field(default="medium", description="Detected complexity level")
    topic_tags: List[str] = Field(default_factory=list, description="Topics discussed")
    sentiment_score: Optional[float] = Field(None, ge=-1.0, le=1.0, description="Sentiment score")
    follow_up_suggestions: List[str] = Field(default_factory=list, description="Suggested follow-up questions")
    
    class Config:
        """Pydantic configuration."""
        use_enum_values = True
    
    def to_dict(self) -> Dict[str, Any]:
        """Convert analysis to dictionary."""
        return self.dict() 