"""
Conversation data models and structures.
"""

from enum import Enum
from typing import List, Optional, Dict, Any
from datetime import datetime
from pydantic import BaseModel, Field, validator
import uuid


class ConversationState(str, Enum):
    """Conversation states."""
    ACTIVE = "active"
    PAUSED = "paused"
    COMPLETED = "completed"
    ARCHIVED = "archived"


class MessageRole(str, Enum):
    """Message roles in conversation."""
    USER = "user"
    ASSISTANT = "assistant"


class MessageType(str, Enum):
    """Types of messages."""
    QUESTION = "question"
    ANSWER = "answer"
    CLARIFICATION = "clarification"
    FOLLOW_UP = "follow_up"
    ACKNOWLEDGMENT = "acknowledgment"
    TRANSITION = "transition"


class Message(BaseModel):
    """Individual message in a conversation."""
    
    id: str = Field(default_factory=lambda: str(uuid.uuid4()))
    role: MessageRole = Field(..., description="Who sent the message")
    content: str = Field(..., description="Message content")
    message_type: MessageType = Field(..., description="Type of message")
    timestamp: datetime = Field(default_factory=datetime.now)
    
    # Context and metadata
    context_hints: List[str] = Field(default_factory=list, description="Context hints for generation")
    confidence_score: Optional[float] = Field(None, ge=0.0, le=1.0, description="Confidence in message appropriateness")
    language: str = Field(default="en", description="Language of the message")
    
    # Persona-specific attributes
    persona_id: Optional[str] = Field(None, description="ID of the persona who sent this message")
    persona_name: Optional[str] = Field(None, description="Name of the persona")
    expertise_level: Optional[str] = Field(None, description="Expertise level when message was sent")
    
    # Analysis metadata
    sentiment_score: Optional[float] = Field(None, description="Sentiment analysis score")
    complexity_score: Optional[float] = Field(None, description="Complexity level of the message")
    topic_tags: List[str] = Field(default_factory=list, description="Topics discussed in this message")
    
    class Config:
        """Pydantic configuration."""
        use_enum_values = True
        json_encoders = {
            datetime: lambda v: v.isoformat()
        }
    
    def to_dict(self) -> Dict[str, Any]:
        """Convert message to dictionary."""
        return self.dict()


class Turn(BaseModel):
    """A conversation turn containing user message and assistant response."""
    
    id: str = Field(default_factory=lambda: str(uuid.uuid4()))
    turn_number: int = Field(..., ge=1, description="Turn number in conversation")
    user_message: Message = Field(..., description="User's message")
    assistant_message: Optional[Message] = Field(None, description="Assistant's response")
    timestamp: datetime = Field(default_factory=datetime.now)
    
    # Turn-specific metadata
    context_analysis: Dict[str, Any] = Field(default_factory=dict, description="Analysis of turn context")
    quality_score: Optional[float] = Field(None, ge=0.0, le=1.0, description="Overall quality score for this turn")
    topic_evolution: List[str] = Field(default_factory=list, description="How topics evolved in this turn")
    
    # Generation metadata
    generation_time: Optional[float] = Field(None, description="Time taken to generate this turn")
    model_used: Optional[str] = Field(None, description="Model used for generation")
    parameters: Dict[str, Any] = Field(default_factory=dict, description="Generation parameters used")
    
    class Config:
        """Pydantic configuration."""
        json_encoders = {
            datetime: lambda v: v.isoformat()
        }
    
    def is_complete(self) -> bool:
        """Check if turn has both user and assistant messages."""
        return self.assistant_message is not None
    
    def get_duration(self) -> Optional[float]:
        """Get duration of this turn if assistant message exists."""
        if self.assistant_message:
            return (self.assistant_message.timestamp - self.user_message.timestamp).total_seconds()
        return None
    
    def to_dict(self) -> Dict[str, Any]:
        """Convert turn to dictionary."""
        return self.dict()


class Conversation(BaseModel):
    """Complete conversation structure."""
    
    id: str = Field(default_factory=lambda: str(uuid.uuid4()))
    title: Optional[str] = Field(None, description="Conversation title")
    description: Optional[str] = Field(None, description="Conversation description")
    
    # Participants
    personas: List[str] = Field(default_factory=list, description="List of persona IDs in conversation")
    primary_persona: Optional[str] = Field(None, description="Primary persona ID")
    active_personas: List[str] = Field(default_factory=list, description="Currently active personas in conversation")
    persona_roles: Dict[str, str] = Field(default_factory=dict, description="Roles of each persona in conversation")
    language: str = Field(default="en", description="Primary language of conversation")
    
    # Structure
    turns: List[Turn] = Field(default_factory=list, description="Conversation turns")
    state: ConversationState = Field(default=ConversationState.ACTIVE, description="Current state")
    
    # Metadata
    created_at: datetime = Field(default_factory=datetime.now)
    updated_at: datetime = Field(default_factory=datetime.now)
    completed_at: Optional[datetime] = Field(None, description="When conversation was completed")
    
    # Configuration
    max_turns: int = Field(default=10, ge=1, le=50, description="Maximum number of turns")
    target_complexity: str = Field(default="medium", description="Target complexity level")
    domain_focus: Optional[str] = Field(None, description="Primary domain focus")
    
    # Quality metrics
    overall_quality_score: Optional[float] = Field(None, ge=0.0, le=1.0, description="Overall conversation quality")
    coherence_score: Optional[float] = Field(None, ge=0.0, le=1.0, description="Conversation coherence")
    persona_consistency_score: Optional[float] = Field(None, ge=0.0, le=1.0, description="Persona consistency")
    
    # Analysis data
    topic_progression: List[str] = Field(default_factory=list, description="How topics progressed")
    sentiment_trend: List[float] = Field(default_factory=list, description="Sentiment trend over conversation")
    complexity_progression: List[float] = Field(default_factory=list, description="Complexity progression")
    
    # Tags and categorization
    tags: List[str] = Field(default_factory=list, description="Conversation tags")
    category: Optional[str] = Field(None, description="Conversation category")
    
    class Config:
        """Pydantic configuration."""
        use_enum_values = True
        json_encoders = {
            datetime: lambda v: v.isoformat()
        }
    
    @validator('turns')
    def validate_turns_order(cls, v):
        """Validate turn numbers are sequential."""
        for i, turn in enumerate(v, 1):
            if turn.turn_number != i:
                raise ValueError(f"Turn numbers must be sequential. Expected {i}, got {turn.turn_number}")
        return v
    
    def get_current_turn_number(self) -> int:
        """Get the current turn number."""
        return len(self.turns) + 1
    
    def is_complete(self) -> bool:
        """Check if conversation is complete."""
        return self.state == ConversationState.COMPLETED or len(self.turns) >= self.max_turns
    
    def add_turn(self, turn: Turn) -> None:
        """Add a turn to the conversation."""
        if self.is_complete():
            raise ValueError("Cannot add turn to completed conversation")
        
        turn.turn_number = self.get_current_turn_number()
        self.turns.append(turn)
        self.updated_at = datetime.now()
        
        # Update state if max turns reached
        if len(self.turns) >= self.max_turns:
            self.state = ConversationState.COMPLETED
            self.completed_at = datetime.now()
    
    def get_last_turn(self) -> Optional[Turn]:
        """Get the last turn in the conversation."""
        return self.turns[-1] if self.turns else None
    
    def get_last_message(self) -> Optional[Message]:
        """Get the last message in the conversation."""
        last_turn = self.get_last_turn()
        if last_turn and last_turn.assistant_message:
            return last_turn.assistant_message
        elif last_turn:
            return last_turn.user_message
        return None
    
    def add_persona(self, persona_id: str, role: str = "participant") -> None:
        """Add a persona to the conversation."""
        if persona_id not in self.personas:
            self.personas.append(persona_id)
        if persona_id not in self.active_personas:
            self.active_personas.append(persona_id)
        self.persona_roles[persona_id] = role
    
    def remove_persona(self, persona_id: str) -> None:
        """Remove a persona from the conversation."""
        if persona_id in self.personas:
            self.personas.remove(persona_id)
        if persona_id in self.active_personas:
            self.active_personas.remove(persona_id)
        if persona_id in self.persona_roles:
            del self.persona_roles[persona_id]
    
    def switch_active_persona(self, persona_id: str) -> bool:
        """Switch the active persona for the next turn."""
        if persona_id in self.personas:
            self.active_personas = [persona_id]  # Set as the only active persona
            return True
        return False
    
    def add_active_persona(self, persona_id: str) -> bool:
        """Add a persona to the active participants."""
        if persona_id in self.personas and persona_id not in self.active_personas:
            self.active_personas.append(persona_id)
            return True
        return False
    
    def get_active_personas(self) -> List[str]:
        """Get currently active personas."""
        return self.active_personas.copy()
    
    def get_persona_role(self, persona_id: str) -> Optional[str]:
        """Get the role of a specific persona."""
        return self.persona_roles.get(persona_id)
    
    def is_multi_persona(self) -> bool:
        """Check if this is a multi-persona conversation."""
        return len(self.personas) > 1
    
    def get_persona_turn_count(self, persona_id: str) -> int:
        """Get the number of turns a specific persona has participated in."""
        count = 0
        for turn in self.turns:
            if turn.user_message.persona_id == persona_id:
                count += 1
            if turn.assistant_message and turn.assistant_message.persona_id == persona_id:
                count += 1
        return count
    
    def get_conversation_summary(self) -> Dict[str, Any]:
        """Get a summary of the conversation."""
        return {
            "id": self.id,
            "title": self.title,
            "turn_count": len(self.turns),
            "state": self.state,
            "language": self.language,
            "primary_persona": self.primary_persona,
            "created_at": self.created_at.isoformat(),
            "completed_at": self.completed_at.isoformat() if self.completed_at else None,
            "quality_score": self.overall_quality_score,
            "tags": self.tags,
        }
    
    def to_dict(self) -> Dict[str, Any]:
        """Convert conversation to dictionary."""
        return self.dict()
    
    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> 'Conversation':
        """Create conversation from dictionary."""
        return cls(**data) 