"""
Language models and configuration.
"""

from enum import Enum
from typing import Dict, Any, Optional, List
from pydantic import BaseModel, Field


class Language(str, Enum):
    """Supported languages."""
    # LTR Languages
    ENGLISH = "en"
    SPANISH = "es"
    FRENCH = "fr"
    GERMAN = "de"
    ITALIAN = "it"
    PORTUGUESE = "pt"
    RUSSIAN = "ru"
    DUTCH = "nl"
    SWEDISH = "sv"
    NORWEGIAN = "no"
    DANISH = "da"
    FINNISH = "fi"
    POLISH = "pl"
    CZECH = "cs"
    HUNGARIAN = "hu"
    ROMANIAN = "ro"
    BULGARIAN = "bg"
    GREEK = "el"
    TURKISH = "tr"
    CATALAN = "ca"
    BASQUE = "eu"
    GALICIAN = "gl"
    
    # RTL Languages
    ARABIC = "ar"
    HEBREW = "he"
    PERSIAN = "fa"
    URDU = "ur"
    KURDISH = "ku"
    SINDHI = "sd"
    PASHTO = "ps"
    DARI = "prs"
    UZBEK_CYRILLIC = "uz-cyrl"
    
    # Asian Languages
    JAPANESE = "ja"
    CHINESE_SIMPLIFIED = "zh-cn"
    CHINESE_TRADITIONAL = "zh-tw"
    KOREAN = "ko"
    THAI = "th"
    VIETNAMESE = "vi"
    INDONESIAN = "id"
    MALAY = "ms"
    FILIPINO = "tl"
    HINDI = "hi"
    BENGALI = "bn"
    TAMIL = "ta"
    TELUGU = "te"
    MARATHI = "mr"
    GUJARATI = "gu"
    KANNADA = "kn"
    MALAYALAM = "ml"
    PUNJABI = "pa"
    NEPALI = "ne"
    SINHALA = "si"
    BURMESE = "my"
    KHMER = "km"
    LAO = "lo"
    MONGOLIAN = "mn"
    TIBETAN = "bo"


class LanguageConfig(BaseModel):
    """Configuration for language-specific generation."""
    
    language: Language = Field(..., description="Target language")
    cultural_context: Dict[str, Any] = Field(default_factory=dict, description="Cultural context settings")
    formality_level: str = Field(default="neutral", description="Formality level (formal, neutral, informal)")
    vocabulary_complexity: str = Field(default="medium", description="Vocabulary complexity level")
    regional_variant: Optional[str] = Field(None, description="Regional language variant")
    
    # Language-specific settings
    use_honorifics: bool = Field(default=False, description="Use honorifics in conversation")
    politeness_level: str = Field(default="standard", description="Politeness level")
    conversation_style: str = Field(default="natural", description="Conversation style")
    
    # RTL/LTR support
    text_direction: str = Field(default="ltr", description="Text direction (ltr, rtl)")
    use_arabic_numerals: bool = Field(default=True, description="Use Arabic numerals (0-9) or native numerals")
    use_native_punctuation: bool = Field(default=True, description="Use native punctuation marks")
    
    class Config:
        """Pydantic configuration."""
        use_enum_values = True
    
    def to_dict(self) -> Dict[str, Any]:
        """Convert config to dictionary."""
        return self.dict()
    
    @classmethod
    def get_default_config(cls, language: Language) -> 'LanguageConfig':
        """Get default configuration for a language."""
        defaults = {
            # LTR Languages
            Language.ENGLISH: {
                "formality_level": "neutral",
                "vocabulary_complexity": "medium",
                "use_honorifics": False,
                "politeness_level": "standard",
                "conversation_style": "natural",
                "text_direction": "ltr",
                "use_arabic_numerals": True,
                "use_native_punctuation": True
            },
            Language.SPANISH: {
                "formality_level": "neutral",
                "vocabulary_complexity": "medium",
                "use_honorifics": False,
                "politeness_level": "standard",
                "conversation_style": "natural",
                "text_direction": "ltr",
                "use_arabic_numerals": True,
                "use_native_punctuation": True
            },
            Language.FRENCH: {
                "formality_level": "formal",
                "vocabulary_complexity": "medium",
                "use_honorifics": False,
                "politeness_level": "polite",
                "conversation_style": "natural",
                "text_direction": "ltr",
                "use_arabic_numerals": True,
                "use_native_punctuation": True
            },
            Language.GERMAN: {
                "formality_level": "formal",
                "vocabulary_complexity": "medium",
                "use_honorifics": True,
                "politeness_level": "polite",
                "conversation_style": "natural",
                "text_direction": "ltr",
                "use_arabic_numerals": True,
                "use_native_punctuation": True
            },
            Language.ITALIAN: {
                "formality_level": "neutral",
                "vocabulary_complexity": "medium",
                "use_honorifics": False,
                "politeness_level": "standard",
                "conversation_style": "natural",
                "text_direction": "ltr",
                "use_arabic_numerals": True,
                "use_native_punctuation": True
            },
            Language.PORTUGUESE: {
                "formality_level": "neutral",
                "vocabulary_complexity": "medium",
                "use_honorifics": False,
                "politeness_level": "standard",
                "conversation_style": "natural",
                "text_direction": "ltr",
                "use_arabic_numerals": True,
                "use_native_punctuation": True
            },
            Language.RUSSIAN: {
                "formality_level": "formal",
                "vocabulary_complexity": "medium",
                "use_honorifics": True,
                "politeness_level": "polite",
                "conversation_style": "natural",
                "text_direction": "ltr",
                "use_arabic_numerals": True,
                "use_native_punctuation": True
            },
            
            # RTL Languages
            Language.ARABIC: {
                "formality_level": "formal",
                "vocabulary_complexity": "medium",
                "use_honorifics": False,
                "politeness_level": "polite",
                "conversation_style": "respectful",
                "text_direction": "rtl",
                "use_arabic_numerals": False,
                "use_native_punctuation": True
            },
            Language.HEBREW: {
                "formality_level": "formal",
                "vocabulary_complexity": "medium",
                "use_honorifics": False,
                "politeness_level": "polite",
                "conversation_style": "respectful",
                "text_direction": "rtl",
                "use_arabic_numerals": True,
                "use_native_punctuation": True
            },
            Language.PERSIAN: {
                "formality_level": "formal",
                "vocabulary_complexity": "medium",
                "use_honorifics": False,
                "politeness_level": "polite",
                "conversation_style": "respectful",
                "text_direction": "rtl",
                "use_arabic_numerals": False,
                "use_native_punctuation": True
            },
            Language.URDU: {
                "formality_level": "formal",
                "vocabulary_complexity": "medium",
                "use_honorifics": False,
                "politeness_level": "polite",
                "conversation_style": "respectful",
                "text_direction": "rtl",
                "use_arabic_numerals": False,
                "use_native_punctuation": True
            },
            
            # Asian Languages
            Language.JAPANESE: {
                "formality_level": "formal",
                "vocabulary_complexity": "medium",
                "use_honorifics": True,
                "politeness_level": "polite",
                "conversation_style": "respectful",
                "text_direction": "ltr",
                "use_arabic_numerals": True,
                "use_native_punctuation": True
            },
            Language.KOREAN: {
                "formality_level": "formal",
                "vocabulary_complexity": "medium",
                "use_honorifics": True,
                "politeness_level": "polite",
                "conversation_style": "respectful",
                "text_direction": "ltr",
                "use_arabic_numerals": True,
                "use_native_punctuation": True
            },
            Language.CHINESE_SIMPLIFIED: {
                "formality_level": "formal",
                "vocabulary_complexity": "medium",
                "use_honorifics": False,
                "politeness_level": "polite",
                "conversation_style": "respectful",
                "text_direction": "ltr",
                "use_arabic_numerals": True,
                "use_native_punctuation": True
            },
            Language.CHINESE_TRADITIONAL: {
                "formality_level": "formal",
                "vocabulary_complexity": "medium",
                "use_honorifics": False,
                "politeness_level": "polite",
                "conversation_style": "respectful",
                "text_direction": "ltr",
                "use_arabic_numerals": True,
                "use_native_punctuation": True
            },
            Language.THAI: {
                "formality_level": "formal",
                "vocabulary_complexity": "medium",
                "use_honorifics": True,
                "politeness_level": "polite",
                "conversation_style": "respectful",
                "text_direction": "ltr",
                "use_arabic_numerals": True,
                "use_native_punctuation": True
            },
            Language.VIETNAMESE: {
                "formality_level": "formal",
                "vocabulary_complexity": "medium",
                "use_honorifics": True,
                "politeness_level": "polite",
                "conversation_style": "respectful",
                "text_direction": "ltr",
                "use_arabic_numerals": True,
                "use_native_punctuation": True
            },
            Language.HINDI: {
                "formality_level": "formal",
                "vocabulary_complexity": "medium",
                "use_honorifics": True,
                "politeness_level": "polite",
                "conversation_style": "respectful",
                "text_direction": "ltr",
                "use_arabic_numerals": False,
                "use_native_punctuation": True
            }
        }
        
        config_data = defaults.get(language, defaults[Language.ENGLISH])
        return cls(language=language, **config_data)


class LanguageUtils:
    """Utility functions for language processing."""
    
    # RTL language codes
    RTL_LANGUAGES = {
        "ar", "he", "fa", "ur", "ku", "sd", "ps", "prs", "uz-cyrl"
    }
    
    # Native numeral systems
    NATIVE_NUMERALS = {
        "ar": "٠١٢٣٤٥٦٧٨٩",  # Arabic numerals
        "fa": "٠١٢٣٤٥٦٧٨٩",  # Persian numerals
        "ur": "٠١٢٣٤٥٦٧٨٩",  # Urdu numerals
        "hi": "०१२३४५६७८९",  # Devanagari numerals
        "bn": "০১২৩৪৫৬৭৮৯",  # Bengali numerals
        "ta": "௦௧௨௩௪௫௬௭௮௯",  # Tamil numerals
        "te": "౦౧౨౩౪౫౬౭౮౯",  # Telugu numerals
        "mr": "०१२३४५६७८९",  # Marathi numerals
        "gu": "૦૧૨૩૪૫૬૭૮૯",  # Gujarati numerals
        "kn": "೦೧೨೩೪೫೬೭೮೯",  # Kannada numerals
        "ml": "൦൧൨൩൪൫൬൭൮൯",  # Malayalam numerals
        "pa": "੦੧੨੩੪੫੬੭੮੯",  # Punjabi numerals
        "ne": "०१२३४५६७८९",  # Nepali numerals
        "si": "෦෧෨෩෪෫෬෭෮෯",  # Sinhala numerals
        "my": "၀၁၂၃၄၅၆၇၈၉",  # Burmese numerals
        "km": "០១២៣៤៥៦៧៨៩",  # Khmer numerals
        "lo": "໐໑໒໓໔໕໖໗໘໙",  # Lao numerals
        "mn": "᠐᠑᠒᠓᠔᠕᠖᠗᠘᠙",  # Mongolian numerals
        "bo": "༠༡༢༣༤༥༦༧༨༩",  # Tibetan numerals
        "th": "๐๑๒๓๔๕๖๗๘๙",  # Thai numerals
    }
    
    @classmethod
    def is_rtl(cls, language_code: str) -> bool:
        """Check if a language is RTL."""
        return language_code in cls.RTL_LANGUAGES
    
    @classmethod
    def get_text_direction(cls, language_code: str) -> str:
        """Get text direction for a language."""
        return "rtl" if cls.is_rtl(language_code) else "ltr"
    
    @classmethod
    def convert_numerals(cls, text: str, language_code: str, use_native: bool = True) -> str:
        """Convert Arabic numerals to native numerals or vice versa."""
        if not use_native or language_code not in cls.NATIVE_NUMERALS:
            return text
        
        native_numerals = cls.NATIVE_NUMERALS[language_code]
        arabic_numerals = "0123456789"
        
        # Convert Arabic numerals to native numerals
        for i, numeral in enumerate(arabic_numerals):
            text = text.replace(numeral, native_numerals[i])
        
        return text
    
    @classmethod
    def format_text_for_language(cls, text: str, language_code: str, config: LanguageConfig) -> str:
        """Format text according to language-specific rules."""
        # Convert numerals if needed
        if config.use_arabic_numerals:
            # Convert native numerals to Arabic numerals
            if language_code in cls.NATIVE_NUMERALS:
                native_numerals = cls.NATIVE_NUMERALS[language_code]
                arabic_numerals = "0123456789"
                for i, numeral in enumerate(native_numerals):
                    text = text.replace(numeral, arabic_numerals[i])
        else:
            # Convert Arabic numerals to native numerals
            text = cls.convert_numerals(text, language_code, use_native=True)
        
        # Add RTL/LTR markers if needed
        if cls.is_rtl(language_code):
            # Add RTL mark at the beginning
            text = "\u202B" + text + "\u202C"
        
        return text
    
    @classmethod
    def get_language_name(cls, language_code: str) -> str:
        """Get human-readable language name."""
        language_names = {
            # LTR Languages
            "en": "English",
            "es": "Spanish",
            "fr": "French",
            "de": "German",
            "it": "Italian",
            "pt": "Portuguese",
            "ru": "Russian",
            "nl": "Dutch",
            "sv": "Swedish",
            "no": "Norwegian",
            "da": "Danish",
            "fi": "Finnish",
            "pl": "Polish",
            "cs": "Czech",
            "hu": "Hungarian",
            "ro": "Romanian",
            "bg": "Bulgarian",
            "el": "Greek",
            "tr": "Turkish",
            "ca": "Catalan",
            "eu": "Basque",
            "gl": "Galician",
            
            # RTL Languages
            "ar": "Arabic",
            "he": "Hebrew",
            "fa": "Persian",
            "ur": "Urdu",
            "ku": "Kurdish",
            "sd": "Sindhi",
            "ps": "Pashto",
            "prs": "Dari",
            "uz-cyrl": "Uzbek (Cyrillic)",
            
            # Asian Languages
            "ja": "Japanese",
            "zh-cn": "Chinese (Simplified)",
            "zh-tw": "Chinese (Traditional)",
            "ko": "Korean",
            "th": "Thai",
            "vi": "Vietnamese",
            "id": "Indonesian",
            "ms": "Malay",
            "tl": "Filipino",
            "hi": "Hindi",
            "bn": "Bengali",
            "ta": "Tamil",
            "te": "Telugu",
            "mr": "Marathi",
            "gu": "Gujarati",
            "kn": "Kannada",
            "ml": "Malayalam",
            "pa": "Punjabi",
            "ne": "Nepali",
            "si": "Sinhala",
            "my": "Burmese",
            "km": "Khmer",
            "lo": "Lao",
            "mn": "Mongolian",
            "bo": "Tibetan"
        }
        
        return language_names.get(language_code, language_code.upper())
    
    @classmethod
    def get_language_family(cls, language_code: str) -> str:
        """Get language family."""
        families = {
            # Indo-European
            "en": "Indo-European (Germanic)",
            "de": "Indo-European (Germanic)",
            "nl": "Indo-European (Germanic)",
            "sv": "Indo-European (Germanic)",
            "no": "Indo-European (Germanic)",
            "da": "Indo-European (Germanic)",
            "es": "Indo-European (Romance)",
            "fr": "Indo-European (Romance)",
            "it": "Indo-European (Romance)",
            "pt": "Indo-European (Romance)",
            "ro": "Indo-European (Romance)",
            "ca": "Indo-European (Romance)",
            "ru": "Indo-European (Slavic)",
            "pl": "Indo-European (Slavic)",
            "cs": "Indo-European (Slavic)",
            "bg": "Indo-European (Slavic)",
            "hi": "Indo-European (Indo-Aryan)",
            "bn": "Indo-European (Indo-Aryan)",
            "ta": "Dravidian",
            "te": "Dravidian",
            "mr": "Indo-European (Indo-Aryan)",
            "gu": "Indo-European (Indo-Aryan)",
            "kn": "Dravidian",
            "ml": "Dravidian",
            "pa": "Indo-European (Indo-Aryan)",
            "ne": "Indo-European (Indo-Aryan)",
            "ur": "Indo-European (Indo-Aryan)",
            "fa": "Indo-European (Iranian)",
            "ku": "Indo-European (Iranian)",
            "ps": "Indo-European (Iranian)",
            "prs": "Indo-European (Iranian)",
            
            # Semitic
            "ar": "Afro-Asiatic (Semitic)",
            "he": "Afro-Asiatic (Semitic)",
            
            # Sino-Tibetan
            "zh-cn": "Sino-Tibetan (Sinitic)",
            "zh-tw": "Sino-Tibetan (Sinitic)",
            "bo": "Sino-Tibetan (Tibetic)",
            
            # Japonic
            "ja": "Japonic",
            
            # Koreanic
            "ko": "Koreanic",
            
            # Austroasiatic
            "vi": "Austroasiatic",
            "km": "Austroasiatic",
            
            # Tai-Kadai
            "th": "Tai-Kadai",
            "lo": "Tai-Kadai",
            
            # Austronesian
            "id": "Austronesian",
            "ms": "Austronesian",
            "tl": "Austronesian",
            
            # Uralic
            "fi": "Uralic",
            "hu": "Uralic",
            
            # Turkic
            "tr": "Turkic",
            "uz-cyrl": "Turkic",
            
            # Mongolian
            "mn": "Mongolic",
            
            # Others
            "el": "Indo-European (Hellenic)",
            "si": "Indo-European (Indo-Aryan)",
            "my": "Sino-Tibetan (Tibeto-Burman)",
            "sd": "Indo-European (Indo-Aryan)",
        }
        
        return families.get(language_code, "Unknown") 