"""
Persona data models and enums.
"""

from enum import Enum
from typing import List, Optional, Dict, Any
from pydantic import BaseModel, Field, validator
import uuid


class ExpertiseLevel(str, Enum):
    """Expertise levels for personas."""
    NOVICE = "novice"
    INTERMEDIATE = "intermediate"
    ADVANCED = "advanced"
    EXPERT = "expert"


class CommunicationStyle(str, Enum):
    """Communication style preferences."""
    FORMAL = "formal"
    CASUAL = "casual"
    TECHNICAL = "technical"
    FRIENDLY = "friendly"
    BUSINESS = "business"
    ACADEMIC = "academic"


class Domain(str, Enum):
    """Primary domains of expertise."""
    TECHNOLOGY = "technology"
    HEALTHCARE = "healthcare"
    EDUCATION = "education"
    BUSINESS = "business"
    CREATIVE = "creative"
    SPECIALIZED = "specialized"


class Persona(BaseModel):
    """Complete persona definition with all attributes."""
    
    # Identity
    id: str = Field(default_factory=lambda: str(uuid.uuid4()))
    name: str = Field(..., description="Persona's name")
    role: str = Field(..., description="Professional role or title")
    background: str = Field(..., description="Brief background description")
    experience_level: int = Field(..., ge=0, le=50, description="Years of experience")
    
    # Domain and Expertise
    domain: Domain = Field(..., description="Primary domain of expertise")
    expertise_level: ExpertiseLevel = Field(..., description="Expertise level")
    domain_subcategories: List[str] = Field(default_factory=list, description="Subcategories within domain")
    knowledge_depth: Dict[str, int] = Field(default_factory=dict, description="Knowledge depth by topic (1-10)")
    
    # Communication Style
    tone: CommunicationStyle = Field(..., description="Primary communication tone")
    vocabulary_patterns: List[str] = Field(default_factory=list, description="Characteristic vocabulary patterns")
    formality_level: int = Field(..., ge=1, le=5, description="Formality level (1=very casual, 5=very formal)")
    language_preference: str = Field(default="en", description="Preferred language code")
    
    # Behavioral Traits
    interaction_preferences: List[str] = Field(default_factory=list, description="How they prefer to interact")
    question_styles: List[str] = Field(default_factory=list, description="Types of questions they ask")
    goal_orientation: List[str] = Field(default_factory=list, description="What they typically want to achieve")
    response_style: str = Field(default="detailed", description="How they typically respond")
    
    # Context Patterns
    context_patterns: Dict[str, List[str]] = Field(default_factory=dict, description="How they respond to different contexts")
    common_questions: List[str] = Field(default_factory=list, description="Frequently asked questions")
    follow_up_patterns: List[str] = Field(default_factory=list, description="How they follow up on responses")
    
    # Additional Attributes
    typical_goals: List[str] = Field(default_factory=list, description="What they typically want to achieve")
    cultural_context: Optional[str] = Field(None, description="Cultural background affecting communication")
    communication_preferences: List[str] = Field(default_factory=list, description="Communication preferences")
    pain_points: List[str] = Field(default_factory=list, description="Common frustrations or challenges")
    motivations: List[str] = Field(default_factory=list, description="What drives their engagement")
    
    # Metadata
    created_at: Optional[str] = Field(None, description="Creation timestamp")
    updated_at: Optional[str] = Field(None, description="Last update timestamp")
    tags: List[str] = Field(default_factory=list, description="Tags for categorization")
    is_custom: bool = Field(default=False, description="Whether this is a custom persona")
    
    class Config:
        """Pydantic configuration."""
        use_enum_values = True
        validate_assignment = True
        
    @validator('formality_level')
    def validate_formality_level(cls, v):
        """Validate formality level is within range."""
        if not 1 <= v <= 5:
            raise ValueError('Formality level must be between 1 and 5')
        return v
    
    @validator('experience_level')
    def validate_experience_level(cls, v):
        """Validate experience level is reasonable."""
        if v < 0 or v > 50:
            raise ValueError('Experience level must be between 0 and 50 years')
        return v
    
    @validator('language_preference')
    def validate_language_preference(cls, v):
        """Validate language preference is a valid language code."""
        valid_languages = ['en', 'es', 'fr', 'de', 'it', 'pt', 'ja', 'zh', 'ko', 'ar']
        if v not in valid_languages:
            raise ValueError(f'Language preference must be one of: {valid_languages}')
        return v
    
    def get_expertise_score(self) -> float:
        """Calculate overall expertise score based on level and experience."""
        level_scores = {
            ExpertiseLevel.NOVICE: 1.0,
            ExpertiseLevel.INTERMEDIATE: 2.5,
            ExpertiseLevel.ADVANCED: 4.0,
            ExpertiseLevel.EXPERT: 5.0
        }
        base_score = level_scores[self.expertise_level]
        experience_bonus = min(self.experience_level / 10.0, 2.0)  # Max 2.0 bonus
        return min(base_score + experience_bonus, 5.0)
    
    def get_complexity_preference(self) -> str:
        """Get preferred complexity level based on expertise."""
        expertise_score = self.get_expertise_score()
        if expertise_score < 2.0:
            return "basic"
        elif expertise_score < 3.5:
            return "intermediate"
        else:
            return "advanced"
    
    def get_question_style(self) -> str:
        """Get primary question style based on expertise and personality."""
        if self.expertise_level == ExpertiseLevel.NOVICE:
            return "learning"
        elif self.expertise_level == ExpertiseLevel.EXPERT:
            return "challenging"
        else:
            return "clarifying"
    
    def to_dict(self) -> Dict[str, Any]:
        """Convert persona to dictionary for serialization."""
        return self.dict()
    
    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> 'Persona':
        """Create persona from dictionary."""
        return cls(**data) 