"""
Cultural adaptation functionality for personas.
"""

from typing import Dict, Any, List, Optional
from ..models.persona import Persona
from ..utils.logging_utils import get_logger

logger = get_logger(__name__)


class CulturalAdapter:
    """Adapt personas for different cultural contexts."""
    
    def __init__(self):
        """Initialize the cultural adapter."""
        self._load_cultural_mappings()
    
    def _load_cultural_mappings(self):
        """Load cultural adaptation mappings."""
        self.cultural_mappings = {
            "en": {
                "communication_style": {
                    "formal": "professional",
                    "casual": "friendly",
                    "technical": "technical"
                },
                "honorifics": False,
                "politeness_level": "standard"
            },
            "ja": {
                "communication_style": {
                    "formal": "respectful",
                    "casual": "friendly",
                    "technical": "technical"
                },
                "honorifics": True,
                "politeness_level": "polite"
            },
            "ko": {
                "communication_style": {
                    "formal": "respectful",
                    "casual": "friendly",
                    "technical": "technical"
                },
                "honorifics": True,
                "politeness_level": "polite"
            },
            "ar": {
                "communication_style": {
                    "formal": "respectful",
                    "casual": "friendly",
                    "technical": "technical"
                },
                "honorifics": False,
                "politeness_level": "polite"
            },
            "es": {
                "communication_style": {
                    "formal": "professional",
                    "casual": "friendly",
                    "technical": "technical"
                },
                "honorifics": False,
                "politeness_level": "standard"
            },
            "fr": {
                "communication_style": {
                    "formal": "professional",
                    "casual": "friendly",
                    "technical": "technical"
                },
                "honorifics": False,
                "politeness_level": "standard"
            },
            "de": {
                "communication_style": {
                    "formal": "professional",
                    "casual": "friendly",
                    "technical": "technical"
                },
                "honorifics": False,
                "politeness_level": "standard"
            }
        }
    
    def adapt_persona_for_culture(
        self, 
        persona: Persona, 
        target_language: str
    ) -> Persona:
        """Adapt a persona for a specific cultural context."""
        if target_language not in self.cultural_mappings:
            logger.warning(f"No cultural mapping for language: {target_language}")
            return persona
        
        cultural_config = self.cultural_mappings[target_language]
        
        # Create adapted persona
        adapted_persona = Persona(
            name=persona.name,
            role=persona.role,
            domain=persona.domain,
            expertise_level=persona.expertise_level,
            communication_style=self._adapt_communication_style(
                persona.communication_style, 
                cultural_config
            ),
            behavioral_traits=self._adapt_behavioral_traits(
                persona.behavioral_traits, 
                target_language
            ),
            context_patterns=self._adapt_context_patterns(
                persona.context_patterns, 
                target_language
            )
        )
        
        return adapted_persona
    
    def _adapt_communication_style(
        self, 
        style: str, 
        cultural_config: Dict[str, Any]
    ) -> str:
        """Adapt communication style for cultural context."""
        style_mapping = cultural_config.get("communication_style", {})
        return style_mapping.get(style, style)
    
    def _adapt_behavioral_traits(
        self, 
        traits: List[str], 
        target_language: str
    ) -> List[str]:
        """Adapt behavioral traits for cultural context."""
        adapted_traits = traits.copy()
        
        # Add culture-specific traits
        if target_language in ["ja", "ko"]:
            if "respectful" not in adapted_traits:
                adapted_traits.append("respectful")
        
        if target_language == "ar":
            if "hospitable" not in adapted_traits:
                adapted_traits.append("hospitable")
        
        return adapted_traits
    
    def _adapt_context_patterns(
        self, 
        patterns: List[str], 
        target_language: str
    ) -> List[str]:
        """Adapt context patterns for cultural context."""
        adapted_patterns = patterns.copy()
        
        # Add culture-specific patterns
        if target_language in ["ja", "ko"]:
            if "honorific_acknowledgment" not in adapted_patterns:
                adapted_patterns.append("honorific_acknowledgment")
        
        return adapted_patterns
    
    def get_cultural_context(self, language: str) -> Dict[str, Any]:
        """Get cultural context for a language."""
        return self.cultural_mappings.get(language, {})
    
    def is_honorific_language(self, language: str) -> bool:
        """Check if a language uses honorifics."""
        cultural_config = self.cultural_mappings.get(language, {})
        return cultural_config.get("honorifics", False)
    
    def get_politeness_level(self, language: str) -> str:
        """Get the politeness level for a language."""
        cultural_config = self.cultural_mappings.get(language, {})
        return cultural_config.get("politeness_level", "standard") 