"""
Custom persona handling functionality.
"""

import json
import yaml
from pathlib import Path
from typing import Dict, Any, Optional, Union
from ..models.persona import Persona
from ..utils.logging_utils import get_logger

logger = get_logger(__name__)


class CustomPersonaHandler:
    """Handle custom persona creation and management."""
    
    def __init__(self):
        """Initialize the custom persona handler."""
        pass
    
    def create_persona_from_dict(self, data: Dict[str, Any]) -> Optional[Persona]:
        """Create a persona from a dictionary."""
        try:
            return Persona(**data)
        except Exception as e:
            logger.error(f"Error creating persona from dict: {e}")
            return None
    
    def create_persona_from_json(self, json_data: str) -> Optional[Persona]:
        """Create a persona from JSON string."""
        try:
            data = json.loads(json_data)
            return self.create_persona_from_dict(data)
        except Exception as e:
            logger.error(f"Error creating persona from JSON: {e}")
            return None
    
    def create_persona_from_yaml(self, yaml_data: str) -> Optional[Persona]:
        """Create a persona from YAML string."""
        try:
            data = yaml.safe_load(yaml_data)
            return self.create_persona_from_dict(data)
        except Exception as e:
            logger.error(f"Error creating persona from YAML: {e}")
            return None
    
    def create_persona_from_file(self, file_path: str) -> Optional[Persona]:
        """Create a persona from a file."""
        try:
            file_path = Path(file_path)
            if not file_path.exists():
                logger.error(f"File not found: {file_path}")
                return None
            
            with open(file_path, 'r', encoding='utf-8') as f:
                content = f.read()
            
            if file_path.suffix.lower() == '.json':
                return self.create_persona_from_json(content)
            elif file_path.suffix.lower() in ['.yml', '.yaml']:
                return self.create_persona_from_yaml(content)
            else:
                logger.error(f"Unsupported file format: {file_path.suffix}")
                return None
                
        except Exception as e:
            logger.error(f"Error creating persona from file {file_path}: {e}")
            return None
    
    def save_persona_to_file(
        self, 
        persona: Persona, 
        file_path: str, 
        format: str = "json"
    ) -> bool:
        """Save a persona to a file."""
        try:
            file_path = Path(file_path)
            file_path.parent.mkdir(parents=True, exist_ok=True)
            
            data = persona.dict()
            
            with open(file_path, 'w', encoding='utf-8') as f:
                if format.lower() == "json":
                    json.dump(data, f, indent=2, ensure_ascii=False)
                elif format.lower() in ["yml", "yaml"]:
                    yaml.dump(data, f, default_flow_style=False, allow_unicode=True)
                else:
                    logger.error(f"Unsupported format: {format}")
                    return False
            
            logger.info(f"Persona saved to {file_path}")
            return True
            
        except Exception as e:
            logger.error(f"Error saving persona to {file_path}: {e}")
            return False
    
    def validate_persona_data(self, data: Dict[str, Any]) -> Dict[str, Any]:
        """Validate persona data and return validation results."""
        results = {
            "is_valid": True,
            "errors": [],
            "warnings": [],
            "missing_fields": []
        }
        
        # Check required fields
        required_fields = ["name", "role", "domain", "expertise_level"]
        for field in required_fields:
            if field not in data or not data[field]:
                results["missing_fields"].append(field)
                results["is_valid"] = False
        
        # Check optional fields
        optional_fields = ["communication_style", "behavioral_traits", "context_patterns"]
        for field in optional_fields:
            if field not in data:
                results["warnings"].append(f"Optional field '{field}' not provided")
        
        # Try to create persona to catch validation errors
        try:
            Persona(**data)
        except Exception as e:
            results["errors"].append(f"Validation error: {str(e)}")
            results["is_valid"] = False
        
        return results
    
    def get_persona_template(self, domain: str = "technology") -> Dict[str, Any]:
        """Get a template for creating a persona."""
        template = {
            "name": "Example Persona",
            "role": "Software Engineer",
            "domain": domain,
            "expertise_level": "intermediate",
            "communication_style": "professional",
            "behavioral_traits": ["analytical", "detail-oriented", "collaborative"],
            "context_patterns": ["question", "clarification", "technical_explanation"],
            "description": "A brief description of the persona",
            "background": "Professional background and experience",
            "goals": ["Primary goal", "Secondary goal"],
            "constraints": ["Constraint 1", "Constraint 2"]
        }
        
        return template
    
    def merge_persona_data(
        self, 
        base_data: Dict[str, Any], 
        updates: Dict[str, Any]
    ) -> Dict[str, Any]:
        """Merge persona data with updates."""
        merged = base_data.copy()
        
        for key, value in updates.items():
            if key in merged and isinstance(merged[key], list) and isinstance(value, list):
                # Merge lists
                merged[key] = list(set(merged[key] + value))
            else:
                # Overwrite with new value
                merged[key] = value
        
        return merged 