"""
Persona generation functionality.
"""

import random
from typing import List, Dict, Any, Optional
from ..models.persona import Persona, ExpertiseLevel, CommunicationStyle, Domain
from ..utils.logging_utils import get_logger

logger = get_logger(__name__)


class PersonaGenerator:
    """Generate personas with various characteristics."""
    
    def __init__(self):
        """Initialize the persona generator."""
        self._load_templates()
    
    def _load_templates(self):
        """Load persona templates and components."""
        self.roles = {
            Domain.TECHNOLOGY: [
                "Software Engineer", "Data Scientist", "DevOps Engineer", "Product Manager",
                "UX Designer", "System Administrator", "QA Engineer", "Frontend Developer",
                "Backend Developer", "Full Stack Developer", "Machine Learning Engineer",
                "Cybersecurity Analyst", "Cloud Architect", "Database Administrator"
            ],
            Domain.HEALTHCARE: [
                "Doctor", "Nurse", "Pharmacist", "Medical Researcher", "Healthcare Administrator",
                "Physical Therapist", "Mental Health Professional", "Medical Technologist",
                "Public Health Specialist", "Health Policy Analyst", "Medical Device Engineer"
            ],
            Domain.EDUCATION: [
                "Teacher", "Professor", "Educational Administrator", "Curriculum Developer",
                "Student", "Educational Researcher", "Librarian", "Tutor", "Academic Advisor",
                "Education Technology Specialist", "School Counselor"
            ],
            Domain.BUSINESS: [
                "Business Analyst", "Marketing Manager", "Sales Representative", "HR Manager",
                "Financial Analyst", "Project Manager", "Consultant", "Entrepreneur",
                "Operations Manager", "Strategy Consultant", "Business Development Manager"
            ],
            Domain.CREATIVE: [
                "Graphic Designer", "Content Writer", "Video Editor", "Photographer",
                "Artist", "Musician", "Filmmaker", "Copywriter", "Creative Director",
                "UI/UX Designer", "Animator"
            ],
            Domain.SPECIALIZED: [
                "Legal Professional", "Architect", "Engineer", "Scientist", "Journalist",
                "Chef", "Fitness Trainer", "Real Estate Agent", "Travel Agent", "Translator"
            ]
        }
        
        self.communication_styles = [
            CommunicationStyle.FORMAL,
            CommunicationStyle.CASUAL,
            CommunicationStyle.TECHNICAL,
            CommunicationStyle.CONVERSATIONAL,
            CommunicationStyle.PROFESSIONAL,
            CommunicationStyle.FRIENDLY
        ]
        
        self.behavioral_traits = [
            "analytical", "creative", "detail-oriented", "big-picture", "collaborative",
            "independent", "patient", "impatient", "curious", "practical", "theoretical",
            "organized", "flexible", "assertive", "reserved", "enthusiastic", "cautious"
        ]
    
    def generate_persona(
        self,
        domain: Optional[Domain] = None,
        expertise_level: Optional[ExpertiseLevel] = None,
        communication_style: Optional[CommunicationStyle] = None,
        name: Optional[str] = None
    ) -> Persona:
        """Generate a persona with specified characteristics."""
        
        # Select random domain if not specified
        if domain is None:
            domain = random.choice(list(Domain))
        
        # Select random expertise level if not specified
        if expertise_level is None:
            expertise_level = random.choice(list(ExpertiseLevel))
        
        # Select random communication style if not specified
        if communication_style is None:
            communication_style = random.choice(self.communication_styles)
        
        # Generate name if not provided
        if name is None:
            name = self._generate_name(domain, expertise_level)
        
        # Select role based on domain
        role = random.choice(self.roles.get(domain, ["Professional"]))
        
        # Generate behavioral traits
        num_traits = random.randint(2, 4)
        traits = random.sample(self.behavioral_traits, num_traits)
        
        # Generate context patterns
        context_patterns = self._generate_context_patterns(domain, expertise_level)
        
        return Persona(
            name=name,
            role=role,
            domain=domain,
            expertise_level=expertise_level,
            communication_style=communication_style,
            behavioral_traits=traits,
            context_patterns=context_patterns
        )
    
    def _generate_name(self, domain: Domain, expertise_level: ExpertiseLevel) -> str:
        """Generate a name for the persona."""
        # Simple name generation based on domain and expertise
        names = {
            Domain.TECHNOLOGY: ["Alex", "Sam", "Jordan", "Casey", "Taylor"],
            Domain.HEALTHCARE: ["Dr. Smith", "Dr. Johnson", "Dr. Williams", "Dr. Brown"],
            Domain.EDUCATION: ["Professor Davis", "Professor Miller", "Professor Wilson"],
            Domain.BUSINESS: ["Michael", "Sarah", "David", "Emily", "James"],
            Domain.CREATIVE: ["Aria", "Luna", "Phoenix", "River", "Sky"],
            Domain.SPECIALIZED: ["Morgan", "Riley", "Quinn", "Avery", "Blake"]
        }
        
        domain_names = names.get(domain, ["Professional"])
        return random.choice(domain_names)
    
    def _generate_context_patterns(
        self, 
        domain: Domain, 
        expertise_level: ExpertiseLevel
    ) -> List[str]:
        """Generate context patterns based on domain and expertise."""
        patterns = []
        
        # Base patterns for all domains
        patterns.extend(["question", "clarification"])
        
        # Domain-specific patterns
        if domain == Domain.TECHNOLOGY:
            patterns.extend(["code_example", "technical_explanation"])
        elif domain == Domain.HEALTHCARE:
            patterns.extend(["medical_advice", "health_guidance"])
        elif domain == Domain.EDUCATION:
            patterns.extend(["educational_explanation", "learning_guidance"])
        elif domain == Domain.BUSINESS:
            patterns.extend(["business_analysis", "strategic_advice"])
        elif domain == Domain.CREATIVE:
            patterns.extend(["creative_suggestion", "artistic_guidance"])
        
        # Expertise-specific patterns
        if expertise_level in [ExpertiseLevel.NOVICE, ExpertiseLevel.INTERMEDIATE]:
            patterns.extend(["basic_explanation", "step_by_step"])
        elif expertise_level in [ExpertiseLevel.ADVANCED, ExpertiseLevel.EXPERT]:
            patterns.extend(["advanced_concept", "detailed_analysis"])
        
        return list(set(patterns))  # Remove duplicates
    
    def generate_persona_batch(
        self, 
        count: int, 
        domain: Optional[Domain] = None,
        expertise_level: Optional[ExpertiseLevel] = None
    ) -> List[Persona]:
        """Generate multiple personas."""
        personas = []
        
        for i in range(count):
            persona = self.generate_persona(domain, expertise_level)
            personas.append(persona)
        
        return personas 