"""
Persona loading functionality.
"""

import json
import yaml
from pathlib import Path
from typing import Dict, Any, List, Optional
from ..models.persona import Persona
from ..utils.logging_utils import get_logger

logger = get_logger(__name__)


class PersonaLoader:
    """Load personas from various file formats."""
    
    def __init__(self, personas_dir: str = "./data/personas"):
        """Initialize the persona loader."""
        self.personas_dir = Path(personas_dir)
        self.personas_dir.mkdir(parents=True, exist_ok=True)
    
    def load_persona_from_file(self, file_path: str) -> Optional[Persona]:
        """Load a single persona from a file."""
        try:
            file_path = Path(file_path)
            if not file_path.exists():
                logger.error(f"Persona file not found: {file_path}")
                return None
            
            with open(file_path, 'r', encoding='utf-8') as f:
                if file_path.suffix.lower() == '.json':
                    data = json.load(f)
                elif file_path.suffix.lower() in ['.yml', '.yaml']:
                    data = yaml.safe_load(f)
                else:
                    logger.error(f"Unsupported file format: {file_path.suffix}")
                    return None
            
            return Persona(**data)
            
        except Exception as e:
            logger.error(f"Error loading persona from {file_path}: {e}")
            return None
    
    def load_personas_from_directory(self, directory: Optional[str] = None) -> List[Persona]:
        """Load all personas from a directory."""
        if directory is None:
            directory = self.personas_dir
        else:
            directory = Path(directory)
        
        personas = []
        
        try:
            for file_path in directory.rglob("*.json"):
                persona = self.load_persona_from_file(str(file_path))
                if persona:
                    personas.append(persona)
            
            for file_path in directory.rglob("*.yml"):
                persona = self.load_persona_from_file(str(file_path))
                if persona:
                    personas.append(persona)
            
            for file_path in directory.rglob("*.yaml"):
                persona = self.load_persona_from_file(str(file_path))
                if persona:
                    personas.append(persona)
                    
        except Exception as e:
            logger.error(f"Error loading personas from directory {directory}: {e}")
        
        logger.info(f"Loaded {len(personas)} personas from {directory}")
        return personas
    
    def load_persona_by_name(self, name: str) -> Optional[Persona]:
        """Load a persona by name from the personas directory."""
        try:
            for file_path in self.personas_dir.rglob("*.json"):
                persona = self.load_persona_from_file(str(file_path))
                if persona and persona.name == name:
                    return persona
            
            for file_path in self.personas_dir.rglob("*.yml"):
                persona = self.load_persona_from_file(str(file_path))
                if persona and persona.name == name:
                    return persona
            
            for file_path in self.personas_dir.rglob("*.yaml"):
                persona = self.load_persona_from_file(str(file_path))
                if persona and persona.name == name:
                    return persona
                    
        except Exception as e:
            logger.error(f"Error loading persona by name {name}: {e}")
        
        return None
    
    def get_available_personas(self) -> List[str]:
        """Get list of available persona names."""
        personas = self.load_personas_from_directory()
        return [p.name for p in personas if p.name]
    
    def validate_persona_file(self, file_path: str) -> bool:
        """Validate a persona file without loading it."""
        try:
            persona = self.load_persona_from_file(file_path)
            return persona is not None
        except Exception:
            return False 