"""
Persona manager for loading and managing persona definitions.
"""

import os
import json
import random
from typing import Dict, List, Optional, Any
from pathlib import Path

from ..models.persona import Persona, Domain, ExpertiseLevel, CommunicationStyle
from ..utils.logging_utils import get_logger

logger = get_logger(__name__)


class PersonaManager:
    """Manages persona loading, caching, and selection."""
    
    def __init__(self, personas_dir: Optional[str] = None):
        """
        Initialize the persona manager.
        
        Args:
            personas_dir: Directory containing persona JSON files (optional)
        """
        self.personas_dir = personas_dir
        self.personas: Dict[str, Persona] = {}
        self.personas_by_domain: Dict[str, List[Persona]] = {}
        self.personas_by_expertise: Dict[str, List[Persona]] = {}
        self.logger = get_logger(__name__)
        
        # Load personas on initialization if directory exists
        if self.personas_dir and os.path.exists(self.personas_dir):
            self._load_personas()
        else:
            self.logger.info("No personas directory specified or directory doesn't exist. Using dynamic personas only.")
    
    def get_persona(self, name_or_id: str) -> Optional[Persona]:
        """
        Get a persona by name or ID.
        
        Args:
            name_or_id: Persona name or ID
            
        Returns:
            Persona object if found, None otherwise
        """
        # First try exact ID match
        if name_or_id in self.personas:
            return self.personas[name_or_id]
        
        # Then try name match
        for persona in self.personas.values():
            if persona.name.lower() == name_or_id.lower():
                return persona
        
        # Try partial name match
        for persona in self.personas.values():
            if name_or_id.lower() in persona.name.lower():
                return persona
        
        return None
    
    def get_persona_by_id(self, persona_id: str) -> Optional[Persona]:
        """
        Get a persona by ID.
        
        Args:
            persona_id: Persona ID
            
        Returns:
            Persona object if found, None otherwise
        """
        return self.personas.get(persona_id)
    
    def get_random_persona(self, domain: Optional[str] = None, expertise: Optional[str] = None) -> Optional[Persona]:
        """
        Get a random persona, optionally filtered by domain and expertise.
        
        Args:
            domain: Filter by domain
            expertise: Filter by expertise level
            
        Returns:
            Random persona matching criteria, or None if no matches
        """
        candidates = self._get_candidate_personas(domain, expertise)
        
        if not candidates:
            self.logger.warning(f"No personas found for domain={domain}, expertise={expertise}")
            return None
        
        return random.choice(candidates)
    
    def get_personas_by_domain(self, domain: str) -> List[Persona]:
        """
        Get all personas in a specific domain.
        
        Args:
            domain: Domain name
            
        Returns:
            List of personas in the domain
        """
        return self.personas_by_domain.get(domain.lower(), [])
    
    def get_personas_by_expertise(self, expertise: str) -> List[Persona]:
        """
        Get all personas with a specific expertise level.
        
        Args:
            expertise: Expertise level
            
        Returns:
            List of personas with the expertise level
        """
        return self.personas_by_expertise.get(expertise.lower(), [])
    
    def list_personas(self, domain: Optional[str] = None, expertise: Optional[str] = None) -> List[Persona]:
        """
        List all personas, optionally filtered.
        
        Args:
            domain: Filter by domain
            expertise: Filter by expertise level
            
        Returns:
            List of matching personas
        """
        return self._get_candidate_personas(domain, expertise)
    
    def add_persona(self, persona: Persona) -> None:
        """
        Add a custom persona to the manager.
        
        Args:
            persona: Persona to add
        """
        self.personas[persona.id] = persona
        self._index_persona(persona)
        self.logger.info(f"Added custom persona: {persona.name}")
    
    def create_dynamic_persona(
        self,
        name: str,
        role: str,
        domain: str,
        expertise_level: str,
        background: str = "",
        communication_style: str = "formal",
        **kwargs
    ) -> Persona:
        """
        Create a dynamic persona on-the-fly without requiring a file.
        
        Args:
            name: Persona name
            role: Professional role
            domain: Domain of expertise
            expertise_level: Expertise level (novice, intermediate, advanced, expert)
            background: Background description
            communication_style: Communication style
            **kwargs: Additional persona attributes
            
        Returns:
            Created Persona object
        """
        # Validate domain
        try:
            domain_enum = Domain(domain.lower())
        except ValueError:
            domain_enum = Domain.TECHNOLOGY  # Default fallback
            self.logger.warning(f"Invalid domain '{domain}', using 'technology' as default")
        
        # Validate expertise level
        try:
            expertise_enum = ExpertiseLevel(expertise_level.lower())
        except ValueError:
            expertise_enum = ExpertiseLevel.INTERMEDIATE  # Default fallback
            self.logger.warning(f"Invalid expertise level '{expertise_level}', using 'intermediate' as default")
        
        # Create persona with default values
        persona_data = {
            "name": name,
            "role": role,
            "domain": domain_enum,
            "expertise_level": expertise_enum,
            "background": background or f"{name} is a {role} with {expertise_level} expertise in {domain}.",
            "experience_level": kwargs.get("experience_level", 5),
            "tone": CommunicationStyle(kwargs.get("communication_style", communication_style) or "formal"),
            "formality_level": kwargs.get("formality_level", 3),
            "language_preference": kwargs.get("language_preference", "en"),
            "interaction_preferences": kwargs.get("interaction_preferences", ["collaborative", "detail-oriented"]),
            "question_styles": kwargs.get("question_styles", ["clarifying", "analytical"]),
            "goal_orientation": kwargs.get("goal_orientation", ["problem-solving", "learning"]),
            "response_style": kwargs.get("response_style", "detailed"),
            "context_patterns": kwargs.get("context_patterns", {
                "question": ["asks for clarification", "seeks examples"],
                "explanation": ["provides detailed responses", "offers practical advice"]
            }),
            "common_questions": kwargs.get("common_questions", [
                f"What are the best practices for {domain}?",
                f"How can I improve my {role} skills?",
                f"What are the latest trends in {domain}?"
            ]),
            "follow_up_patterns": kwargs.get("follow_up_patterns", [
                "asks for more details",
                "requests examples",
                "suggests alternatives"
            ]),
            "typical_goals": kwargs.get("typical_goals", [
                "Improve technical skills",
                "Stay updated with industry trends",
                "Solve complex problems"
            ]),
            "communication_preferences": kwargs.get("communication_preferences", [
                "Clear and concise explanations",
                "Practical examples",
                "Step-by-step guidance"
            ]),
            "pain_points": kwargs.get("pain_points", [
                "Complex technical concepts",
                "Keeping up with rapid changes",
                "Balancing depth and accessibility"
            ]),
            "motivations": kwargs.get("motivations", [
                "Continuous learning",
                "Problem solving",
                "Professional growth"
            ]),
            "tags": kwargs.get("tags", [domain, expertise_level, role]),
            "is_custom": True,
            "created_at": kwargs.get("created_at"),
            "updated_at": kwargs.get("updated_at")
        }
        
        # Create and add the persona
        persona = Persona(**persona_data)
        self.add_persona(persona)
        
        return persona
    
    def create_persona_from_template(
        self,
        template_name: str,
        name: str,
        role: str,
        **kwargs
    ) -> Persona:
        """
        Create a persona from a predefined template.
        
        Args:
            template_name: Name of the template to use
            name: Persona name
            role: Professional role
            **kwargs: Override template values
            
        Returns:
            Created Persona object
        """
        templates = {
            "senior_developer": {
                "domain": "technology",
                "expertise_level": "expert",
                "background": f"{name} is a senior developer with extensive experience in software development.",
                "communication_style": "formal",
                "experience_level": 8,
                "formality_level": 4,
                "interaction_preferences": ["analytical", "detail-oriented", "mentoring"],
                "question_styles": ["challenging", "architectural", "best-practices"],
                "goal_orientation": ["code-quality", "system-design", "team-mentoring"],
                "response_style": "comprehensive",
                "common_questions": [
                    "What's the best architecture for this use case?",
                    "How can we optimize this code for performance?",
                    "What are the security implications of this approach?"
                ]
            },
            "junior_developer": {
                "domain": "technology",
                "expertise_level": "novice",
                "background": f"{name} is a junior developer learning the ropes of software development.",
                "communication_style": "casual",
                "experience_level": 1,
                "formality_level": 2,
                "interaction_preferences": ["learning", "collaborative", "curious"],
                "question_styles": ["learning", "clarifying", "basic"],
                "goal_orientation": ["learning", "skill-building", "understanding"],
                "response_style": "educational",
                "common_questions": [
                    "How does this work?",
                    "Can you explain this concept?",
                    "What's the difference between these approaches?"
                ]
            },
            "data_scientist": {
                "domain": "technology",
                "expertise_level": "advanced",
                "background": f"{name} is a data scientist focused on machine learning and analytics.",
                "communication_style": "academic",
                "experience_level": 6,
                "formality_level": 3,
                "interaction_preferences": ["data-driven", "analytical", "research-oriented"],
                "question_styles": ["analytical", "statistical", "methodological"],
                "goal_orientation": ["insights", "accuracy", "interpretability"],
                "response_style": "analytical",
                "common_questions": [
                    "What's the statistical significance of this result?",
                    "How can we validate this model?",
                    "What are the assumptions behind this approach?"
                ]
            },
            "product_manager": {
                "domain": "business",
                "expertise_level": "intermediate",
                "background": f"{name} is a product manager focused on user needs and business goals.",
                "communication_style": "business",
                "experience_level": 4,
                "formality_level": 3,
                "interaction_preferences": ["user-focused", "collaborative", "strategic"],
                "question_styles": ["user-needs", "business-impact", "feasibility"],
                "goal_orientation": ["user-value", "business-success", "team-alignment"],
                "response_style": "strategic",
                "common_questions": [
                    "What's the user value of this feature?",
                    "How does this align with our business goals?",
                    "What are the trade-offs we need to consider?"
                ]
            },
            "designer": {
                "domain": "creative",
                "expertise_level": "intermediate",
                "background": f"{name} is a designer focused on user experience and visual design.",
                "communication_style": "casual",
                "experience_level": 3,
                "formality_level": 2,
                "interaction_preferences": ["visual", "user-centered", "creative"],
                "question_styles": ["user-experience", "aesthetic", "usability"],
                "goal_orientation": ["user-experience", "visual-quality", "accessibility"],
                "response_style": "visual",
                "common_questions": [
                    "How does this improve the user experience?",
                    "What's the visual hierarchy here?",
                    "How accessible is this design?"
                ]
            }
        }
        
        if template_name not in templates:
            raise ValueError(f"Unknown template: {template_name}. Available templates: {list(templates.keys())}")
        
        template = templates[template_name]
        template.update(kwargs)  # Override with provided kwargs
        
        # Filter out None values from template
        filtered_template = {k: v for k, v in template.items() if v is not None}
        
        # Ensure required arguments are provided
        domain = filtered_template.get("domain", "technology")
        expertise_level = filtered_template.get("expertise_level", "intermediate")
        
        return self.create_dynamic_persona(
            name=name,
            role=role,
            domain=domain,
            expertise_level=expertise_level,
            **filtered_template
        )
    
    def remove_persona(self, persona_id: str) -> bool:
        """
        Remove a persona from the manager.
        
        Args:
            persona_id: ID of persona to remove
            
        Returns:
            True if removed, False if not found
        """
        if persona_id in self.personas:
            persona = self.personas[persona_id]
            del self.personas[persona_id]
            self._remove_from_index(persona)
            self.logger.info(f"Removed persona: {persona.name}")
            return True
        return False
    
    def reload_personas(self) -> None:
        """Reload all personas from disk."""
        self.personas.clear()
        self.personas_by_domain.clear()
        self.personas_by_expertise.clear()
        self._load_personas()
        self.logger.info("Reloaded all personas")
    
    def get_persona_stats(self) -> Dict[str, Any]:
        """
        Get statistics about loaded personas.
        
        Returns:
            Dictionary with persona statistics
        """
        stats = {
            "total_personas": len(self.personas),
            "by_domain": {},
            "by_expertise": {},
            "custom_personas": 0,
            "built_in_personas": 0
        }
        
        # Count by domain
        for domain, personas in self.personas_by_domain.items():
            stats["by_domain"][domain] = len(personas)
        
        # Count by expertise
        for expertise, personas in self.personas_by_expertise.items():
            stats["by_expertise"][expertise] = len(personas)
        
        # Count custom vs built-in
        for persona in self.personas.values():
            if persona.is_custom:
                stats["custom_personas"] += 1
            else:
                stats["built_in_personas"] += 1
        
        return stats
    
    def _load_personas(self) -> None:
        """Load all personas from the personas directory."""
        if not self.personas_dir:
            self.logger.info("No personas directory specified. Using dynamic personas only.")
            return
            
        personas_path = Path(self.personas_dir)
        
        if not personas_path.exists():
            self.logger.info(f"Personas directory not found: {self.personas_dir}. Using dynamic personas only.")
            return
        
        # Load personas from all subdirectories
        for json_file in personas_path.rglob("*.json"):
            try:
                with open(json_file, 'r', encoding='utf-8') as f:
                    persona_data = json.load(f)
                
                persona = Persona.from_dict(persona_data)
                self.personas[persona.id] = persona
                self._index_persona(persona)
                
                self.logger.debug(f"Loaded persona: {persona.name} from {json_file}")
                
            except Exception as e:
                self.logger.error(f"Failed to load persona from {json_file}: {e}")
        
        self.logger.info(f"Loaded {len(self.personas)} personas")
    
    def _index_persona(self, persona: Persona) -> None:
        """Index a persona by domain and expertise for quick lookup."""
        # Index by domain
        domain = str(persona.domain).lower()
        if domain not in self.personas_by_domain:
            self.personas_by_domain[domain] = []
        self.personas_by_domain[domain].append(persona)
        
        # Index by expertise
        expertise = str(persona.expertise_level).lower()
        if expertise not in self.personas_by_expertise:
            self.personas_by_expertise[expertise] = []
        self.personas_by_expertise[expertise].append(persona)
    
    def _remove_from_index(self, persona: Persona) -> None:
        """Remove a persona from the indexing structures."""
        # Remove from domain index
        domain = str(persona.domain).lower()
        if domain in self.personas_by_domain:
            self.personas_by_domain[domain] = [
                p for p in self.personas_by_domain[domain] if p.id != persona.id
            ]
        
        # Remove from expertise index
        expertise = str(persona.expertise_level).lower()
        if expertise in self.personas_by_expertise:
            self.personas_by_expertise[expertise] = [
                p for p in self.personas_by_expertise[expertise] if p.id != persona.id
            ]
    
    def _get_candidate_personas(
        self, 
        domain: Optional[str] = None, 
        expertise: Optional[str] = None
    ) -> List[Persona]:
        """Get candidate personas based on filters."""
        candidates = list(self.personas.values())
        
        # Filter by domain
        if domain:
            domain_lower = domain.lower()
            candidates = [p for p in candidates if str(p.domain).lower() == domain_lower]
        
        # Filter by expertise
        if expertise:
            expertise_lower = expertise.lower()
            candidates = [p for p in candidates if str(p.expertise_level).lower() == expertise_lower]
        
        return candidates 