"""
Persona mixing and combination functionality.
"""

import random
from typing import List, Dict, Any, Optional
from ..models.persona import Persona, ExpertiseLevel, CommunicationStyle, Domain
from ..utils.logging_utils import get_logger

logger = get_logger(__name__)


class PersonaMixer:
    """Mix and combine persona characteristics."""
    
    def __init__(self):
        """Initialize the persona mixer."""
        pass
    
    def mix_personas(
        self, 
        personas: List[Persona], 
        weights: Optional[List[float]] = None
    ) -> Persona:
        """Mix multiple personas into a new persona."""
        if not personas:
            raise ValueError("At least one persona is required")
        
        if len(personas) == 1:
            return personas[0]
        
        # Use equal weights if not specified
        if weights is None:
            weights = [1.0 / len(personas)] * len(personas)
        
        # Normalize weights
        total_weight = sum(weights)
        weights = [w / total_weight for w in weights]
        
        # Select base persona (highest weight)
        base_persona = personas[weights.index(max(weights))]
        
        # Mix characteristics
        mixed_persona = Persona(
            name=base_persona.name,
            role=base_persona.role,
            domain=self._mix_domains(personas, weights),
            expertise_level=self._mix_expertise_levels(personas, weights),
            communication_style=self._mix_communication_styles(personas, weights),
            behavioral_traits=self._mix_behavioral_traits(personas, weights),
            context_patterns=self._mix_context_patterns(personas, weights)
        )
        
        return mixed_persona
    
    def _mix_domains(self, personas: List[Persona], weights: List[float]) -> Domain:
        """Mix domains based on weights."""
        domain_counts = {}
        for persona, weight in zip(personas, weights):
            domain = persona.domain
            domain_counts[domain] = domain_counts.get(domain, 0) + weight
        
        # Return domain with highest weight
        return max(domain_counts.items(), key=lambda x: x[1])[0]
    
    def _mix_expertise_levels(self, personas: List[Persona], weights: List[float]) -> ExpertiseLevel:
        """Mix expertise levels based on weights."""
        expertise_levels = list(ExpertiseLevel)
        expertise_scores = [0.0] * len(expertise_levels)
        
        for persona, weight in zip(personas, weights):
            level_index = expertise_levels.index(persona.expertise_level)
            expertise_scores[level_index] += weight
        
        # Return expertise level with highest score
        return expertise_levels[expertise_scores.index(max(expertise_scores))]
    
    def _mix_communication_styles(self, personas: List[Persona], weights: List[float]) -> CommunicationStyle:
        """Mix communication styles based on weights."""
        style_counts = {}
        for persona, weight in zip(personas, weights):
            style = persona.communication_style
            style_counts[style] = style_counts.get(style, 0) + weight
        
        # Return style with highest weight
        return max(style_counts.items(), key=lambda x: x[1])[0]
    
    def _mix_behavioral_traits(self, personas: List[Persona], weights: List[float]) -> List[str]:
        """Mix behavioral traits based on weights."""
        trait_counts = {}
        
        for persona, weight in zip(personas, weights):
            for trait in persona.behavioral_traits:
                trait_counts[trait] = trait_counts.get(trait, 0) + weight
        
        # Select top traits based on weights
        sorted_traits = sorted(trait_counts.items(), key=lambda x: x[1], reverse=True)
        num_traits = min(4, len(sorted_traits))  # Limit to 4 traits
        
        return [trait for trait, _ in sorted_traits[:num_traits]]
    
    def _mix_context_patterns(self, personas: List[Persona], weights: List[float]) -> List[str]:
        """Mix context patterns based on weights."""
        pattern_counts = {}
        
        for persona, weight in zip(personas, weights):
            for pattern in persona.context_patterns:
                pattern_counts[pattern] = pattern_counts.get(pattern, 0) + weight
        
        # Select top patterns based on weights
        sorted_patterns = sorted(pattern_counts.items(), key=lambda x: x[1], reverse=True)
        num_patterns = min(6, len(sorted_patterns))  # Limit to 6 patterns
        
        return [pattern for pattern, _ in sorted_patterns[:num_patterns]]
    
    def create_persona_variation(
        self, 
        base_persona: Persona, 
        variation_strength: float = 0.3
    ) -> Persona:
        """Create a variation of a base persona."""
        if not 0.0 <= variation_strength <= 1.0:
            raise ValueError("Variation strength must be between 0.0 and 1.0")
        
        # Create a slightly modified version of the base persona
        variation = Persona(
            name=base_persona.name,
            role=base_persona.role,
            domain=base_persona.domain,
            expertise_level=base_persona.expertise_level,
            communication_style=base_persona.communication_style,
            behavioral_traits=base_persona.behavioral_traits.copy(),
            context_patterns=base_persona.context_patterns.copy()
        )
        
        # Randomly modify some characteristics based on variation strength
        if random.random() < variation_strength:
            # Modify communication style
            all_styles = list(CommunicationStyle)
            if base_persona.communication_style in all_styles:
                all_styles.remove(base_persona.communication_style)
                if all_styles:
                    variation.communication_style = random.choice(all_styles)
        
        if random.random() < variation_strength:
            # Modify behavioral traits
            if variation.behavioral_traits:
                # Remove one trait
                trait_to_remove = random.choice(variation.behavioral_traits)
                variation.behavioral_traits.remove(trait_to_remove)
        
        if random.random() < variation_strength:
            # Modify context patterns
            if variation.context_patterns:
                # Remove one pattern
                pattern_to_remove = random.choice(variation.context_patterns)
                variation.context_patterns.remove(pattern_to_remove)
        
        return variation 