"""
Persona validation functionality.
"""

from typing import List, Dict, Any, Optional
from ..models.persona import Persona
from ..utils.logging_utils import get_logger

logger = get_logger(__name__)


class PersonaValidator:
    """Validate persona data and structure."""
    
    def __init__(self):
        """Initialize the persona validator."""
        pass
    
    def validate_persona(self, persona: Persona) -> Dict[str, Any]:
        """Validate a persona and return validation results."""
        results = {
            "is_valid": True,
            "errors": [],
            "warnings": [],
            "score": 1.0
        }
        
        # Check required fields
        if not persona.name:
            results["errors"].append("Persona name is required")
            results["is_valid"] = False
        
        if not persona.role:
            results["errors"].append("Persona role is required")
            results["is_valid"] = False
        
        if not persona.domain:
            results["errors"].append("Persona domain is required")
            results["is_valid"] = False
        
        if not persona.expertise_level:
            results["errors"].append("Persona expertise level is required")
            results["is_valid"] = False
        
        # Check optional fields and provide warnings
        if not persona.communication_style:
            results["warnings"].append("Communication style not specified")
        
        if not persona.behavioral_traits:
            results["warnings"].append("Behavioral traits not specified")
        
        if not persona.context_patterns:
            results["warnings"].append("Context patterns not specified")
        
        # Calculate validation score
        total_checks = 4  # Required fields
        passed_checks = sum([
            bool(persona.name),
            bool(persona.role),
            bool(persona.domain),
            bool(persona.expertise_level)
        ])
        
        results["score"] = passed_checks / total_checks
        
        return results
    
    def validate_persona_data(self, data: Dict[str, Any]) -> Dict[str, Any]:
        """Validate persona data dictionary."""
        try:
            persona = Persona(**data)
            return self.validate_persona(persona)
        except Exception as e:
            return {
                "is_valid": False,
                "errors": [f"Invalid persona data: {str(e)}"],
                "warnings": [],
                "score": 0.0
            }
    
    def get_validation_summary(self, personas: List[Persona]) -> Dict[str, Any]:
        """Get validation summary for multiple personas."""
        results = {
            "total_personas": len(personas),
            "valid_personas": 0,
            "invalid_personas": 0,
            "average_score": 0.0,
            "common_errors": {},
            "common_warnings": {}
        }
        
        if not personas:
            return results
        
        total_score = 0.0
        all_errors = []
        all_warnings = []
        
        for persona in personas:
            validation = self.validate_persona(persona)
            total_score += validation["score"]
            
            if validation["is_valid"]:
                results["valid_personas"] += 1
            else:
                results["invalid_personas"] += 1
            
            all_errors.extend(validation["errors"])
            all_warnings.extend(validation["warnings"])
        
        results["average_score"] = total_score / len(personas)
        
        # Count common errors and warnings
        from collections import Counter
        results["common_errors"] = dict(Counter(all_errors))
        results["common_warnings"] = dict(Counter(all_warnings))
        
        return results 