"""
File utility functions for persona-chat.
"""

import os
import json
import yaml
from pathlib import Path
from typing import Dict, Any, Optional, List
from ..utils.logging_utils import get_logger

logger = get_logger(__name__)


def ensure_directory(directory_path: str) -> None:
    """
    Ensure a directory exists, creating it if necessary.
    
    Args:
        directory_path: Path to the directory
    """
    Path(directory_path).mkdir(parents=True, exist_ok=True)


def read_json_file(file_path: str) -> Dict[str, Any]:
    """
    Read and parse a JSON file.
    
    Args:
        file_path: Path to the JSON file
        
    Returns:
        Parsed JSON data
        
    Raises:
        FileNotFoundError: If file doesn't exist
        json.JSONDecodeError: If file is not valid JSON
    """
    try:
        with open(file_path, 'r', encoding='utf-8') as f:
            return json.load(f)
    except FileNotFoundError:
        logger.error(f"File not found: {file_path}")
        raise
    except json.JSONDecodeError as e:
        logger.error(f"Invalid JSON in {file_path}: {e}")
        raise


def write_json_file(file_path: str, data: Dict[str, Any], indent: int = 2) -> None:
    """
    Write data to a JSON file.
    
    Args:
        file_path: Path to the JSON file
        data: Data to write
        indent: JSON indentation level
    """
    ensure_directory(os.path.dirname(file_path))
    
    try:
        with open(file_path, 'w', encoding='utf-8') as f:
            json.dump(data, f, indent=indent, ensure_ascii=False)
        logger.debug(f"Wrote JSON file: {file_path}")
    except Exception as e:
        logger.error(f"Failed to write JSON file {file_path}: {e}")
        raise


def read_yaml_file(file_path: str) -> Dict[str, Any]:
    """
    Read and parse a YAML file.
    
    Args:
        file_path: Path to the YAML file
        
    Returns:
        Parsed YAML data
        
    Raises:
        FileNotFoundError: If file doesn't exist
        yaml.YAMLError: If file is not valid YAML
    """
    try:
        with open(file_path, 'r', encoding='utf-8') as f:
            return yaml.safe_load(f)
    except FileNotFoundError:
        logger.error(f"File not found: {file_path}")
        raise
    except yaml.YAMLError as e:
        logger.error(f"Invalid YAML in {file_path}: {e}")
        raise


def write_yaml_file(file_path: str, data: Dict[str, Any]) -> None:
    """
    Write data to a YAML file.
    
    Args:
        file_path: Path to the YAML file
        data: Data to write
    """
    ensure_directory(os.path.dirname(file_path))
    
    try:
        with open(file_path, 'w', encoding='utf-8') as f:
            yaml.dump(data, f, default_flow_style=False, allow_unicode=True)
        logger.debug(f"Wrote YAML file: {file_path}")
    except Exception as e:
        logger.error(f"Failed to write YAML file {file_path}: {e}")
        raise


def find_files(directory: str, pattern: str = "*.json") -> List[str]:
    """
    Find files matching a pattern in a directory.
    
    Args:
        directory: Directory to search
        pattern: File pattern to match
        
    Returns:
        List of matching file paths
    """
    directory_path = Path(directory)
    if not directory_path.exists():
        return []
    
    return [str(f) for f in directory_path.rglob(pattern)]


def get_file_extension(file_path: str) -> str:
    """
    Get the file extension from a file path.
    
    Args:
        file_path: Path to the file
        
    Returns:
        File extension (without the dot)
    """
    return Path(file_path).suffix[1:].lower()


def is_valid_file(file_path: str, allowed_extensions: Optional[List[str]] = None) -> bool:
    """
    Check if a file is valid based on extension and existence.
    
    Args:
        file_path: Path to the file
        allowed_extensions: List of allowed file extensions
        
    Returns:
        True if file is valid, False otherwise
    """
    if not os.path.exists(file_path):
        return False
    
    if allowed_extensions:
        extension = get_file_extension(file_path)
        return extension in allowed_extensions
    
    return True 