"""
Parallel processing utility functions for persona-chat.
"""

import multiprocessing
from typing import List, Any, Callable, Dict
from joblib import Parallel, delayed
from ..utils.logging_utils import get_logger

logger = get_logger(__name__)


def run_parallel(
    func: Callable,
    items: List[Any],
    n_jobs: int = -1,
    backend: str = "multiprocessing",
    **kwargs
) -> List[Any]:
    """
    Run a function in parallel on a list of items.
    
    Args:
        func: Function to run in parallel
        items: List of items to process
        n_jobs: Number of jobs (-1 for all CPUs)
        backend: Backend to use ('multiprocessing' or 'threading')
        **kwargs: Additional arguments to pass to the function
        
    Returns:
        List of results
    """
    if n_jobs == -1:
        n_jobs = multiprocessing.cpu_count()
    
    logger.debug(f"Running {func.__name__} in parallel with {n_jobs} jobs")
    
    try:
        results = Parallel(n_jobs=n_jobs, backend=backend)(
            delayed(func)(item, **kwargs) for item in items
        )
        return results
    except Exception as e:
        logger.error(f"Error in parallel processing: {e}")
        raise


def run_parallel_with_progress(
    func: Callable,
    items: List[Any],
    n_jobs: int = -1,
    backend: str = "multiprocessing",
    **kwargs
) -> List[Any]:
    """
    Run a function in parallel with progress tracking.
    
    Args:
        func: Function to run in parallel
        items: List of items to process
        n_jobs: Number of jobs (-1 for all CPUs)
        backend: Backend to use ('multiprocessing' or 'threading')
        **kwargs: Additional arguments to pass to the function
        
    Returns:
        List of results
    """
    if n_jobs == -1:
        n_jobs = multiprocessing.cpu_count()
    
    logger.debug(f"Running {func.__name__} in parallel with progress tracking")
    
    try:
        results = Parallel(n_jobs=n_jobs, backend=backend, verbose=1)(
            delayed(func)(item, **kwargs) for item in items
        )
        return results
    except Exception as e:
        logger.error(f"Error in parallel processing: {e}")
        raise


def chunk_list(items: List[Any], chunk_size: int) -> List[List[Any]]:
    """
    Split a list into chunks of specified size.
    
    Args:
        items: List to chunk
        chunk_size: Size of each chunk
        
    Returns:
        List of chunks
    """
    return [items[i:i + chunk_size] for i in range(0, len(items), chunk_size)]


def get_optimal_n_jobs(total_items: int, max_jobs: int = None) -> int:
    """
    Calculate optimal number of jobs for parallel processing.
    
    Args:
        total_items: Total number of items to process
        max_jobs: Maximum number of jobs to use
        
    Returns:
        Optimal number of jobs
    """
    cpu_count = multiprocessing.cpu_count()
    
    if max_jobs:
        optimal = min(cpu_count, max_jobs, total_items)
    else:
        optimal = min(cpu_count, total_items)
    
    return max(1, optimal) 