"""
Random utility functions for persona-chat.
"""

import random
from typing import List, Any, Dict, Tuple
from ..utils.logging_utils import get_logger

logger = get_logger(__name__)


def weighted_choice(choices: List[Tuple[Any, float]]) -> Any:
    """
    Make a weighted random choice from a list of (item, weight) tuples.
    
    Args:
        choices: List of (item, weight) tuples
        
    Returns:
        Selected item
    """
    if not choices:
        raise ValueError("Choices list cannot be empty")
    
    items, weights = zip(*choices)
    return random.choices(items, weights=weights, k=1)[0]


def random_selection(items: List[Any], count: int = 1, allow_duplicates: bool = False) -> List[Any]:
    """
    Randomly select items from a list.
    
    Args:
        items: List of items to select from
        count: Number of items to select
        allow_duplicates: Whether to allow duplicate selections
        
    Returns:
        List of selected items
    """
    if not items:
        return []
    
    if allow_duplicates:
        return random.choices(items, k=count)
    else:
        return random.sample(items, min(count, len(items)))


def weighted_random_selection(
    items: List[Any], 
    weights: List[float], 
    count: int = 1, 
    allow_duplicates: bool = False
) -> List[Any]:
    """
    Make weighted random selections from a list.
    
    Args:
        items: List of items to select from
        weights: List of weights corresponding to items
        count: Number of items to select
        allow_duplicates: Whether to allow duplicate selections
        
    Returns:
        List of selected items
    """
    if not items or not weights:
        return []
    
    if len(items) != len(weights):
        raise ValueError("Items and weights lists must have the same length")
    
    if allow_duplicates:
        return random.choices(items, weights=weights, k=count)
    else:
        # For weighted sampling without replacement, we need to use a different approach
        # This is a simplified version
        selected = []
        remaining_items = list(items)
        remaining_weights = list(weights)
        
        for _ in range(min(count, len(items))):
            if not remaining_items:
                break
            
            choice = random.choices(remaining_items, weights=remaining_weights, k=1)[0]
            selected.append(choice)
            
            # Remove selected item
            index = remaining_items.index(choice)
            remaining_items.pop(index)
            remaining_weights.pop(index)
        
        return selected


def shuffle_list(items: List[Any]) -> List[Any]:
    """
    Shuffle a list and return a new list.
    
    Args:
        items: List to shuffle
        
    Returns:
        Shuffled list
    """
    shuffled = items.copy()
    random.shuffle(shuffled)
    return shuffled


def random_range(min_val: float, max_val: float) -> float:
    """
    Generate a random float in the given range.
    
    Args:
        min_val: Minimum value
        max_val: Maximum value
        
    Returns:
        Random float in range
    """
    return random.uniform(min_val, max_val)


def random_int_range(min_val: int, max_val: int) -> int:
    """
    Generate a random integer in the given range (inclusive).
    
    Args:
        min_val: Minimum value
        max_val: Maximum value
        
    Returns:
        Random integer in range
    """
    return random.randint(min_val, max_val)


def set_random_seed(seed: int) -> None:
    """
    Set the random seed for reproducible results.
    
    Args:
        seed: Random seed value
    """
    random.seed(seed)
    logger.debug(f"Set random seed to: {seed}")


def random_boolean(probability: float = 0.5) -> bool:
    """
    Generate a random boolean with given probability of True.
    
    Args:
        probability: Probability of returning True (0.0 to 1.0)
        
    Returns:
        Random boolean
    """
    return random.random() < probability 