"""
Text utility functions for persona-chat.
"""

import re
from typing import List, Dict, Any
from ..utils.logging_utils import get_logger

logger = get_logger(__name__)


def clean_text(text: str) -> str:
    """
    Clean and normalize text.
    
    Args:
        text: Text to clean
        
    Returns:
        Cleaned text
    """
    if not text:
        return ""
    
    # Remove extra whitespace
    text = re.sub(r'\s+', ' ', text.strip())
    
    # Remove special characters that might cause issues
    text = re.sub(r'[^\w\s\.,!?;:()\[\]{}"\'-]', '', text)
    
    return text


def extract_keywords(text: str, max_keywords: int = 10) -> List[str]:
    """
    Extract keywords from text.
    
    Args:
        text: Text to extract keywords from
        max_keywords: Maximum number of keywords to extract
        
    Returns:
        List of keywords
    """
    if not text:
        return []
    
    # Simple keyword extraction - in a real system, this would use NLP
    words = re.findall(r'\b\w+\b', text.lower())
    
    # Filter out common stop words
    stop_words = {
        'the', 'a', 'an', 'and', 'or', 'but', 'in', 'on', 'at', 'to', 'for', 'of', 'with', 'by',
        'is', 'are', 'was', 'were', 'be', 'been', 'have', 'has', 'had', 'do', 'does', 'did',
        'will', 'would', 'could', 'should', 'may', 'might', 'can', 'this', 'that', 'these', 'those',
        'i', 'you', 'he', 'she', 'it', 'we', 'they', 'me', 'him', 'her', 'us', 'them'
    }
    
    keywords = [word for word in words if word not in stop_words and len(word) > 3]
    
    # Count frequency and return most common
    from collections import Counter
    keyword_counts = Counter(keywords)
    
    return [word for word, count in keyword_counts.most_common(max_keywords)]


def count_words(text: str) -> int:
    """
    Count words in text.
    
    Args:
        text: Text to count words in
        
    Returns:
        Word count
    """
    if not text:
        return 0
    
    words = re.findall(r'\b\w+\b', text)
    return len(words)


def count_sentences(text: str) -> int:
    """
    Count sentences in text.
    
    Args:
        text: Text to count sentences in
        
    Returns:
        Sentence count
    """
    if not text:
        return 0
    
    sentences = re.split(r'[.!?]+', text)
    return len([s for s in sentences if s.strip()])


def calculate_readability_score(text: str) -> float:
    """
    Calculate a simple readability score.
    
    Args:
        text: Text to analyze
        
    Returns:
        Readability score (higher = more readable)
    """
    if not text:
        return 0.0
    
    word_count = count_words(text)
    sentence_count = count_sentences(text)
    
    if sentence_count == 0:
        return 0.0
    
    # Simple Flesch Reading Ease approximation
    avg_sentence_length = word_count / sentence_count
    readability = 206.835 - (1.015 * avg_sentence_length)
    
    return max(0.0, min(100.0, readability))


def extract_entities(text: str) -> List[str]:
    """
    Extract named entities from text.
    
    Args:
        text: Text to extract entities from
        
    Returns:
        List of potential entities
    """
    if not text:
        return []
    
    # Simple entity extraction - in a real system, this would use NER
    # Look for capitalized words that might be entities
    entities = re.findall(r'\b[A-Z][a-z]+(?:\s+[A-Z][a-z]+)*\b', text)
    
    # Filter out common words that are capitalized
    common_caps = {'I', 'The', 'This', 'That', 'These', 'Those', 'A', 'An'}
    entities = [entity for entity in entities if entity not in common_caps]
    
    return list(set(entities))


def detect_language(text: str) -> str:
    """
    Detect the language of text.
    
    Args:
        text: Text to detect language for
        
    Returns:
        Language code (default: 'en')
    """
    if not text:
        return 'en'
    
    # Simple language detection - in a real system, this would use langdetect
    # For now, just return English as default
    return 'en'


def normalize_text(text: str) -> str:
    """
    Normalize text for consistent processing.
    
    Args:
        text: Text to normalize
        
    Returns:
        Normalized text
    """
    if not text:
        return ""
    
    # Convert to lowercase
    text = text.lower()
    
    # Remove extra whitespace
    text = re.sub(r'\s+', ' ', text.strip())
    
    # Remove punctuation (optional)
    # text = re.sub(r'[^\w\s]', '', text)
    
    return text 