"""
Validation utility functions for persona-chat.
"""

from typing import Dict, Any, List, Optional
from ..models.persona import Persona
from ..models.conversation import Conversation
from ..utils.logging_utils import get_logger

logger = get_logger(__name__)


def validate_persona(persona_data: Dict[str, Any]) -> Dict[str, Any]:
    """
    Validate persona data.
    
    Args:
        persona_data: Persona data to validate
        
    Returns:
        Validation results
    """
    results = {
        "is_valid": True,
        "errors": [],
        "warnings": []
    }
    
    # Check required fields
    required_fields = ["name", "role", "domain", "expertise_level", "tone"]
    for field in required_fields:
        if field not in persona_data:
            results["is_valid"] = False
            results["errors"].append(f"Missing required field: {field}")
    
    # Check domain validity
    if "domain" in persona_data:
        valid_domains = ["technology", "healthcare", "education", "business", "creative", "specialized"]
        if persona_data["domain"] not in valid_domains:
            results["is_valid"] = False
            results["errors"].append(f"Invalid domain: {persona_data['domain']}")
    
    # Check expertise level validity
    if "expertise_level" in persona_data:
        valid_levels = ["novice", "intermediate", "advanced", "expert"]
        if persona_data["expertise_level"] not in valid_levels:
            results["is_valid"] = False
            results["errors"].append(f"Invalid expertise level: {persona_data['expertise_level']}")
    
    # Check formality level
    if "formality_level" in persona_data:
        try:
            level = int(persona_data["formality_level"])
            if not (1 <= level <= 5):
                results["is_valid"] = False
                results["errors"].append("Formality level must be between 1 and 5")
        except (ValueError, TypeError):
            results["is_valid"] = False
            results["errors"].append("Formality level must be an integer")
    
    # Check experience level
    if "experience_level" in persona_data:
        try:
            level = int(persona_data["experience_level"])
            if not (0 <= level <= 50):
                results["warnings"].append("Experience level should be between 0 and 50 years")
        except (ValueError, TypeError):
            results["is_valid"] = False
            results["errors"].append("Experience level must be an integer")
    
    return results


def validate_conversation(conversation_data: Dict[str, Any]) -> Dict[str, Any]:
    """
    Validate conversation data.
    
    Args:
        conversation_data: Conversation data to validate
        
    Returns:
        Validation results
    """
    results = {
        "is_valid": True,
        "errors": [],
        "warnings": []
    }
    
    # Check required fields
    required_fields = ["id", "turns"]
    for field in required_fields:
        if field not in conversation_data:
            results["is_valid"] = False
            results["errors"].append(f"Missing required field: {field}")
    
    # Check turns structure
    if "turns" in conversation_data:
        turns = conversation_data["turns"]
        if not isinstance(turns, list):
            results["is_valid"] = False
            results["errors"].append("Turns must be a list")
        else:
            for i, turn in enumerate(turns):
                turn_validation = validate_turn(turn)
                if not turn_validation["is_valid"]:
                    results["is_valid"] = False
                    results["errors"].append(f"Turn {i+1}: {turn_validation['errors']}")
    
    return results


def validate_turn(turn_data: Dict[str, Any]) -> Dict[str, Any]:
    """
    Validate turn data.
    
    Args:
        turn_data: Turn data to validate
        
    Returns:
        Validation results
    """
    results = {
        "is_valid": True,
        "errors": [],
        "warnings": []
    }
    
    # Check required fields
    required_fields = ["turn_number", "user_message"]
    for field in required_fields:
        if field not in turn_data:
            results["is_valid"] = False
            results["errors"].append(f"Missing required field: {field}")
    
    # Check user message
    if "user_message" in turn_data:
        message_validation = validate_message(turn_data["user_message"])
        if not message_validation["is_valid"]:
            results["is_valid"] = False
            results["errors"].extend(message_validation["errors"])
    
    # Check assistant message (optional)
    if "assistant_message" in turn_data and turn_data["assistant_message"]:
        message_validation = validate_message(turn_data["assistant_message"])
        if not message_validation["is_valid"]:
            results["is_valid"] = False
            results["errors"].extend(message_validation["errors"])
    
    return results


def validate_message(message_data: Dict[str, Any]) -> Dict[str, Any]:
    """
    Validate message data.
    
    Args:
        message_data: Message data to validate
        
    Returns:
        Validation results
    """
    results = {
        "is_valid": True,
        "errors": [],
        "warnings": []
    }
    
    # Check required fields
    required_fields = ["role", "content"]
    for field in required_fields:
        if field not in message_data:
            results["is_valid"] = False
            results["errors"].append(f"Missing required field: {field}")
    
    # Check role validity
    if "role" in message_data:
        valid_roles = ["user", "assistant"]
        if message_data["role"] not in valid_roles:
            results["is_valid"] = False
            results["errors"].append(f"Invalid role: {message_data['role']}")
    
    # Check content
    if "content" in message_data:
        content = message_data["content"]
        if not isinstance(content, str):
            results["is_valid"] = False
            results["errors"].append("Content must be a string")
        elif not content.strip():
            results["warnings"].append("Content is empty")
    
    return results


def validate_language_code(language_code: str) -> bool:
    """
    Validate language code.
    
    Args:
        language_code: Language code to validate
        
    Returns:
        True if valid, False otherwise
    """
    valid_languages = ["en", "es", "fr", "de", "it", "pt", "ja", "zh", "ko", "ar"]
    return language_code in valid_languages


def validate_domain(domain: str) -> bool:
    """
    Validate domain name.
    
    Args:
        domain: Domain to validate
        
    Returns:
        True if valid, False otherwise
    """
    valid_domains = ["technology", "healthcare", "education", "business", "creative", "specialized"]
    return domain in valid_domains


def validate_expertise_level(level: str) -> bool:
    """
    Validate expertise level.
    
    Args:
        level: Expertise level to validate
        
    Returns:
        True if valid, False otherwise
    """
    valid_levels = ["novice", "intermediate", "advanced", "expert"]
    return level in valid_levels 